open Odoc_info
open Parameter
open Value
open Type
open Exception
open Class
open Module
open Odoc_html
open Doc_outils

class gen () =
  object (self)
    inherit Odoc_html.html as html
	method html_of_example t = "<b>Example</b> :"^(self#html_of_text t)^"<br>\n"		
	method html_of_voir t = 
			let lexbuf = Lexing.from_string (self#html_of_text t) in
			let result = (*Doc_lien_yacc.depart*)  Doc_lien_lex.lexer lexbuf in
			match result with 
			Module (n,c) -> "<b>See</b> :<a href="^(n)^".html >"^(c)^"</a><br>\n"
			|ModFonct (n,f,c)-> let ff = Doc_outils.code f in
					"<b>See</b> :  <a href=\""^(n)^".html#VAL"^ff^"\" >"^(c)^"</a><br>\n"
			|Fonction (f,c) -> let ff = Doc_outils.code f in
					"<b>See</b> : <a href=\"#VAL"^ff^"\" >"^(c)^"</a><br>\n"
			|URL (u,c) -> "<b>See</b> : <a href="^(u)^" >"^(c)^"</a><br>\n"


		
	method generate module_list = html#generate module_list    

	
    (** A method to create index files. *)
    method generate_elements_index :
        'a.
        'a list ->
          ('a -> Odoc_info.Name.t) ->
            ('a -> Odoc_info.info option) ->
              ('a -> string) -> string -> string -> unit =
    fun elements name info target title simple_file ->
      try
        let chanout = open_out (Filename.concat !Args.target_dir simple_file) in
        output_string chanout
          (
           "<html>\n"^
           (self#header (self#inner_title title)) ^
           "<body>\n"^
            "<center><h1>"^title^"</h1></center>\n");

        let sorted_elements = List.sort
            (fun e1 -> fun e2 -> compare (Name.simple (name e1)) (Name.simple (name e2)))
            elements
        in
        let groups = Odoc_info.create_index_lists sorted_elements (fun e -> Name.simple (name e)) in
        let f_ele e =
          let simple_name = Name.simple (name e) in
          let father_name = Name.father (name e) in
	let nom = transforme_nom (simple_name) in
		 output_string chanout
            ("<tr><td><a href=\""^(target e)^"\">"^(nom)^"</a> "^
             (if simple_name <> father_name then
               "["^"<a href=\""^(fst (Naming.html_files father_name))^"\">"^father_name^"</a>]"
             else
               ""
             )^
             "</td>\n"^
             "<td>"^(self#html_of_info_first_sentence (info e))^"</td></tr>\n"
            )
        in
        let f_group l =
          match l with
            [] -> ()
          | e :: _ ->
              let s =
                match (Char.uppercase (Name.simple (name e)).[0]) with
                  'A'..'Z' as c -> String.make 1 c
                | _ -> ""
              in
              output_string chanout ("<tr><td align=\"left\"><br>"^s^"</td></tr>\n");
              List.iter f_ele l
        in
        output_string chanout "<table>\n";
        List.iter f_group groups ;
        output_string chanout "</table><br>\n" ;
        output_string chanout "</body>\n</html>";
        close_out chanout
      with
        Sys_error s ->
          raise (Failure s)



    (** Generate a file containing the module type in the given file name. *)
    method output_module_type in_title file mtyp =
      let s = suppression_type(transforme_nom (String.concat "\n"
          (Str.split (Str.regexp "\n") (Odoc_info.string_of_module_type ~complete: true mtyp))
      ))
	in
      self#output_code in_title file s




    	method html_of_value v =
		let nom = transforme_nom (v.val_name) in
      		Odoc_info.reset_type_names ();
      		(*"<pre>"^(self#keyword "val")^" "^*)
      		"<pre> "^
      		(* html mark *)
      		"<a name=\""^( (Naming.value_target v))^"\"></a>"^(self#keyword
      		(match v.val_code with
        		None -> Name.simple nom
        		(*None -> Name.simple v.val_name*)
      			| Some c ->
          			let file = Naming.file_code_value_complete_target v in
          				self#output_code nom (Filename.concat !Args.target_dir file) c;
          				"<a href=\""^file^"\">"^(Name.simple nom)^"</a>"
      		))^" : "^
      					(self#html_of_type_expr (Name.father nom) v.val_type)^"</pre>"^
      					(self#html_of_info v.val_info)^
      					(if !Args.with_parameter_list then
       					 self#html_of_parameter_list (Name.father nom) v.val_parameters
      					else
        				self#html_of_described_parameter_list (Name.father nom) v.val_parameters
      					)

			(*| Some c ->
          			let file = Naming.file_code_value_complete_target v in
          				self#output_code v.val_name (Filename.concat !Args.target_dir file) c;
          				"<a href=\""^file^"\">"^(Name.simple v.val_name)^"</a>"
      				)^" : "^
      					(self#html_of_type_expr (Name.father v.val_name) v.val_type)^"</pre>"^
      					(self#html_of_info v.val_info)^
      					(if !Args.with_parameter_list then
       					 self#html_of_parameter_list (Name.father v.val_name) v.val_parameters
      					else
        				self#html_of_described_parameter_list (Name.father v.val_name) v.val_parameters
      					)
*)


    (** Return html code to display a [Types.type_expr].*)
    method html_of_type_expr m_name t =
      let s = String.concat "\n"
          (Str.split (Str.regexp "\n") (Odoc_info.string_of_type_expr t))
      in
      let s2 = Str.global_replace (Str.regexp "\n") "<br>       " s in
      "<code class=\"type\">"^(suppression_type(self#create_fully_qualified_idents_links m_name s2))^"</code>"


    (** Return html code for the first sentence of a description.
       The titles and lists in this first sentence has been removed.*)
    method html_of_info_first_sentence info_opt =

      let debut x =
	let rec get_before_dot s =
	  try
	    let len = String.length s in
	    let n = String.index s '.' in
	    if n + 1 >= len then
	      (* le point est le dernier caractre *)
	      (true, s, "")
	    else
	      match s.[n+1] with
		  ' ' | '\n' | '\r' | '\t' ->
		    (true, String.sub s 0 (n+1),
		     String.sub s (n+1) (len - n - 1))
		| _ ->
		    let b, s2, s_after = get_before_dot (String.sub s (n + 1) (len - n - 1)) in
		      (b, (String.sub s 0 (n+1))^s2, s_after)
	      with
		  Not_found -> (false, s, "")
	in

	let rec first_sentence_text_ele text_ele bb=
	  match text_ele with
	    | Odoc_types.Raw s ->
		let b, s2, s_after = get_before_dot s in
		  (b, Odoc_types.Raw s2, Some (Odoc_types.Raw s_after),(if(s.[0]='\n'&& (String.length s)=1) then false else true))
	    | Odoc_types.Code x
	    | Odoc_types.CodePre x
	    | Odoc_types.Verbatim x ->if(bb) then (false,text_ele , None,true) else (false,text_ele , None,false)
	    | Odoc_types.Bold t ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b, Odoc_types.Bold t2, Some (Odoc_types.Bold t3),true)
	    | Odoc_types.Italic t ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b, Odoc_types.Italic t2, Some (Odoc_types.Italic t3),true)
	    | Odoc_types.Center t ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b, Odoc_types.Center t2, Some (Odoc_types.Center t3),true)
	    | Odoc_types.Left t ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b, Odoc_types.Left t2, Some (Odoc_types.Left t3),true)
	    | Odoc_types.Right t ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b, Odoc_types.Right t2, Some (Odoc_types.Right t3),true)
	    | Odoc_types.Emphasize t ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b, Odoc_types.Emphasize t2, Some (Odoc_types.Emphasize t3),true)
	    | Odoc_types.Block t ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b, Odoc_types.Block t2, Some (Odoc_types.Block t3),true)
	    | Odoc_types.Title (n, l_opt, t) ->
		let (b, t2, t3) = first_sentence_text t true in
		  (b,
		   Odoc_types.Title (n, l_opt, t2),
		   Some (Odoc_types.Title (n, l_opt, t3)),true)
	    | Odoc_types.Newline ->
		if(bb) then
		(true, Odoc_types.Raw "", Some Odoc_types.Newline,true)
		else (false, Odoc_types.Raw "", Some Odoc_types.Newline,false)
	    | Odoc_types.List _
	    | Odoc_types.Enum _
	    | Odoc_types.Latex _
	    | Odoc_types.Link _
	    | Odoc_types.Ref _
	    | Odoc_types.Superscript _
	    | Odoc_types.Subscript _ ->(false, text_ele, None,true)
	and    
	 first_sentence_text t b=
	  match t with
	      [] -> (false, [], [])
	    | ele :: q ->
		let (stop, ele2, ele3_opt,garde) = first_sentence_text_ele ele b in
		  if stop then
		    if(garde) then
		    (stop, [ele2],
		     match ele3_opt with None -> q | Some e -> e :: q)
		    else
		      (false, [],match ele3_opt with None -> q | Some e -> e :: q)
		  else
		    if(garde) then
		      let (stop2, q2, rest) = first_sentence_text q true in
		      (stop2, ele2 :: q2, rest)
		    else
		    let (stop2, q2, rest) = first_sentence_text q false in
		      (false, q2, rest)
	in
	let (_,t2,_) = first_sentence_text x false in
	  t2
      in


      match info_opt with
        None -> ""
      | Some info ->
          let module M = Odoc_info in
          let dep = info.M.i_deprecated <> None in
          "<div class=\"info\">\n"^
          (if dep then "<font color=\"#CCCCCC\">" else "") ^
          (match info.M.i_desc with
            None -> ""
          | Some d when d = [Odoc_info.Raw ""] -> ""
          | Some d -> (self#html_of_text
                         (Odoc_info.text_no_title_no_list
                            (debut d)))^"\n"
          )^
          (if dep then "</font>" else "") ^
          "</div>\n"



	
    initializer
      tag_functions <- ("example", self#html_of_example) :: tag_functions; 
      tag_functions <- ("voir", self#html_of_voir) :: tag_functions; 
  end

let my_generator = new gen () 
let _ = Odoc_args.set_doc_generator (Some (my_generator :> Odoc_args.doc_generator))

