open StdLabels

(** Type pour les fonction et leur type *)
type type_fonction = { mutable nom : string ;  (** Nom de la fonction *)
		       mutable typ : string list list ;  (** liste des types des  arguments de sfonctions *)
		     }

(** Commentaire pour les fonctions *)
type type_commentaire = { mutable name : string ;  (** Nom de la fonction *)
		       mutable commentaire : string  ;  (** commentaire *)
		       mutable types :  string list list ;  (** types *)
		       mutable modules : string  ;  (** modules *)
		       mutable typecom : int  ;  (** typescom 0-> fonction 1->module *)
		       mutable recherche : string ; (** nom de la focntion pour rechercher son type *)
		     }


(** Commentaire pour les module *)
type type_module  = { mutable modul : string ;  (** Nom du module *)
		       mutable com : string  ;  (** commentaire *)
		       mutable contenu : type_commentaire list (** Ffonction dans le module *)
		     }




(** Fonction qui a partir d'un descripteur de fichier [descr_fichier] lit tous les caracteres possibles et renvoie la chaine de caracteres *)
let lire descr_fichier = 
  let text = ref "" in
  let rec iter_lecture sortie =
    try
      let buffer_len = 10000 in(* Il faut un grand buffer !!!!*)
      let chaine2 = String.create buffer_len in
	let nblus2 = Unix.read sortie chaine2 0 buffer_len in
          text:=!text^(String.sub chaine2 ~pos:0 ~len:nblus2);
          if nblus2>0
          then
            iter_lecture sortie ;
          () with Unix.Unix_error(x,_,_)-> Printf.printf "Probleme de lecture";
  in
    iter_lecture  descr_fichier;
    !text;;



(** Fonction qui a partir d'un descripteur de fichier [descr_fichier] lit tous les caracteres possibles et renvoie la chaine de caracteres *)
let lire2 descr_fichier = 
  let buff = Buffer.create 10000 and
    canal = Unix.in_channel_of_descr descr_fichier in
  let rec iter_lecture sortie =
    try
      Buffer.add_channel buff sortie 10000;
      iter_lecture sortie;

          with Unix.Unix_error(x,_,_)-> Printf.printf "Probleme de lecture";
	    |End_of_file -> ();
  in
    iter_lecture canal;
    Buffer.contents buff;;


(** Fonction qui enleve tous les espaces qui sont en prefixe d'une chaine de caracteres [chaine] et la retourne *) 
let rec enleve_espace chaine =
  if((compare chaine "")<>0)
  then
    begin
    match (String.sub chaine  ~pos:0 ~len:1) with
	" "->enleve_espace (String.sub chaine ~pos:1  ~len:(String.length chaine))
      |_-> chaine
    end
  else
    chaine;;
 


(** Fonction qui execute le programme [exec] ( a rechercher dans le $PATH ou a partir de son chemin ) avec les arguments [arg] ayant pour stdin [pipe_in_lecture], pour stdout [pipe_out_ecriture], pour stderr [pipe_err_ecriture]. *)   
let lance exec arg  pipe_in_lecture pipe_out_ecriture pipe_err_ecriture =
  Unix.dup2 pipe_in_lecture Unix.stdin;
  Unix.dup2 pipe_out_ecriture Unix.stdout;
  Unix.dup2 pipe_out_ecriture Unix.stderr;
  try
    Unix.execvp exec arg;
  with _ ->
    begin
      try
	Unix.execv exec arg; 
      with _-> prerr_endline "Pas d'executable trouve";
    end;
  while(true) do () done;
  exit 0;;

 
(** Fonction qui lance le programme [exec] ( a rechercher dans le $PATH ou a partir de son chemin ) avec les arguments [arg] ayant pour stdin [pipe_in_lecture], pour stdout [pipe_out_ecriture], pour stderr [pipe_err_ecriture] en parallele du programme actuel en "forkant". Le pere recoit le PID  de son fils, et le fils recoit 0 *)
let lancer exec arg  pipe_in_lecture pipe_out_ecriture pipe_err_ecriture =
match Unix.fork() with 
  0 -> lance exec arg  pipe_in_lecture pipe_out_ecriture pipe_err_ecriture;0;
  |pid -> pid ;;

(** Fonction qui execute le programme [exec] ( a rechercher dans le $PATH ou a partir de son chemin ) avec les arguments [arg]  *) 
let nouveau_proc exec arg =
  try
    Unix.execvp exec arg;
  with _ ->
    begin
      try
	Unix.execv exec arg; 
      with _-> prerr_endline "Pas d'executable trouve";
    end;
  while(true) do () done;
  exit 0;;

(** Fonction qui lance le programme [exec] ( a rechercher dans le $PATH ou a partir de son chemin ) avec les arguments [arg] en parallele du programme actuel en "forkant". Le pere recoit le PID  de son fils, et le fils recoit 0 *)
let nouveau_process exec arg =
match Unix.fork() with
    0 -> nouveau_proc exec arg;0;
  |pid -> pid;;


(** Fonction qui  partir d'une chaine de caractre [chaine], de sa taille [taille_chaine], d'un indice [indice] a partir duquel la fonction est execute et d'une liste de chaine de caractere, renvoie une liste de chaine de caractere spar par des espaces *)
let rec explode chaine taille_chaine indice acc =
	if(indice=taille_chaine) then acc
	else
	match (String.get chaine indice) with
		' '-> explode chaine taille_chaine (indice+1) (""::acc)
		|x-> explode chaine taille_chaine (indice+1) (((List.hd(acc)^(Char.escaped x)))::List.tl(acc))



