(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: bag.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2008-05-21 13:30:27 $
 * Version: $Revision: 1.11 $
 * -------------------------------------------------------------------
 *
 * Status:  
 *    Implantation au niveau "representation MGS" des fonctions sur les 
 *    bag. Voir commentaire dans Seq/seq.ml
 *
 *    Dans un rpertoire Xxx correspondant  une collection xxx, 
 *    le type xxx est defini dans def_xxx.ml et les operations sur 
 *    ce type sont dfinies dans core_xxx.ml. Le fichier kernel_xxx.ml
 *    contient la declaration dans la librairie MGS des fcts dfinies 
 *    dans xxx.ml
 *
 *    L'organisation de xxx.ml fichier suit l'ordre suivant:
 *
 *       1/ Fonctions de construction: ... -> seq (sauf extension)
 *             cons, join, append, shape, xxxify, iota, extension
 *
 *       2/ Prdicats, accs et observateur
 *             compare, val2bool, size, empty, count, 
 *             member, random, hd, tl, take
 *
 *       3/ Fold et autre itrations
 *             fold, map, iter, foldindexed, mapindexed, iterindexed,
 *             forall, exists, filter
 *
 *       4/ Neighbors, neighbor
 *
 *
 *       5/ Fonctions spcifiques
 *
 *
 *       6/ mise  jour du descripteur du type
 *              champs: make, xxxify et empty
 *
 * Ici :   xxx = seq
 *
 *=========================================================================
 *)


open Global;;
open Def_bag;;
open Valeur;;
open Core_bag;;
open Type;;
open Dispatch_sig;;



__register_version "$RCSfile: bag.ml,v $" "$Revision: 1.11 $" "$Date: 2008-05-21 13:30:27 $";;

(** alias **)
let bag_desc = Type_bag.bag_desc
;;



(** === Constructeurs ======================================================- **)


let _cons_any_bag (e, s) = VAL_bag(Core_bag.add_b e   (fst s), snd s)

and _cons_undef_bag (_, s) = VAL_bag s

and _append_bag_bag (s1, s2) = VAL_bag(union_b (fst s1) (fst s2), commontype (snd s1) (snd s2))

and _join_bag_bag (s1, s2) = VAL_bag(union_b (fst s1) (fst s2), commontype (snd s1) (snd s2))

and _join_any_bag (e, s) = if true_coll e then Seq._join_any_any (e, VAL_bag(s)) else VAL_bag(add_b e (fst s), snd s)
			       
and _join_bag_any (s, e) = if true_coll e then Seq._join_any_any (VAL_bag(s), e) else VAL_bag(add_b e (fst s), snd s)

and _join_bag_undef (s, _) = VAL_bag s

and _join_undef_bag (_, s) = VAL_bag s
;;


let  _shape_bag (b, deco) = 
  let taille_b = Core_bag.cardinal_b b
  in VAL_funct 
       (function 
	  | VAL_seq (s, _) -> 
	      if (taille_b <> Array.length s)
	      then raise_error "shape (bag) : argument is not of the same size of the original"
	      else VAL_bag ((Array.fold_right (fun x y -> Core_bag.add_b x y) s Core_bag.empty_b), deco)
	  | _  -> raise_error "shape (bag) : a sequence as argument is required")
;;

let _sequify_bag s = VAL_seq(Array.of_list (elements_b (fst s)), Type_seq.seq_desc) ;;

let _bagify_bag s = VAL_bag(fst s, bag_desc)
and _bagify_seq s = VAL_bag(Array.fold_left (fun ss x -> add_b x ss) empty_b (fst s), bag_desc)
and _bagify_any x = 
  if not(true_coll x) then VAL_bag(singleton_b(x), bag_desc)
  else 
    Dispatch_sig.fold 
      (VAL_funct(fun x -> VAL_funct(function
				      | VAL_bag(y, t) -> VAL_bag((Core_bag.add_b x y), t)
				      | _ -> failwith "internal error: bagify_any")))
      bag_desc.empty 
      x
;;


let _iota_int_bag (n, (v, col)) = 
  if ( n < 0)
  then raise (Erreur "iota_bag with a negative argument")
  else (let res = ref v
	in
	  for i = 0 to (n-1)
	  do
	    res := Core_bag.add_b (VAL_int i) !res
	  done;
	  VAL_bag (!res, col))
;;

let _inter_bag_bag ((s1,t1),(s2,t2)) =
  let t = Type.commontype t1 t2 in
    VAL_bag(Core_bag.inter_b s1 s2, t)
;;

let rec _diff_bag_bag ((s1,t1),(s2,t2)) =
  let t = Type.commontype t1 t2 in
    VAL_bag(Core_bag.diff_b s1 s2, t)

and _diff_bag_any ((s1,t1),v) =
  _diff_bag_bag ((s1,t1),(to_bag(_join_bag_any((to_bag(t1.empty),t1),v)),t1))
(*,(_join_bag_any (t1.empty,v)))*)

and _mult_int_bag (i,(b,t)) =
  if (i=0)
  then t.empty
  else
    if (i < 0)
    then raise (Erreur "Bag: _mult_int_bag: negative coefficient for multiplication")
    else VAL_bag(Core_bag.multn_b i b, t)
;;


(** Extension --------------------------------- **)


let rec extension_bag desc a = 
  VAL_bag(Array.fold_left add_elem Core_bag.empty_b a, desc)

and add_elem bag = function
  | VAL_seq(aa, _) -> Core_bag.add_b aa.(1) bag
  | _ -> raise (Erreur "Internal Error: extension: add_elem in extension_bag")
;;



(** === Prdicats, accs et observateurs =========================================== **)

(** _compareMgsVal_bag_bag est dfini dans core_bag *)

let _val2bool_bag (b, _) = VAL_bool(not(Core_bag.is_empty_b b))
;;


let _size_bag (b, _) = VAL_int(cardinal_b b)
and _empty_bag (b, _) = VAL_bool(b = Core_bag.empty_b)
;;



let _count_any_bag (a, (b, _)) = VAL_int (Core_bag.num_b a b)
;;

let _member_any_bag (a, (b, _)) = VAL_bool (Core_bag.mem_b a b) 



exception Ok of mgs_value;;

let _random_bag(s, _) = 
  let n = Core_bag.cardinal_b s in
    if n == 0 then VAL_undef "random: bad argument: empty bag" 
    else 
      let nn = ref(Random.int n) in
	try
	  Core_bag.iter_b (function x -> if !nn == 0 then raise (Ok x) else decr(nn)) s;
	  assert(false);
	  VAL_undef("internal error in _random_bag")
	with Ok x -> x
;;



let _hd_bag s = 
  let ss = fst s in if (0 = cardinal_b ss) then VAL_undef("hd on empty bag") else choose_b ss

and _tl_bag s = 
  let ss = fst s 
  in 
    if (0 = cardinal_b ss)
    then VAL_undef("tl on empty bag") 
    else VAL_bag(remove_b (choose_b ss) ss, snd s)
;;

(** take_bag_int est pris en charge par le _take_any_int dfini dans Seq/seq.ml **)



(* === Fold et autre itrations ================================ *)

let _fold_funct_any_bag (fonc, zero, (bag, _)) = 
  Core_bag.fold_b  (fun v suite -> (app_2 fonc v suite)) bag zero

and _map_funct_bag (fonc, (bag, c)) = 
  VAL_bag ((Core_bag.fold_b (fun v suite -> Core_bag.add_b (fonc v) suite) bag Core_bag.empty_b), c)

and _iter_funct_bag (fonc, (bag, _)) = 
  (Core_bag.iter_b (fun x   -> fonc x; ()) bag);  VAL_undef "iter bag"

and _mapindexed_funct_bag (fct, (bag, t)) = 
  let elts = Core_bag.elements_b bag
  and cpt = ref (-1)
  in VAL_bag((List.fold_left
		(fun acc v -> incr(cpt); Core_bag.add_b (app_2 fct (VAL_int !cpt) v) acc) 
		Core_bag.empty_b 
		elts),
	     t)

and _iterindexed_funct_bag (fonc, (bag, _)) = 
  let idx = ref 0 in 
    Core_bag.iter_b (fun x -> ignore(app_2 fonc (VAL_int !idx) x); incr idx) bag;  
    VAL_undef "iterindexed bag"

and _foldindexed_funct_any_bag (fct, zero, (bag, _)) = 
  let cpt = ref (-1)
  in Core_bag.fold_b (fun v acc -> incr(cpt); app_3 fct (VAL_int !cpt) v acc) bag zero

and _forall_funct_bag (p, (b, _)) = VAL_bool(for_all_b (fun x -> Auxfct.bool_from_val(p x)) b)

and _exists_funct_bag (p, (b, _)) = VAL_bool(exists_b (fun x -> Auxfct.bool_from_val(p x)) b)

and _filter_funct_bag (p, (b, t)) = VAL_bag(filter_b (fun x -> Auxfct.bool_from_val(p x)) b, t)
;;





(* === Neighbors, neighbor ================================ *)

let _neighbors_bag_int ((t, _), i) = 
  let sz         = Core_bag.cardinal_b t
  and retourne v = VAL_seq (v, Type_seq.seq_desc)
  in
    if (i < 0) or (i >= sz)
    then VAL_undef "neighbors (bag): undefined neighbor"
    else
      let ret = Array.make (sz-1) (VAL_undef "")
      and idx = ref 0
      and ptr = ref 0
      in
	Core_bag.iter_b (fun v -> if ((post_inc idx) <> i) then (ret.(post_inc ptr) <- v)) t;
	retourne ret
;;





(* === Fonctions spcifiques ======================================================- *)


(* Flatten ----------------------------------------------------------- *)


let _flatten_bag (s, t) =
  let ss = Core_bag.fold_b
	     (fun v suite -> match v with 
		| VAL_bag (a, _) -> Core_bag.union_b a suite
		| x -> Core_bag.add_b x suite)
	     s 
	     Core_bag.empty_b
  in VAL_bag(ss, t)
;;

let _subset_bag_bag ((a,_),(b,_)) = VAL_bool(Core_bag.subset_b a b) ;;

(** Une fonction partition sur les monoides ----------------------------------- *)



let rec _partition_int_bag (n, (s, t)) =
  let ls = split_bag n s in
  let ss = List.fold_left (fun acc x -> Core_bag.add_b (VAL_bag(x, t)) acc) Core_bag.empty ls
  in VAL_bag(ss, t)

(* On utilise le predicat de Bag.partition_b en faisant un effet de
 * bord. En esperant qu'on appele ce predicat qu'une seule fois pour
 * chaque element, ce qui permet de correler le nombre de fois ou il a
 * ete appele et le nombre d'lment selectionn 
 *)

and split_bag n s = 
  if n >= Core_bag.cardinal_b s then [s] else
    begin
      let i = ref 0 in 
      let fct x = (incr i; !i <= n) in
      let (x, suite) = Core_bag.partition_b fct s 
      in x::(split_bag n suite)
    end
;;




(** Partition en 2 classes suivant un predicat ----------------------------------------------- *)


let   _partition_funct_bag (f, (s, t)) = 
  let (s1, s2) = Core_bag.partition_b (function x -> Auxfct.bool_from_val (f x)) s
  in VAL_bag(Core_bag.add_b (VAL_bag(s1, t)) (Core_bag.singleton_b (VAL_bag(s2, t))), 
	     t)
;;



(** Une fonction partition par quivalence sur les monoide ----------------------------------- *)

let rec _partitioneq_funct_bag (f, (s, t)) =
  let eql = ref [] 
  and fill acc (_, lx) = Core_bag.add_b (VAL_bag(make_bag_from_list !lx, t)) acc
  in 
    begin
      Core_bag.iter_b (function e -> update2 eql f e) s;
      VAL_bag(List.fold_left fill Core_bag.empty_b !eql, t)
    end
    
and make_bag_from_list l = 
  List.fold_left (fun acc x -> Core_bag.add_b x acc) Core_bag.empty_b l

and update2 eql f x = eql := Auxfct.update_equivalence f x !eql
;;



(* ================================================================ *)

(*** !!! Ne pas oublier de mettre  jour les champs
 ***     _ make 
 ***     _ xxxify
 ***     _ empty
 *** du descripteur de type 
 ***)

bag_desc.empty <- VAL_bag(Core_bag.empty_b, bag_desc);;
bag_desc.xxxify <- VAL_funct(bagify);;
bag_desc.make <- extension_bag bag_desc;;
