(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: auxfct.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2007-06-27 12:47:47 $
 * Version: $Revision: 1.5 $
 * -------------------------------------------------------------------
 *
 * Status:  
 *    Fonctions auxilliaires qui necessitent Dispatch_sig mais qui sont 
 *    utilises un peu partout dans les fichiers SOURCES_2
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Dispatch_sig;;
open Big_int;;

let bool_from_val x = match val2bool x with
  | VAL_bool x  -> x
  | _ -> 
      let err = "Error: internal: bool_from_val: got a non boolean value !!!"
      in Printf.fprintf stderr "%s\ngot:\t%s\n\tcompute:\t%s\n" 
	   err 
	   (!forward_prsvalue x) 
	   (!forward_prsvalue (val2bool x));
	failwith err
;;

let not_val x = match val2bool x with
  | VAL_bool true  -> mgsfalse
  | _ -> mgstrue
;;

let _test_bool_'bool(x, y) = if x = y then VAL_bool(not x) else (VAL_bool y)
;;

let assertion msg x = match val2bool x with
  | VAL_bool true -> x
  | _ -> Printf.printf "\n\n%s\nAssertion failed:\n\t%s\n\t= %s\n\n"
      "############################################################" 
      (!forward_prsvalue msg)
      (!forward_prsvalue x);
      flush stdout;
      raise MgsAssertionFailed
;;




(** La fonction while prend 2 arguments qui sont des fonctions MGS:
 ** - la premiere sert  valuer une condition
 ** - la seconde est le corp du while
 ** Ces fonctions prennent un arguments bidon : c'est juste pour geler l'expression de
 ** la structure de contrle while correspondant.
 **)
let _whilemgs_funct_funct (c, e) = 
  let arg = VAL_undef "dummy arg for frozen expression in while" 
  and res = ref(VAL_undef("empty while")) in
    begin
      while (bool_from_val(c(arg))) do res := e(arg) done;
      !res
    end
;;




(** Une fonction qui imprime des listes **)

let rec plist pps ppval debut sep fin l = 
  (pps debut); (plist_aux pps ppval sep l); (pps fin)

and plist_aux pps ppval sep = function
  | [] -> ()
  | a::[] -> ppval a; ()
  | a::l -> (ppval a); (pps sep); plist_aux pps ppval sep l


(** plist_aux2 pps ppval sep x f : une fonction qui imprime des listes.
 ** Meme fonction que plist_aux pps ppval sep sauf pour chaque lment 
 ** de la liste [x], on applique avant la fonction f
 **)

and plist_aux2 pps ppval sep x f = match x with
  | [] -> ()
  | a::[] -> ppval (f a); ()
  | a::l -> (ppval (f a)); (pps sep); plist_aux2 pps ppval sep l f 

(** Une fonction qui imprime des arrays **)

and parray pps ppval debut sep fin vect = 
  let flag = ref false in
    begin
      (pps debut);
      for i = 0 to (Array.length vect) - 1 do
	(if !flag then pps sep else flag := true);
	ppval vect.(i);
      done;
      (pps fin)
    end
;;


let rond f g = function x -> f (g x);;

(* la fonction update renvoie une liste de classes d'quivalence. Une
 * classe d'equivalence est un couple (e, ref l) o e est un representant de
 * la classe d'equivalence et l la liste des elements de la classe (en
 * particulier e est dans l). update prend en argument une fonction f,
 * un element x  rajouter dans la liste de classe d'equivalence et
 * cette liste. Si on trouve un representant e qui est equivalent a x
 * (test par la fonction f), alors on met  jour la liste des elements
 * de la classe de e. Sinon On cree une nouvelle classe qui contient
 * juste e et on la rajoute en fin. 
 *
 * Voic les fonctions _partition_...
 *)
let rec update_equivalence f x = function
  | [] -> [(x, ref [x])]
  | (((e, l) as cl)::suite) as eql -> 
      if bool_from_val((pf1 (f e)) x)
      then ((l := x::!l); eql)
      else cl::(update_equivalence f x suite)
;;

(** On tend le fold sur les entiers :
 ** fold f zero n va calculer: f(n, f(n-1, f(n-2, ...f(zero))))
 **)

let rec int_fold f zero = function
    0 -> zero
  | n -> (assert(n > 0);               (* Marre du stack overflow !! *)
	  f n (int_fold f zero (n-1)))
;;

let puissance x y  = Pervasives.exp(y *. Pervasives.log(x));;

let fact_tbl = Hash.create 50000;;


let rec fact_big_int b = 
  let bint = int_of_big_int b
  in
(*    Printf.printf "La taille de la fact_tbl est %d\n" (Hash.size fact_tbl);flush Pervasives.stdout; *)
    (try
       Hash.find fact_tbl bint
     with Not_found -> (let r = fact_rec b
			in
			  Hash.add fact_tbl bint r;
			  r))

and fact_rec b = 
  if (0 >= (compare_big_int b unit_big_int))
  then unit_big_int
  else mult_big_int b (fact_big_int(pred_big_int b))


and good_cnp_big_int n p = 
  if (0 >= compare_big_int n p)
  then (unit_big_int)
  else
    let num   = fact_big_int n
    and denom = mult_big_int (fact_big_int p) (fact_big_int (sub_big_int n p))
    in
    let r = div_big_int num denom
    in
      r

and good_anp_big_int n p = 
  if (0 > compare_big_int n p)
  then (unit_big_int)
  else
    let num   = fact_big_int n
    and denom = fact_big_int (sub_big_int n p)
    in
    let r = div_big_int num denom
    in
      r

and cnp_big_int n p =
  let i     = (try
		 (Big_int.int_of_big_int p)
	       with Failure "int_of_big_int" -> (raise (Erreur (Printf.sprintf "AUXFCT: could not convert %s to int" (string_of_big_int p)))))
  and n_dec = ref n
  and res   = ref unit_big_int
  in
    for j = 1 to i
    do
      begin
	res   := mult_big_int !res  !n_dec;
	n_dec := pred_big_int !n_dec
      end
    done;
    let r = div_big_int !res (fact_big_int p)
    in
(*      if (0 <> compare_big_int r (good_cnp_big_int n p)) then raise (Erreur (Printf.sprintf "C(%s, %s) : %s calcule, %s attendu" 
									       (string_of_big_int n)
									       (string_of_big_int p)
									       (string_of_big_int r)
									       (string_of_big_int (good_cnp_big_int n p))));
*)
      r
    

;;
    
