(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: gbf.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2007-06-26 01:48:18 $
 * Version: $Revision: 1.7 $
 * -------------------------------------------------------------------
 *
 * Status:  
 *    Implantation au niveau "representation MGS" des fonctions sur les 
 *    sequences.
 *
 *    Dans un rpertoire Xxx correspondant  une collection xxx, 
 *    le type xxx est defini dans def_xxx.ml et les operations sur 
 *    ce type sont dfinies dans core_xxx.ml. Le fichier kernel_xxx.ml
 *    contient la declaration dans la librairie MGS des fcts dfinies 
 *    dans xxx.ml
 *
 *    L'organisation de xxx.ml fichier suit l'ordre suivant:
 *
 *       1/ Fonctions de construction: ... -> seq (sauf extension)
 *             cons, join, append, shape, xxxify, iota, extension
 *
 *       2/ Prdicats, accs et observateur
 *             compare, val2bool, size, empty, count, 
 *             member, random, hd, tl, take
 *
 *       3/ Fold et autre itrations
 *             fold, map, iter, foldindexed, mapindexed, iterindexed,
 *             forall, exists, filter
 *
 *       4/ Neighbors, neighbor
 *
 *
 *       5/ Fonctions spcifiques
 *
 *
 *       6/ mise  jour du descripteur du type
 *              champs: make, xxxify et empty
 *
 * Ici :   xxx = gbf
 *
 *=========================================================================
 *)


open Global;;
open Def_seq;;
open Def_gbf;;
open Valeur;;
open Type;;
open Auxfct;;
open Dispatch_sig;;

open Type_gbf ;;
open Core_gbf ;;


__register_version "$RCSfile: gbf.ml,v $" "$Revision: 1.7 $" "$Date: 2007-06-26 01:48:18 $";;

let seq_desc = Type_seq.seq_desc
and set_desc = Type_set.set_desc
and bag_desc = Type_bag.bag_desc
;;

let find_gbf_def_from_deco d = 
  try
    Hash.find gbf_tbl d.name
  with Not_found -> (
    try
      Hash.find gbf_tbl ((List.nth d.derive_from 4).name)
    with Not_found -> raise (Erreur ("Gbf: find_gbf_def_from_deco: non existing Gbf "^d.name))
  )

let _val2bool_posgbf x = VAL_bool(array_exists (fun n -> 0 <> n)  (fst x)) ;;

let rec _size_gbf (tbl, _)               = VAL_int(Hash.size tbl)
and _size_posgbf p                   = VAL_int (-7)

and _following_any_any (s, p)        = let (a, b) = following

					 (function
					    | VAL_seq (vecteur, _) -> vecteur
					    | _                    -> raise (Erreur "Gbf: Following_any_any: Sequence of sequences of ... sequences required as first argment")
					 )

					 (if (is_collection s) 
					  then (sequify s)
					  else (VAL_seq ([| s |], seq_desc)))
					 
					 (Array.map
					    (function
					       | VAL_posgbf (coord, gbd) -> (coord, gbd)
					       | _                       -> raise (Erreur "Gbf: Following_any_any: Gbfpos are required for the second argument")
					    )
					    (if (is_collection p)
					     then (to_seq (sequify p))
					     else [| p |])
					 ) in
  VAL_gbf(a, (Hash.find Type.type_tbl b))
					 
and _add_gbf_gbf ((g1,t1), (g2,t2))  =
  let t = Type.commontype t1 t2 in
  let (g1,g2) = if (Hash.size g1) <= (Hash.size g2) then (g1,g2) else (g2,g1) in
    if (t.concrete) then (
      let ret = (Hash.copy g1) in
	Hash.iter (fun k d -> Hash.replace ret k d) g2 ;
	VAL_gbf(ret,t)
    ) else
      raise (Erreur "Gbf: _add_gbf_gbf: incompatible types of gbf")

and _setGBFPos_gbf_posgbf_any ((g,d), (coord,d'), v) =
  let gbf_def = find_gbf_def_from_deco d
  in
    if (gbf_def.gbf_name <> d'.gbf_name)
    then raise (Erreur "Gbf: _setGBFPos: the position is not from the Gbf")
    else
      Hash.replace g coord v ;
      VAL_gbf(g,d)


and _add_posgbf_posgbf (x1, x2)      = VAL_posgbf(add_sub 1 (x1, x2))
and _sub_posgbf_posgbf (x1, x2)      = VAL_posgbf(add_sub (-1) (x1, x2))
and _mult_int_posgbf (x1, x2)        = VAL_posgbf(mult_entier_posgbf (x1, x2))
and _mult_posgbf_int (x, y)          = VAL_posgbf(mult_entier_posgbf (y, x))

and _hd_gbf   (tbl, td)              = snd(Hash.hd tbl)
and _tl_gbf   (tbl, td)              = VAL_gbf ((Hash.tl tbl), td)
and _last_gbf (tbl, td)              = snd(Hash.last tbl)

and _exists_funct_gbf (f, (tbl, _))  = VAL_bool (Hash.exists (fun _ v -> bool_from_val v) tbl)
and _forall_funct_gbf (f, (tbl, _))  = VAL_bool (Hash.forall (fun _ v -> bool_from_val v) tbl)

and _count_any_gbf  (a, (tbl, _)) = VAL_int (let nb = ref 0
					     in
					       Hash.iter (fun _ v -> if (0 = compare_mgs_value v a) then (incr nb)) tbl;
					       !nb)

and _member_any_gbf  (a, (tbl, _)) = (try
					Hash.iter (fun _ v -> if (0 = compare_mgs_value v a) then raise BreakLoop) tbl;
					VAL_bool false
				      with BreakLoop -> VAL_bool true)

and _shape_gbf (g, deco) = (let taille = Hash.size g in
			    let tab    = (let r   = Array.make taille [||]
					  and idx = ref (taille-1)                                    (* Afin d'etre en accord avec sequify, i.e.              *)
					  in                                                          (* soit g un GBF, (shape(g))(map((\x.10*x),sequify(g)))  *)
					    Hash.iter (fun k _ -> r.(!idx) <- k; decr idx) g; (* place bien les elements de g la ou il faut            *)
					    r)
			    in
			      VAL_funct (function VAL_seq (s', _) -> (if (taille <> Array.length s')
								      then raise (Erreur "Gbf: Shape: argument is not of the same size of the original")
								      else (let g = Hash.create taille
									    in
									      Array.iteri (fun idx v -> Hash.add g tab.(idx) v) s';
									      VAL_gbf (g, deco)))
					   | _                    -> raise (Erreur "Gbf: Shape: a sequence as argument is required")))

and _iter_funct_gbf (fonc, (tbl, _))             = (Hash.iter (fun _ x -> fonc x; ()) tbl); VAL_undef "iter"

and _iterindexed_funct_gbf (fonc, (tbl, d))      = (let gbf_def = find_gbf_def_from_deco d
						    in
						      Hash.iter (fun k v -> (app_2 fonc (VAL_posgbf (k, gbf_def)) v); ()) tbl); VAL_undef "iter"
			       
and _map_funct_gbf (fonc, gbf)                   = VAL_gbf(map (fun k v -> (fonc v)) (fst gbf), snd gbf)
					  
and _mapindexed_funct_gbf (fonc, gbf)            = (
  let gbf_def = find_gbf_def_from_deco (snd gbf)
  in
    VAL_gbf(Core_gbf.map (fun k v -> (app_2 fonc (VAL_posgbf (k, gbf_def)) v)) (fst gbf), snd gbf)
)

and _fold_funct_any_gbf (fonc, zero, gbf)        = Core_gbf.fold (fun k v suite -> (app_2 fonc v suite)) (fst gbf) zero

and _foldindexed_funct_any_gbf (fonc, zero, gbf) =
  let gbf_def = find_gbf_def_from_deco (snd gbf)
  in
    Core_gbf.fold (fun k v suite -> (app_3 fonc (VAL_posgbf (k, gbf_def)) v suite)) (fst gbf) zero

and _neighbors_gbf_posgbf ((tbl, d), (t', d'))   = (
  let gbf_def = find_gbf_def_from_deco d in
    if (gbf_def.gbf_name <> d'.gbf_name)
    then raise (Erreur "Gbf: neighbors: the position is not from the Gbf of the argument")
    else (

      let tab = some_neighbors
		  gbf_def
		  tbl
		  t'
		  (Array.map
		     (function c -> (c, gbf_def))
		     (gbf_def.gbf_neighbors_array)
		  ) in
      let vals = Array.map (Hash.find tbl) tab in
	VAL_seq (vals, seq_desc)
    )
)

and neighbor (tbl, d) (t', d') v =
  let gbf_def = find_gbf_def_from_deco d
  in
    if (gbf_def.gbf_name <> d'.gbf_name)
    then raise (Erreur "Gbf: neighbor: the path/neighbor is not from the Gbf of the argument")
    else (let tab = some_neighbors gbf_def tbl t' v in
	  let vals = Array.map (Hash.find tbl) tab
	  in
	    vals
	 )

and _neighbor_gbf_posgbf_posgbf (g1, p1, p2)  = VAL_seq(neighbor g1 p1 [| p2 |], seq_desc)

and _neighbor_gbf_posgbf_seq (g1, p1, (s, _)) = VAL_seq(neighbor g1 p1 (Array.map
									  (function
									     | VAL_posgbf (coord, gbd) -> (coord, gbd)
									     | _                       -> raise (Erreur "Gbf: neighbor_gbf_posgbf_seq: Gbfpos are required for the second argument"))
									  s), seq_desc)

and _empty_gbf (tbl, _)   = VAL_bool (0 = Hash.size tbl)

and _setify_gbf  (tbl, _) = VAL_set((Hash.fold (fun _ e suite -> Core_set.add e suite) tbl Core_set.empty), set_desc)
and _bagify_gbf  (tbl, _) = VAL_bag((Hash.fold (fun _ e suite -> Core_bag.add_b e suite) tbl Core_bag.empty_b), bag_desc)
and _sequify_gbf (tbl, _) = VAL_seq((Array.of_list (Hash.fold (fun _ e suite -> e::suite) tbl [])), seq_desc)

and _sequify_posgbf (tab, _) = VAL_seq ((Array.map (function x -> VAL_int x) tab), seq_desc)

and _compareMgsVal_gbf_gbf ((tbl1, d1), (tbl2, d2)) = (let tdiff = compare_type d1 d2 
						       in 
							 if (tdiff <> 0) 
							 then VAL_int(tdiff)
							 else VAL_int (Hash.compare compare_mgs_value tbl1 tbl2))


and _smith_gbf(_, d) =
  let gdef = find_gbf_def_from_deco d in
  let diag = Array.make (smith_dim gdef.gbf_id) 0 in
    smith_diagonale gdef.gbf_id diag;
    VAL_seq(Array.map (fun x -> VAL_int x) diag, seq_desc)    


and _backpos_posgbf(tbl, gdef) =  
  let pos = Array.copy tbl in
    smith_back gdef.gbf_id pos;
    VAL_seq(Array.map (fun x -> VAL_int x) pos, seq_desc)

and _take_gbf_posgbf((g,c),(tbl,gdef)) =
  try
    Hash.find g tbl
  with Not_found -> VAL_undef "" 
;;


(** Extension --------------------------------- **)

let rec extension_gbf desc a =
  Type.check_vplist desc.name (to_funct desc.pospredicate) a;
  desc.empty (*VAL_gbf(Array.fold_left add_elem (Core_achain.new_achain()) a, desc)*)

(*and  add_elem ch =  function
  | VAL_seq(aa, _) -> Core_achain.add_acell ch (to_acell aa.(0)) aa.(1)
  | _ -> raise (Erreur "Internal Error: extension: add_elem in extension_achain")
*)

;;
