/*=======================================================-*-c++-*-======
 *
 * MGS : un modele general de systeme
 *
 * (U)TopoIa  : Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *            : Olivier Michel      : michel@lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: smith.h,v $ 
 * Path: %P%
 * Description:
 * Created:
 * Author: OM & JLG
 * Modified: $Date: 2005-12-16 18:33:18 $ at 
 * Version: $Revision: 1.2 $
 * Status: 
 * -------------------------------------------------------------------
 * Comments:  
 *              Forme Normal de Smith pour les GBF
 *
 * ===================================================================
 */



#ifndef _SMITH_H
#define _SMITH_H
#ifdef __GNUG__
#pragma interface
#endif





# ifdef EXPORT_MGS_C_DEVELOPMENT

#include <vector>
#include <assert.h>

namespace MGS {

typedef const char* String;
typedef vector<int> IntVect;
typedef vector<String> StringVect;

}

# else

#include "root.h"

#endif // --- INSIDE_MGS_C_DEVELOPMENT ----


namespace MGS {


template <class T> class Matrix;
typedef IntVect CoefVect;
typedef vector<CoefVect> Equations;
ostream& operator<<(ostream&, const CoefVect&);





// L'objet qui stocke les informations necessaires a la comparaison

struct SmithData
{
      // On calcule avec des long long,
      // mais cela plante sous Mac-OS X
      // typedef long long LongInteger;
      typedef long int LongInteger;
      // mais on stockera les matrices en long tout simple.
      typedef CoefVect::value_type Integer;

      Integer convert(LongInteger li)
      {
	 if ((((LongInteger)LONG_MIN) < li) 
	     && (li < ((LongInteger)LONG_MAX)))
	    return li;
	 else
	 {
	    cerr << "Sorry: normalization in abelian GBF "
		 << (name ? name : "") << " recquire bigger int"
		 << endl << "Send your example to the MGS team" << endl;
	    assert(0);
	    return 0;
	 }
      }

      Matrix<Integer>* L;   // matrice de transformation ver la fns
      Matrix<Integer>* L_1; // inverse de L
      CoefVect image;       // coefficients de torsion
      unsigned int ligne;   // nbre de generateurs
      unsigned int colone;  // nbre d'equations
      unsigned int rank0;   // nbre de coef de torsion a 0
      unsigned int rankt;   // nbre de coef de torsion !=0 et != 1
      String name;          // un nom informatif
      bool warn;            // si je dois etre verbeux ou silencieux
      
      SmithData(String s, 
		const StringVect& gen, const Equations&, 
		bool warn = true);

      ~SmithData();

      size_t hash(const CoefVect&) const;
      void normalise(CoefVect& v) const;
      bool check_normal(const CoefVect& v) const;
      void back(CoefVect& v) const;
      void modulo(CoefVect& v) const;
      void inc(CoefVect& pos, int alpha, const CoefVect& translate) const;

      unsigned int rank(unsigned int i) const { return image[i]; }

      // Methode ad-hoc pour un affichage ad-hoc des GBF de 
      // dimension 2 au plus
      int dim() const { return rank0 + rankt; }
      int nth_coeff(int) const;


   private:
      void make_image(Matrix<LongInteger>*);
      void make_left(Matrix<LongInteger>* left);
};






// -------------------------------------------------------

} // fin du fichier

#endif
