/*
 *
 *                              MGS
 *                              LaMI
 *                       Universite d'Evry Val d'Essonne
 *                   Olivier MICHEL, Jean-Louis GIAVITTO
 * File: $RCSfile: stub_gbf.c,v $ 
 * Path: %P%
 * Description:
 * Created:
 * Author: Olivier Michel & Jean-Louis Giavitto
 * Modified: $Date: 2004-12-14 14:58:24 $ at 
 * Version: $Revision: 1.1 $
 * Status:
 *
 */

#include <stdio.h>
#include <mlvalues.h>
#include <memory.h>

/* ****************************************************************************************************************************
 #####                                     #     #					       #######			       
#     #  #    #     #     #####  #    #	   ##    #   ####   #####   #    #    ##    #          #         ####   #####   #    #
#        ##  ##     #       #    #    #	   # #   #  #    #  #    #  ##  ##   #  #   #          #        #    #  #    #  ##  ##
 #####   # ## #     #       #    ######	   #  #  #  #    #  #    #  # ## #  #    #  #          #####    #    #  #    #  # ## #
      #  #    #     #       #    #    #	   #   # #  #    #  #####   #    #  ######  #          #        #    #  #####   #    #
#     #  #    #     #       #    #    #	   #    ##  #    #  #   #   #    #  #    #  #          #        #    #  #   #   #    #
 #####   #    #     #       #    #    #	   #     #   ####   #    #  #    #  #    #  ######     #         ####   #    #  #    #

 **************************************************************************************************************************** */


value smith_init(value nom_string, 
		 value gens_string_array,
		 value equations_matrix,
		 value warning_bool)
{
  CAMLparam4(nom_string, gens_string_array, equations_matrix, warning_bool);

  CAMLlocal2(ptr, base);
  int nb_equations, ptr_idx, base_idx;
  char *chaine;

  /* On cree un nouveau vecteur de generateurs */
  gbf_init_vect();

  /* On y ajoute tous les generateurs */
  for(ptr = gens_string_array, ptr_idx = 0; ptr_idx < Wosize_val(gens_string_array); ptr_idx++)
    {
      chaine = String_val(Field(ptr, ptr_idx));
      /*      printf("On a le generateur %s\n", chaine); */
      gbf_add_generator(chaine);
    }

  /* On initialise les equations                           */
  /* et on cree le vecteur de taille nombre_de_generateurs */
  gbf_init_equations();
  
  /*printf("Initialisation des equations\n"); */

  /* On cree les equations */
  for(base = equations_matrix, base_idx = 0, nb_equations = Wosize_val(base); base_idx < nb_equations; base_idx ++)
    for(ptr = Field(base, base_idx), ptr_idx = 0; ptr_idx < Wosize_val(ptr); ptr_idx++)
      {
	/*	printf("Valeur du coefficient du %dieme generateur %d\n", ptr_idx, Int_val(Field(ptr, ptr_idx))); */
	gbf_add_coeff(Int_val(Field(ptr, ptr_idx)));
      }

  fflush(stdout);

  /* Et la matrice pour la FNS est cree */
  CAMLreturn(Val_int(gbf_make_smith(String_val(nom_string), Bool_val(warning_bool))));
}


value smith_compute_index(value smith_index_int,  value ml_array)
{
  CAMLparam2(smith_index_int, ml_array);

  int len = Wosize_val(ml_array), i;
  int tab[len];

  for(i=0; i < len; i++) 
    tab[i]=Int_val(Field(ml_array, i));
      
  gbf_normalise(tab, len, Int_val(smith_index_int));
      
  for(i=0; i < len; i++) 
    Field(ml_array, i) = Val_int(tab[i]);

  CAMLreturn(Val_unit);
}


value smith_normalise_back_and_mod(value ginit_int, value gquotient_int, value ml_array)
{
  CAMLparam3(ginit_int, gquotient_int, ml_array);

  int len = Wosize_val(ml_array), i;
  int tab[len];

  for(i=0; i < len; i++)
    tab[i]=Int_val(Field(ml_array, i));

  gbf_smith_normalise_back_and_mod(Int_val(ginit_int), Int_val(gquotient_int), tab, len);

  for(i=0; i < len; i++) 
    Field(ml_array, i) = Val_int(tab[i]);

  CAMLreturn(Val_unit);
}

value smith_back(value ginit_int, value ml_array)
{
  CAMLparam2(ginit_int, ml_array);

  int len = Wosize_val(ml_array), i;
  int tab[len];

  for(i=0; i < len; i++)
    tab[i]=Int_val(Field(ml_array, i));

  gbf_smith_back(Int_val(ginit_int), tab, len);

  for(i=0; i < len; i++) 
     Field(ml_array, i) = Val_int(tab[i]);

  CAMLreturn(Val_unit);
}

value smith_normalise_to_initial(value ginit_int, value ml_array)
{
  CAMLparam2(ginit_int, ml_array);

  int len = Wosize_val(ml_array), i;
  int tab[len];

  for(i=0; i < len; i++)
    tab[i]=Int_val(Field(ml_array, i));

  gbf_smith_normalise_to_initial(tab, len, Int_val(ginit_int));

  for(i=0; i < len; i++) 
    Field(ml_array, i) = Val_int(tab[i]);

  CAMLreturn(Val_unit);
}

value smith_neighborsq(value ginit_int, value origine_array, value destination_array, value chemin_array)
{
  CAMLparam4(ginit_int, origine_array, destination_array, chemin_array);

  int len = Wosize_val(origine_array);
  int i;
  int origine_tab[len], destination_tab[len], chemin_tab[len];

  for(i=0; i < len; i++)
    {
      origine_tab[i]    = Int_val(Field(origine_array, i));
      destination_tab[i]= Int_val(Field(destination_array, i));
      chemin_tab[i]     = Int_val(Field(chemin_array, i));
    }

  if (gbf_neighborsQ(Int_val(ginit_int), origine_tab, len, destination_tab, chemin_tab) == 1)
    CAMLreturn(Val_true);
  else 
    CAMLreturn(Val_false);  
}

value smith_compute_neighbor(value ginit_int, value nb_int, value origine_array, value chemin_array, value destination_array)
{
  CAMLparam5(ginit_int, nb_int, origine_array, chemin_array, destination_array);

  int len = Wosize_val(origine_array);
  int i;
  int origine_tab[len], chemin_tab[len], destination_tab[len];

  for(i=0; i < len; i++)
    {
      origine_tab[i]    = Int_val(Field(origine_array, i));
      chemin_tab[i]     = Int_val(Field(chemin_array, i));
    }

  /* destination_tab[i]= Int_val(Field(destination_array, i)); */


  gbf_compute_neighbors(Int_val(ginit_int), origine_tab, len, Int_val(nb_int), chemin_tab, destination_tab);

  for (i=0; i < len; i++) 
    Field(destination_array, i) = Val_int(destination_tab[i]);

  CAMLreturn(Val_unit);  
}

value smith_dim(value ginit_int)
{
  CAMLparam1(ginit_int);
  CAMLreturn(Val_int(gbf_smith_dim(Int_val(ginit_int))));
}


value smith_coeffs(value ginit_int, value tableau_array)
{
  CAMLparam2(ginit_int, tableau_array);

  int i, tableau_tab[2];

  gbf_smith_coeffs(Int_val(ginit_int), tableau_tab);

  Field(tableau_array, 0) = Val_int(tableau_tab[0]);
  Field(tableau_array, 1) = Val_int(tableau_tab[1]);

  CAMLreturn(Val_unit);
}


value smith_diagonale(value ginit_int, value tableau_array)
{
  CAMLparam2(ginit_int, tableau_array);

  int i, dim, tableau_tab[100];

  dim = gbf_smith_diagonale(Int_val(ginit_int), tableau_tab);

  for (i=0; i <= dim; i++)
     Field(tableau_array, i) = Val_int(tableau_tab[i]);

  CAMLreturn(Val_unit);
}

