(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: topo_gbf.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-11-29 17:21:00 $
 * Version: $Revision: 1.5 $
 * -------------------------------------------------------------------
 *
 * Status: voir le commentaire general dans topology.ml
 *          
 * Ici on developpe les instance de generic_topo qui correspondent 
 * aux collections Associatives/Commutative (et parfois idempotente)
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Printer;;
open Topology;;
open Core_gbf;;
open Gbf;;
open Def_gbf;;

__register_version "$RCSfile: topo_gbf.ml,v $" "$Revision: 1.5 $" "$Date: 2005-11-29 17:21:00 $";;


let string_of_int_array a =  "|"^(Array.fold_right (fun v suite-> (string_of_int v)^(if (suite = "") then "" else " ")^suite) a "")^"|"


and in_place_shuffle_array len a = 
  let nb  = ref (if (len <= 1)
		 then 0
		 else 
		   if (len = 2) 
		   then 1 
		   else len) (* Pour eviter que sur un tableau de longueur 2 on fasse 2 permutations inverses. *)
  in                         (* Le risque est moins grand sur un tableau de taille superieure.                 *)
    while (!nb > 0)
    do
      let src = Random.int len
      and dst = Random.int len
      in
	if (src = dst)
	then decr nb
	else (let tmp = a.(src)
	      in
		a.(src) <- a.(dst);
		a.(dst) <- tmp;
		decr nb)
    done;
    a
;;

exception FoundPos of (int array);;

class topo_gbf ((c : 'collection), couleur) mgsval =
object(self)
  inherit [int array] generic_topo 
    string_of_int_array               (** afficheur de position    **)
    (c, couleur)                      (** collection a transformer **)
    mgsval                            (** l'argument complet       **)
    (let n      = Global.crandom (Hash.size c) in
     let (k, _) = Hash.nth n c
     in
       (*       Printf.printf "On commence le filtrage du sommet %d de coordonnees %s\n" n (string_of_int_array k); *)
       k)                             (** position de depart       **)

  val _gbf_def             = Core_gbf.find_gbf_def couleur.name

  val _voisins = Array.make (2 * Array.length((Core_gbf.find_gbf_def couleur.name).generateurs)) [||]

  val _used    = (let tbl = Hash.create (Hash.size c)
		  in
		    Hash.iter (fun k v -> Hash.add tbl k PosFree) c;
		    tbl)

  val mutable _batch = ([] : (Terme.path_type * 'position path * mgs_value) list)
  val mutable _last_index = Global.crandom (Hash.size c)

  method state p             = Hash.find _used p
  method update_state fonc p = Hash.replace _used p (fonc(Hash.find _used p))

  method unvisit_all = Hash.iter (fun k v -> if (pos_visited v) then (Hash.replace _used k (pos_unvisit v))) _used

  method mgs_value_from_pos pos = VAL_posgbf (pos, _gbf_def)

  method next_usable _ =
    if (!match_random <= 0) then
      try
	Hash.iter_from
	  _last_index
	  (fun k v -> if (v = PosFree) then raise (FoundPos k))
	  _used;
	raise NoMoreUsable
      with FoundPos p -> (assert
			    begin 
			      if !trace_opt then
				(Printf.printf "---- topoGBF: next_usable: found %s (1)\n" (string_of_int_array p)); 
			      true
			    end;
			  _last_index <- Hash.last_iter_index();
			  p)
    else
      try
	Hash.iter_from_random_key
	  (fun k v -> if (v = PosFree) then raise (FoundPos k))
	  _used;
	raise NoMoreUsable
      with FoundPos p -> (assert
			    begin 
			      if !trace_opt then
				(Printf.printf "---- topoGBF: next_usable: found %s (2)\n" (string_of_int_array p)); 
			      true
			    end;
			  p)
	

  method add_match pt p v = _batch <- (pt, p, v)::_batch
  method get_value_from_pos p = Hash.find c p

  method some_neighbors pos kind =
    (let { gbf_type            = gb_type;
	   generateurs         = gens;
	   matrice             = mat;
	   gbf_id              = gbf_idx;
	   gbf_neighbors_array = gna
	 } = _gbf_def in

     let nb_gens               = Array.length gens in
     let origine               = Array.copy pos                                             (* Le point de depart    t              *)
     and dest                  = Array.make nb_gens 0 in                                    (* Le point origine+voisin              *)
     let compute_neighbor vois = smith_compute_neighbor gbf_idx 1 origine vois dest (* Le calcul de l'adresse destination   *) in

     let voisins_effectifs   = ref 0
       
     and array_voisin, quantite = (match kind
				   with
				       AllDirections    -> gna, (Array.length gna)

				     | SomeDirections a -> (Array.map 
							      (function (VAL_posgbf (c, {gbf_id = idx})) -> assert(idx=gbf_idx); c
								 |      _                                -> raise (Erreur "only path in GBF match"))
							      a), (Array.length a)
				  )
     in
       for i = 0 to (quantite - 1)
       do
	 compute_neighbor array_voisin.(i);
	 (try
	    let statut = Hash.find _used dest                         (* Un element se trouve-t-il a cette position ? *)
	    in
	      if (pos_usable statut)                                          (* Est-il utilisable ?                          *)
	      then (_voisins.(!voisins_effectifs) <-  Array.copy dest;
		    incr voisins_effectifs)
	      else ()
	  with Not_found -> ())
       done;
       !voisins_effectifs, (Array.sub (if (!match_random > 0) then (in_place_shuffle_array !voisins_effectifs _voisins) else _voisins) 0 !voisins_effectifs)
    )

  method direction_from_to p1 p2 = VAL_undef "direction"


  method private ajoute_tous tbl (kind, chemin, rhs) = 
    let idx = ref 0 
    and longueur_chemin = List.length chemin
    in
      (match rhs
       with
	   VAL_seq (seq, col) -> (if (longueur_chemin <> (Array.length seq))
				  then raise (Erreur "in GBF, replacement must be of same arity")
				  else (List.iter
					  (function pos -> ((match seq.(!idx)
							     with 
								 VAL_undef _ -> ()                           (* C'est que l'on souhaite le supprimer *)
							       | x           -> Hash.replace tbl pos x);
							    incr idx))
					  chemin))
	 | VAL_undef _        -> if (longueur_chemin = 1) then () else (raise (Erreur "in GBF, replacement must be of same arity"))

	 | x                  -> if (longueur_chemin = 1) then Hash.add tbl (List.hd chemin) x)


  method build_result =
    let tbl       = Hash.create (Hash.size c)
    in
      List.iter (self#ajoute_tous tbl) _batch;
      Hash.iter (fun k v -> (if (pos_usable v)
				then Hash.replace tbl k (Hash.find c k))) _used;
      VAL_gbf (tbl, couleur)
end
;;
