(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: type_gbf.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2007-06-06 06:48:45 $
 * Version: $Revision: 1.3 $
 * -------------------------------------------------------------------
 *
 * Voir le fichier Common/type.ml
 * 
 *=========================================================================
 *)



open Global;;
open Valeur;;
open Type;;
open Type_scalar;;

__register_version "$RCSfile: type_gbf.ml,v $" "$Revision: 1.3 $" "$Date: 2007-06-06 06:48:45 $";;


(*!*)(** 
  @module scalar  
  @name 'posgbf
  @type any_bool
  Returns [true] if the argument is a posgbf and [false] for any other value. 

  [posgbf] is also the type of the simplex.
*)(*!*)
let posgbf_desc = create_type
	        { name = "'posgbf";
		  concrete = true;
		  leibniz = false;
		  predicate = wrap_pred(is_posgbf);
		  empty = VAL_undef "dummy for default value field in type descriptor 'posgbf" ;
		  pospredicate = scalar_desc.pospredicate;
		  make = scalar_desc.make;
		  xxxify = VAL_undef "Type_scalar: undef_desc: not yet implemented on posgbf";
		  derive_from = [scalar_desc];
		  father_of = [];
		  uid = get_new_uid();
		}
;;

(*!*)(** 
       @module collections
       @name 'gbf
       @type any_bool
       Set type and predicate.
       @voir 'set      'set
       @voir 'bag      'seq
       @voir 'monoidal 'monoidal
*)(*!*)

let gbf_desc = create_type
		 { name = "'gbf";
		   concrete = false;
		   leibniz = false;
		   predicate = wrap_pred(is_newton);
		   empty = VAL_undef "dummy for empty field in type descriptor 'gbf";
		   pospredicate = pospredicate_posgbf;
		   make = make_dummy;
		   xxxify = xxxify_dummy "'gbf";
		   derive_from = [newton_desc];
		   father_of = [];
		   uid = get_new_uid();
		 }
;;

gbf_desc.predicate <- wrap_pred(coll_pred gbf_desc);;

let rec extension_gbf nom desc a      = (*raise (Erreur "extension_gbf not yet implemented")*)
  Type.check_vplist desc.name (to_funct desc.pospredicate) a;
  let g = to_gbf desc.empty in
    VAL_gbf(Array.fold_left add_elem g a, desc)

and  add_elem h =  function
  | VAL_seq(aa, _) -> Hash.add h (fst (to_posgbf aa.(0))) aa.(1) ; h
  | _ -> raise (Erreur "Internal Error: extension: add_elem in extension_gbf")
;;


(** !!! Attention !!!
 *** Ne pas oublier de mettre  jour les champs
 ***     _ make 
 ***     _ xxxify
 ***     _ empty
 *** du descripteur de type dans le fichier bag.ml
 ***)

(* === Les gbfs ====================================== *)

let rec create_new_gbf nom gbfspec =
  let ret =
    try
      let t = Hash.find type_tbl nom
      in let err = "Error: type  "^nom^"  already defined."
      in prerr_endline err; prftype stderr t; raise (Erreur err)
    with Not_found -> 
      (if Hash.mem Environnement.var_tbl nom
       then prerr_endline ("Warning: type predicate  "^nom^"  hiddes a function with same name");
       let nt = create_type { name = nom;
		 	      concrete = true;
			      leibniz = false;
			      predicate = gbf_desc.predicate;
			      empty =  VAL_gbf ((Hash.create 0), gbf_desc);
			      pospredicate = pospredicate_posgbf;
			      make = make_dummy;
			      xxxify = xxxify_dummy nom;
			      derive_from = [gbf_desc];
			      father_of = [];
			      uid = get_new_uid();
			    }
       in 
	 begin
	   nt.empty <- change_type nt nt.empty;
	   nt.predicate <- wrap_pred(coll_pred nt);
	   nt.make <- extension_gbf nom nt;
	   ignore(Environnement.addfct nom nt.predicate);
	   nt
	 end)
  in
    Core_gbf.process Def_gbf.Abelian nom gbfspec;
    ret
