(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: gendispatch.mll,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2006-07-21 19:25:17 $
 * Version: $Revision: 1.7 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *=========================================================================
 *)

{

open Pervasives;;
open Printf;;
open Unix;;
open Global;;


__register_version "$RCSfile: gendispatch.mll,v $" "$Revision: 1.7 $" "$Date: 2006-07-21 19:25:17 $";;


(** Noms du fichiers de signatures et 
 ** du fichiers contenant l'implementation 
 ** des fct de dispatch 
 **)
let sigfile = "dispatch_sig.ml"
and fctfile = "dispatch.ml"
and invfile = "LIST_dispatch.txt"
;;

let post_sig = "
  let compare_mgs_value a b =
    match compareMgsVal a b with
      | VAL_int n -> n
      | _ -> failwith \"internal error: compare_mgs_value: compareMgsVal does not return an integer !\"
  ;;
  let equal_mgs_value a b = 
    match compareMgsVal a b with
      | VAL_int 0 -> true
      | VAL_int _ -> false
      | _ -> failwith \"internal error: equal_mgs_value: compareMgsVal does not return an integer !\"
  ;;
  let notequal_mgs_value a b = 
    match compareMgsVal a b with
      | VAL_int 0 -> false
      | VAL_int _ -> true
      | _ -> failwith \"internal error: equal_mgs_value: compareMgsVal does not return an integer !\"
  ;;
  let equalMgsVal a b = 
    match compareMgsVal a b with
      | VAL_int 0 -> mgstrue
      | VAL_int _ -> mgsfalse
      | _ -> failwith \"internal error: equal_mgs_value: compareMgsVal does not return an integer !\"
  ;;
  let notequalMgsVal a b = 
    match compareMgsVal a b with
      | VAL_int 0 -> mgsfalse
      | VAL_int _ -> mgstrue
      | _ -> failwith \"internal error: equal_mgs_value: compareMgsVal does not return an integer !\"
  ;;
"
;;

(** ======================================================= **)



let comment = ref 0;;
let current_fun = ref "";;
let current_args = ref ([] : string list);;



(** Table des fonctions a dispatcher 
 ** un token de la forme _xxx_a1_a2_a2 donne lieu a une entree
 **  - de cle xxx
 **  - de valeur [a1; a2; a3]
 **)
let fun_table = (Hashtbl.create 1024 : (string, string list) Hashtbl.t)
;;


(** creation de l'ensemble des noms des fonctions a dispatcher **)

module Ss = Set.Make(struct type t = string let compare=compare end);;

let all_name () = Hashtbl.fold (fun k v s ->  Ss.add k s) fun_table Ss.empty
;;


(** on refait le nom  _xxx_a1_a2_a2 a partir de x et de [a1; a2; a3] **)

let case_name f l = List.fold_left (fun x y -> x^"_"^y) ("_"^f) l
;;




(** entete des fichiers generes ------------------------------------------ *)

let banner out openflag =
  fprintf out 
    "\n(***************************************************************************************";
  fprintf out 
    "\n ** GENERATED FILE: DO NOT MODIFY BY HAND\n **\n" ;
  let tm = Unix.localtime(Unix.gettimeofday())
  in fprintf out " ** %s %d/%d/%d at %2d:%s%d:%s%d\n"
       "this file was generated the: "
       (1900 + tm.tm_year)
       (1 + tm.tm_mon)
       tm.tm_mday
       tm.tm_hour
       (if tm.tm_min < 10 then "0" else "")
       tm.tm_min
       (if tm.tm_sec < 10 then "0" else "")
       tm.tm_sec;
    fprintf out " ** by gendispatch:  %s %s %s\n%s\n\n\n\n"
      "$RCSfile: gendispatch.mll,v $"
      "$Revision: 1.7 $" 
      "$Date: 2006-07-21 19:25:17 $"
      " ***************************************************************************************)";
    if openflag then fprintf out "\n\nopen Global;;\nopen Valeur;;\n\n"
;;


(* --- generation des signatures ----------------------------------------- *)

let rec gen_sig out names = 
  print_string ("-- genere sig file "^sigfile^"\n");
  banner out true;
  Ss.iter (gsig1 out) names;
  fprintf out "\n\n(** post-declarations included from gendispatch.post_sig **)\n\n%s\n" post_sig;
  close_out out

and gsig1 out f = gsig2 out f (Hashtbl.find_all fun_table f)

(** f = nom de la fct,
 ** params = liste des listes des arguments de dispatchs, i.e.
 ** si on avait eu _xxx_a1_a2_a2 et _xxx_b1_b2_b2 alors
 ** f = x et params = [[a1;a2;a3]; [b1;b2;b3]]
 **)
and gsig2 out f params = 
  let n = check_size f params;
  in fprintf out "let (%s_forward : (" f;
     psig out (List.hd params);
     fprintf out ") ref) = ref (function _ -> failwith \"internal error: forward dispatch stub %s\");;\nlet %s x = !%s_forward x;;\n\n" f f f

and psig out = function
  | []   -> fprintf out "Valeur.mgs_value"
  | a::l -> fprintf out "%s -> " (signame a); psig out l

and signame = function
  | s when  '\'' = String.get s 0 -> String.sub s 1 ((String.length s)-1)
  | _ -> "Valeur.mgs_value"


(** verifie que dans la liste params precedente chaque element a la
 ** meme longueur, i.e. correspond a des cas prenant le meme nombre
 ** d'argument. Ce nombre d'argument est retourne.
 **)
and check_size f = function
  | [] -> failwith("No case in function "^f)
  | a::l -> check_size2 f (List.length a) l

and check_size2 f n = function
  | [] -> n
  | a::l -> if n = List.length a then check_size2 f n l
	    else failwith("Bad case "^(case_name f a))
;;



(* --- generation des fonctions de dispatch ------------------------------ *)


(* Fct de comparaison des cas :
 * on veut trier les cas du plus particulier au plus general.
 * "any" est le filtre qui pese le plus (i.e.: le + general) et 
 * les sequences de filtres sont ordonnes lexicographiquement. 
 *)
let rec case_compare l1 l2 = 
  match l1, l2 with
  | [], [] -> 0
  | _, [] -> 1
  | [], _ -> -1
  | "any"::l1', "any"::l2' -> case_compare l1' l2'
  | "any"::l1', _::l2' -> 1
  | _::l1', "any"::l2' -> -1
  | a::l1', b::l2' -> if a = b then case_compare l1' l2'
                      else (if a > b then 1 else -1)
;;

(* ensemble de cas: ca permet d'avoir l'unicite des cas et en plus de les trier *)
module SetCases = Set.Make(struct type t = string list let compare = case_compare end);;

exception SlashFound of int;;

let rec gen_fct out names =
  print_string ("-- genere fct file "^fctfile^"\n");
  banner out true;
  gopen out;
  fprintf  out "\nlet rec __dummyDispatchFct x = x\n\n";
  Ss.iter (gfct1 out) names;
  fprintf out ";;\n\n(* initialization of forward variables *)\n\n";
  Ss.iter (gfct3 out) names;
  fprintf out "\nprint_endline \"(* dispatch functions initialized *)\"";
  close_out out
	
and gopen out =
  fprintf out "open Valeur;;\n\n";
  Arg.parse [] (gopen2 out) "cat file_1 ... file_n | gendispatch file_1 ... file_n";
  fprintf out "\n\n"

and filename s =
  try
    for i = ((String.length s) - 1) downto 0 do
      if ((s.[i]) = '/') then raise (SlashFound i)
    done ;
    s
  with
      SlashFound i -> String.sub s (i+1) ((String.length s) - i - 1)

and gopen2 out x =
  fprintf out "open %s;;\n" (String.capitalize (filename x) )

and gfct1 out f  = 
  let list_cases = (Hashtbl.find_all fun_table f)
  in let set_cases = List.fold_left (fun s elt -> SetCases.add elt s) SetCases.empty list_cases
  in let sorted_cases = SetCases.elements set_cases
  in gfct2 out f sorted_cases


(** f et params comme pour gsig2 
 **)
and gfct2 out f params =
  let last_case = List.hd (List.rev params) 
  in
    fprintf out "\nand __%s " f;
    parg_without_comma out "y" last_case;
    fprintf out " = match ";
    parg out "y" last_case;
    fprintf out " with\n";
    List.iter (gcase out f) params;
    defaultcase out f last_case;
    fprintf out "\n"

(** generation de l'initialisation de la variable forward d'une fct. dispatch **)
and gfct3 out f = fprintf out "Dispatch_sig.%s_forward := __%s;;\n" f f 

(** generation d'un cas du dispatch: l contient le type des arguments *)
and gcase out f l =
  fprintf out "| ";
  pcase out l;
  fprintf out " -> %s(" (case_name f l);
  parg out "x" l;
  fprintf out ")\n"
    
(** ecriture du pattern **)
and pcase out l = pcase2 out 0 l

and pcase2 out n = function 
  | [] -> ()
  | [a] -> fprintf out "%s" (fname n a)
  | a::l -> fprintf out "%s, " (fname n a); pcase2 out (n+1) l

and fname n = function
  | "any" -> "x"^(string_of_int n)
  | s when '\'' = String.get s 0 -> "x"^(string_of_int n)
  | s ->  "(VAL_"^s^" x"^(string_of_int n)^")"

and parg out s l = parg2 out s 0 l

and parg_without_comma out s l = parg3 out s 0 l

and parg2 out s n = function 
  | [] -> ()
  | [_] -> fprintf out "%s%d" s n
  | _::l -> fprintf out "%s%d, " s n; 
            parg2 out s (n+1) l

and parg3 out s n = function 
  | [] -> ()
  | [_] -> fprintf out "%s%d" s n
  | _::l -> fprintf out "%s%d " s n; 
            parg3 out s (n+1) l


(** creation du cas par defaut: 
 **              x1, x2, x3 -> <message d'erreur>
 ** qui recupere tous les cas non definie pour une fonction de dispatch.
 ** On utilise la fonction :
 **             constructor_string
 ** qui doit etre definie dans valeur.ml, qui prend une mgs_value et qui
 ** retourne une chaine de caractere identifiant son constructeur 
 ** (i.e. son type)
 ** On ne genere un cas par defaut que s'il existe un argument qui ne soit 
 ** ni any ni un argument 'xxx (ou xxx est un type arbitraire)
 **)
and defaultcase out f l = 
  if List.exists (fun x -> (x <> "any") & ('\'' <> String.get x 0)) l then
  begin
    fprintf out "| ";
    parg out "x" l; 
    fprintf out " -> let err = \"Error: dispatch function %s, case: \"^" f;
    perr out 0 l;
    fprintf out "\": not defined.\" in\n";
    fprintf out "\t\t(if false == !Global.silent_error then begin\n";
    fprintf out "\t\t\tPervasives.flush_all();\n";
    fprintf out "\t\t\tPrintf.fprintf stderr \"%%s\\n\" err;\n";
    let cpt = ref 0 in
      List.iter (fun x -> 
		   (if '\'' <> x.[0] then 
		      fprintf out 
			"\t\t\t Printf.fprintf stderr \"\\targument: %%a\\n\" Printer.prfvalue2 x%d;\n" (!cpt));
		   incr cpt) l;
      fprintf out "\t\tend);\n";
      fprintf out "\t\tif !Global.abort_on_error then raise (Erreur err)\n";
      fprintf out "\t\telse ((Valeur.pf1 !Valeur.current_error_handler) (VAL_string \"dispatch function %s failed\"))\n" f
  end
 
and perr out n = function
  | [] -> ()
  | [a] -> 
      (if '\'' = a.[0] 
       then fprintf out "\"%s\"^" a
       else fprintf out "(constructor_string x%d)^" n);
  | a::l -> 
      (if '\'' = a.[0] 
       then fprintf out "\"%s * \"^" a
       else fprintf out "(constructor_string x%d)\n\t^\" * \"^" n);
      perr out (n+1) l
;;


(** gen_inv:
 ** pour tous les types, ont veut generer la liste des fonctions dispatches 
 ** dans lequel il apparait comme argument. On part de la fun_table et on cree 
 ** une autre table, inv_tbl, de cle le nom des types et de valeur l'ensemble 
 ** des fonctions dans lequel il apparait. Plus exactement les elements de cet
 ** ensemble sont des couples (non sans argument, nom complet).
 **)

module S2 = Set.Make(struct type t = (string*string) let compare=compare end);;

let rec gen_inv out tbl =
  print_string ("-- genere catalog file "^invfile^"\n");
  banner out false;
  let inv_tbl = (Hashtbl.create 1024 : (string, S2.t) Hashtbl.t) in
    begin
      Hashtbl.iter (add_name_to_type inv_tbl) tbl;
      let args = Hashtbl.fold (fun arg _  l -> arg::l) inv_tbl [] in
      let sorted_args = List.sort compare args
      in List.iter (fun x -> print_inv_tbl out x (Hashtbl.find inv_tbl x)) sorted_args
    end

and add_name_to_type tbl fct argl = List.iter (add_one_name_to_type tbl fct (case_name fct argl)) argl

and add_one_name_to_type tbl fct fullfct arg =
  if Hashtbl.mem tbl arg
  then Hashtbl.replace tbl arg (S2.add (fct, fullfct) (Hashtbl.find tbl arg))
  else Hashtbl.replace tbl arg (S2.singleton (fct, fullfct))

and print_inv_tbl out arg fcts =
  begin
    fprintf out "\n\n-------------------------------------------------------\n";
    fprintf out "%s\n" arg;
    fprintf out "-------------------------------------------------------\n";
    let len = S2.fold (fun (name, _) x -> max x (String.length name)) fcts 0
    in S2.iter (fun (f, ff) -> fprintf out "\t%s \t%s\n" (pad_name len f) ff) fcts
  end




(*** la fonction qui enchaine les traitements:
 *** - generation des signatures
 *** - generation des fichiers de dispatch
 *** - generation du rapport inverse (fichier de documentation
 ***)
let gen_code () = 
	let names = all_name()
	in gen_sig (open_out sigfile) names;
	   gen_fct (open_out fctfile) names;
	   gen_inv (open_out invfile) fun_table
;;


let stats() = 
  let all       = all_name() 
  and fonctions = Array.make 100 0
  and fmin      = Array.make 100 0
  and fmax      = Array.make 100 0
  and dispatch  = Array.make 100 0
  and nb        = ref 0
  in
    for i = 0 to 99
    do
      begin
	fmin.(i) <- max_int;
	fmax.(i) <- min_int;
      end
    done;
    Ss.iter (function nom -> (let liste = Hashtbl.find_all fun_table nom
			      in
			      let sz = List.length(List.hd liste)
			      in
				for j = 0 to 10
				do
				  begin
				    let subset = List.filter (function x -> List.length(x) == j) liste
				    in
				      fmin.(j) <- min (fmin.(j)) (List.length(subset));
				      fmax.(j) <- max (fmax.(j)) (List.length(subset));
				  end
				done;
				fonctions.(sz) <- fonctions.(sz) + 1;
				List.iter (function elt -> (let size = List.length elt
							    in
							      if (size > !nb) 
							      then nb := size
							      else ();

							      dispatch.(size) <- dispatch.(size) + 1)) liste)) all;

    Printf.printf "Il y a en tout %d fonctions surchargees definies\n" (Ss.cardinal all);
    Printf.printf "Il y a en tout %d fonctions ecrites\n" (let nb = ref 0 in Hashtbl.iter (fun _ _ ->  incr nb) fun_table; !nb);
    for i = 0 to (!nb + 1)
    do
      begin
	Printf.printf "Arite %d : Il y a %d dispatchs qui correspondent effectivement a %d fonctions avec min %d et max %d \n" i dispatch.(i) fonctions.(i) fmin.(i) fmax.(i) ;
	flush Pervasives.stdout
      end
    done
;;


}



let digit  = ['0'-'9']
let ident  = ['A'-'Z''a'-'z']+['a'-'z''A'-'Z''0'-'9']*
let charident  = ['A'-'Z''a'-'z''0'-'9''_']+
let white = [' ''\n''\t']


rule token =
parse
  | (white)'_'(ident)'_' { 
              let lbuf = Lexing.lexeme lexbuf
              in current_fun := String.sub lbuf 2 ((String.length lbuf) - 3);
	      current_args := [];
	      parse_id lexbuf }
  | "(*"          { incr comment; parse_comment lexbuf }
  | eof           { gen_code(); 
	            print_string "-- dispatch analysis done\n";
	            print_newline();
	            stats();
		    exit(0) }
  | _             { token lexbuf }


and parse_comment = parse
    '('+'*'        { incr comment; parse_comment lexbuf }
  | '*'+')'        { decr comment; (if (!comment = 0) 
				    then token lexbuf
				    else parse_comment lexbuf)}
  | '*'            { parse_comment lexbuf }
  | '('            { parse_comment lexbuf }
  | [^'(''*''\n']* { parse_comment lexbuf }
  | '\n'           { parse_comment lexbuf }

and parse_id = parse
    '\''(ident) { current_args := (Lexing.lexeme lexbuf)::(!current_args); parse_id lexbuf }
  | ident      { current_args := (Lexing.lexeme lexbuf)::(!current_args); parse_id lexbuf }
  | '_'        { parse_id lexbuf }
  | _          { Hashtbl.add fun_table !current_fun (List.rev !current_args); token lexbuf }

(* --- End ------------------------------------------------------- *)
