(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: kernel_del.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-07-12 14:23:11 $
 * Version: $Revision: 1.2 $
 * -------------------------------------------------------------------
 *
 * Status:  
 *    Implantation au niveau "representation MGS" des fonctions sur les 
 *    sequences.
 *
 *    Dans un rpertoire Xxx correspondant  une collection xxx, 
 *    le type xxx est defini dans def_xxx.ml et les operations sur 
 *    ce type sont dfinies dans core_xxx.ml. Le fichier kernel_xxx.ml
 *    contient la declaration dans la librairie MGS des fcts dfinies 
 *    dans xxx.ml
 *
 *    L'organisation de xxx.ml fichier suit l'ordre suivant:
 *
 *       1/ Fonctions de construction: ... -> seq (sauf extension)
 *             cons, join, append, shape, xxxify, iota, extension
 *
 *       2/ Prdicats, accs et observateur
 *             compare, val2bool, size, empty, count, 
 *             member, random, hd, tl, take
 *
 *       3/ Fold et autre itrations
 *             fold, map, iter, foldindexed, mapindexed, iterindexed,
 *             forall, exists, filter
 *
 *       4/ Neighbors, neighbor
 *
 *
 *       5/ Fonctions spcifiques
 *
 *
 *       6/ mise  jour du descripteur du type
 *              champs: make, xxxify et empty
 *
 * Ici :   xxx = gbf
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Environnement;;
open Dispatch;;
open Dispatch_sig;;

__register_version "$RCSfile: kernel_del.ml,v $" "$Revision: 1.2 $" "$Date: 2005-07-12 14:23:11 $";;

Printf.printf "(* loading kernel functions on DEL into user-space ... ";
flush Pervasives.stdout;;


(*!*)


(*** @module delaunay
    The delaunay triangulation collection
*)

(**
  @name 'voronoi
  @type del_bool

  [voronoi(d, true)] returns a sequence of voronoi vertices together
  with the Delaunay's vertices it is adjacent to.

  [voronoi(d, false)] returns a record of (1) the coordinates of the
  voronoi vertices and (2) for each Delaunay vertex, all the Voronoi
  vertices adjacent ot the vertex. The adjacent vertices are in polar
  order (clockwise order).

  Remark that (1) this function can only be used on a Delaunay value     
  that has been defined with the option of carrying Voronoi informations 
  together with the Delaunay triangulation and (2) it is only provided   
  in dimension 2.                                                        
*)
Environnement.addenv_impure "'voronoi" (fun_2 voronoi);;

(**
  @module delaunay
  @name 'area
  @type del

  [area(d)] returns a sequence of values specifying the area of each voronoi cell centered on a Delaunau vertex.

  Remark that (1) this function can only be used on a Delaunay value     
  that has been defined with the option of carrying Voronoi informations 
  together with the Delaunay triangulation and (2) it is only provided   
  in dimension 2.                                                        
*)
Environnement.addenv_impure "'area" (fun_1 voronoiarea);;

(**
  @module collections 
  @name 'delaunayfy 
  @type del_any_del

  Build a delaunay.

  [delaunayfy(d, c)] build a delaunay from an existing delaunay [d]
  and the elements of the collection [c] ([c] is not a record). The
  result has the same type as [d] and its elements is the union of the
  element of [d] and [c].

  If the second argument is a scalar value or a record [s], the
  elements in the result are the elements of [d] plus the element [s].

  @voir Delaunay chapter Delaunay.
*)
addenv_impure "'delaunayfy"   (fun_2 __delaunayfy);;

(* ============================================================ *)

(* --- fin de l'installation des fonctions --- *)

Printf.printf "done *)\n";

(* ======================================================= *)
