/*
 *
 *                              MGS
 *                              LaMI
 *                       Universite d'Evry Val d'Essonne
 *                   Olivier MICHEL, Jean-Louis GIAVITTO
 * File: $RCSfile: stub_del.c,v $ 
 * Path: %P%
 * Description:
 * Created:
 * Author: Olivier Michel & Jean-Louis Giavitto
 * Modified: $Date: 2005-11-23 13:57:10 $ at 
 * Version: $Revision: 1.3 $
 * Status:
 *
 */
/* do_delaunay_triangulation(int nb_elements, int dimension, int **tableau) */


/*
data structures

    Qhull produces the following data structures for dimension d:

        * A coordinate is a real number in floating point format.
        * A point is an array of d coordinates. With option 'QJ', the coordinates are joggled by a small amount.
        * A vertex is an input point.
        * A hyperplane is d normal coefficients and an offset. The length of the normal is one. The hyperplane defines a halfspace. If V is a normal, b is 
          an offset, and x is a point inside the convex hull, then Vx+b <0.
        * An outer plane is a positive offset from a hyperplane. When Qhull is done, all points will be below all outer planes.
        * An inner plane is a negative offset from a hyperplane. When Qhull is done, all vertices will be above the corresponding inner planes.
        * An orientation is either 'top' or 'bottom'. It is the topological equivalent of a hyperplane's geometric orientation.
        * A simplicial facet is a set of d neighboring facets, a set of d vertices, a hyperplane equation, an inner plane, an outer plane, and an orientation. 
          For example in 3-d, a simplicial facet is a triangle.
        * A centrum is a point on a facet's hyperplane. A centrum is the average of a facet's vertices. Neighboring facets are convex if each centrum is below the neighbor
          facet's hyperplane.
        * A ridge is a set of d-1 vertices, two neighboring facets, and an orientation. For example in 3-d, a ridge is a line segment.
        * A non-simplicial facet is a set of ridges, a hyperplane equation, a centrum, an outer plane, and an inner plane. The ridges determine a set
          of neighboring facets, a set of vertices, and an orientation. Qhull produces a non-simplicial facet when it merges two facets together.
          For example, a cube has six non-simplicial facets.

http://www.msri.org/info/computing/docs/qhull/qh-optv.htm
http://www-imk.fzk.de/~kouker/compugeom/qhull/qh-faq.htm#3dd
file:///tmp/qhull-2003.1/html/qhull.htm#outputs
file:///tmp/qhull-2003.1/html/index.htm#structure
file:///tmp/qhull-2003.1/src/index.htm


*/

#include <mlvalues.h>       /* for OCAML */
#include <memory.h>
#include <qhull/qhull_a.h>  /* Available in the libqhull3 and libqhull-dev packages */
#include <assert.h>


// Comme je suis pas sur du hack du code de Qhull, je prevois 
// que certains cas imprevus peuvent arriver...
#define ABORTERROR(m)  {fprintf(stderr, "Unexpected error in module stub_del at point %s\n", m); fflush(stderr); exit(-1); }


CAMLprim value delaunay_stub_usage()
{
  CAMLparam0();
  fprintf(stdout, "(* registering qhull library (http://www.geom.umn.edu/locate/qhull) *)\n");
  fprintf(stdout, "(*                           (http://www.qhull.org/                 *)\n");
  fflush(stdout);
  CAMLreturn(Val_unit);
}

/*
  qh_call_qhull( void )
    template for calling qhull from inside your program
    remove #if 0, #endif to compile
    define: char qh_version[]= "...";

  returns: 
    exit code (see qh_ERR... in qhull.h)
    all memory freed

  notes:
    This can be called any number of times.  

  see:
    qh_call_qhull_once()
    
*/

/* Commente, sinon cela ne passe pas en make static... */
/*  char qh_version[]= "qhull (version 3.1 ?) integrated into MGS"; */



/* dim       : dimension of points                 */
/* numpoints : number of points                    */
/* points    : array of coordinates for each point */

#define DEL_OPT_SIZE 200

CAMLprim value qh_call_qhull(int numpoints, int dim, coordT *points, char *opt_delaunay, boolT with_voronoi)
{
  CAMLparam0();
  CAMLlocal5(caml_delaunay, new_head, tmp_value, caml_ptr, voisin);
  CAMLlocal2(caml_voronoi, caml_open);

  boolT ismalloc;           /* True if qhull should free points in qh_freeqhull() or reallocation */
  char flags[DEL_OPT_SIZE]; /* option flags for qhull, see qh_opt.htm */
  FILE *outfile= NULL;      /* output from qh_produce_output()
			       use NULL to skip qh_produce_output() */
  FILE *errfile= stderr;    /* error messages from qhull code */
  int exitcode;             /* 0 if no error from qhull */
  facetT *facet;	    /* set by FORALLfacets */
  int curlong, totlong;	    /* memory remaining after qh_memfreeshort       */
  int i, idx;
  int nb_voronoi_vertex=0, idx_vor=0;  /* Le nombre de sommets des cellules de Voronoi */

  ismalloc = False;


  if (strlen(opt_delaunay) >= (DEL_OPT_SIZE - 20))          // 20, c'est pour "qhull d " et de la marge !
    ABORTERROR("Optional Delaunay argument too long...")
  else
    sprintf(flags, "qhull d %s", opt_delaunay);

  /*  fprintf(stderr, "La chaine passee a Qhull est --> %s <----\n", flags); */

  if ((with_voronoi == True) && (dim != 2))
    ABORTERROR("You cannot ask for Voronoi informations in dimension <> 2.")

  exitcode= qh_new_qhull (dim, numpoints, points, ismalloc, flags, outfile, errfile); 


  if (!exitcode) 
    {
      vertexT *vertex, **vertexp;                        /* Utilise par l'iterateur                                                    */

      qh_setvoronoi_all();


      if (with_voronoi == True)                        
	{
	  FORALLfacets                                   /* On compte le nombre de facets centrees sur un sommet de Voronoi */
	    if (!facet->upperdelaunay) 
	      nb_voronoi_vertex++;
	                                                 /* On initialise l'array Voronoi au nombre de sommets */
	  caml_voronoi = alloc_tuple(nb_voronoi_vertex);
	  caml_open    = alloc_tuple(numpoints);         // et l'info de cellule de Voronoi ouverte?

	  for (i=0; i < numpoints; i++)                       
	    Store_field(caml_open, i, Val_false);       // Toutes les cellules sont par defaut closes.
	}
      else
	{
	  caml_voronoi = alloc_tuple(0);                 // Si on ne veut pas les informations de Voronoi, on alloue un "tableau" vide
	  caml_open    = alloc_tuple(0);
	}
	                                                 /* On initialise l'array Delaunay au nombre de sommets */
      caml_delaunay = alloc_tuple(numpoints);
      
      for(i=0; i < numpoints; i++)                       /* On le rempli de [] afin qu'au prochain alloc, car on construit la liste */
	Store_field(caml_delaunay, i, Val_int(0));       /* par la tete.                                                            */


      FORALLfacets                                       /* On itere sur les facets du graphe de Delaunay, de centre un sommet du graphe de Voronoi */
	if (!facet->upperdelaunay) 
	  {
	    int 
	      *sommets_sur_facet,
	      nb_sommets_sur_facet = qh_setsize (facet->vertices),      /* Combien de sommets sont voisins pour cette facet */
	      local_idx = 0, inside, k, num = qh hull_dim-1;


	    //	    fprintf(stderr, "Dans Delaunary, pour la facette d'ID %d, on a la coordonnee du sommet du Graphe de Voronoi qui est  : ", facet->id);
	    //	    for (k=0; k < num; k++)
	    //	      fprintf (stderr, qh_REAL_1, facet->center[k]);
	    //	    printf("\n");
	    
  	    sommets_sur_facet = (int *)calloc(nb_sommets_sur_facet, sizeof(int));

	    if (with_voronoi == True)
	      {
		new_head = alloc_tuple(4);            // On alloue le bloc float * float * int list ... FIXME : pkoi 4 et pas 3 ????
		assert(num == 2);
		for (k=0; k < num; k++)
		  Store_field(new_head, k, copy_double(facet->center[k])); // coords
		Store_field(new_head, 2, Val_int(0));        // [] en troisieme position. On va construire la liste par la tete !
	      }
	    
	    FOREACHvertex_(facet->vertices)                                      /* Pour la facet courante, tous les sommets voisins */
	      {
		sommets_sur_facet[local_idx++] = qh_pointid(vertex->point);

		if (with_voronoi == True)
		  {
		    tmp_value = alloc_tuple(2);  // On stocke l'ID des vertex (cad le sommet du graphe de D) de la facette centree sur le sommet de Voronoi
		    Store_field(tmp_value, 0, Val_int(qh_pointid(vertex->point))); 
		    Store_field(tmp_value, 1, Field(new_head, 2));
		    Store_field(new_head,  2, tmp_value);
		  }

		//		fprintf(stderr, "On a comme voisins les sommets d'id %d et de coordonnees", qh_pointid(vertex->point));
		//		for (k=0; k < num; k++)
		//		  fprintf (stderr, qh_REAL_1, vertex->point[k]);
		//		fprintf(stderr, "\n");		
	      }

	    if (with_voronoi == True)
	      {
		Store_field(caml_voronoi, idx_vor, new_head);
		idx_vor += 1;
	      }
	    
	    for(local_idx = 0; local_idx < nb_sommets_sur_facet; local_idx++)  /* Pour tous les voisins sur une facet               */
	      {
		int sommet_courant = sommets_sur_facet[local_idx];

		for(inside = 0 ; inside < nb_sommets_sur_facet; inside ++)       /* Etant donne un sommet, on considere tous les autres */
		  if (inside != local_idx)
		  {
		    boolT pas_dedans = True;

		    caml_ptr = Field(caml_delaunay, sommet_courant);
		    voisin   = Val_int(sommets_sur_facet[inside]);

		    for( ; (caml_ptr != Val_int(0)) && pas_dedans; caml_ptr = Field(caml_ptr, 1)) /* On ne veut pas de doublons */
		      if (Field(caml_ptr, 0) == voisin)
			pas_dedans = False;

		    if (pas_dedans)
		      {
			new_head = alloc_tuple(2);                                         /* Alloue  une cons-cell */
			Store_field(new_head, 0, voisin);
			Store_field(new_head, 1, Field(caml_delaunay, sommet_courant));
			Store_field(caml_delaunay, sommet_courant, new_head);
		      }
		  }
	      }
	    free(sommets_sur_facet);
	  }
	else
	  {
	    // On est pas sur une upperdelaunay, donc on correspond a une face dont les sommets de Delaunay sont dans une cellule de Voronoi ouverte
	    if (with_voronoi == True)
	      {
		FOREACHvertex_(facet->vertices)                                      /* Pour la facet courante, tous les sommets voisins */
		  {
		    //		    fprintf(stderr, "La facette exterieure est d'id %d\n", qh_pointid(vertex->point));
		    //		    fprintf(stderr, "\n");
		    Store_field(caml_open, qh_pointid(vertex->point), Val_true);   // Cellule pour ce sommet de Delaunay ouverte. Val_int(1) \equiv TRUE
		  }
	      }
	  }
    }
  else 
    ABORTERROR("call to qh_new_qhull of the qhull library")

  qh_freeqhull(!qh_ALL);
  qh_memfreeshort (&curlong, &totlong);
  if (curlong || totlong) 
    fprintf (errfile, "qhull internal warning (main): did not free %d bytes of long memory (%d pieces)\n", totlong, curlong);

  tmp_value = alloc_tuple(3);
  Store_field(tmp_value, 0, caml_delaunay);
  Store_field(tmp_value, 1, caml_voronoi);
  Store_field(tmp_value, 2, caml_open);

  CAMLreturn(tmp_value);
}

/*
main()
{
  coordT points[]  = { 2, 2, 5, 6, 8, 2, 10, 7 };
  
  qh_call_qhull(4, 2, points);
}

*/
/* ****************************************************************************************************************************

######                                                              #######						     
#     #  ######  #         ##    #    #  #    #    ##     #   #	       #     #####      #      ##    #    #   ####	     
#     #  #       #        #  #   #    #  ##   #   #  #     # #	       #     #    #     #     #  #   ##   #  #    #	     
#     #  #####   #       #    #  #    #  # #  #  #    #     #	       #     #    #     #    #    #  # #  #  #	     
#     #  #       #       ######  #    #  #  # #  ######     #	       #     #####      #    ######  #  # #  #  ###   ###
#     #  #       #       #    #  #    #  #   ##  #    #     #	       #     #   #      #    #    #  #   ##  #    #   ###
######   ######  ######  #    #   ####   #    #  #    #     #	       #     #    #     #    #    #  #    #   ####    ###

**************************************************************************************************************************** */

/* nb_int              : le nombre de sommets
   dimension_int       : dimension des sommets (doit etre > 2)
   tableau_array_array : un tableau de sommets ; chaque sommet etant un tableau de flottants
*/

CAMLprim value delaunay_neighbors(value nb_int, value dimension_int, value tableau_array_array, value delaunay_opt_string, value with_voronoi_bool)
{
  CAMLparam5(nb_int, dimension_int, tableau_array_array, delaunay_opt_string, with_voronoi_bool);

  CAMLlocal3(base, ptr, ret); 

  int nb = Int_val(nb_int);
  int dimension = Int_val(dimension_int);
  int compteur = 0;
  int base_idx, ptr_idx;

  coordT *points = (coordT *)malloc(nb*dimension*sizeof(coordT));


  for(base = tableau_array_array, base_idx = 0; base_idx < Wosize_val(base); base_idx ++)   /* On itere sur chaque sommet             */
    {
      assert(Wosize_val(base) == nb);                                                       /* verifie que c'est bien une matrice (1) */
      for(ptr = Field(base, base_idx), ptr_idx = 0; ptr_idx < dimension; ptr_idx++)         /* On itere sur chaque dimension          */
	*(points+compteur++) = (coordT)Double_field(ptr, ptr_idx);                          /* Double_field car valeurs non tagues    */
    }

  ret = qh_call_qhull(nb, dimension, points, String_val(delaunay_opt_string), Bool_val(with_voronoi_bool));
  free(points);
  CAMLreturn(ret);
}





