(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: core_graph_generique.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-11-23 11:10:57 $
 * Version: $Revision: 1.2 $
 * -------------------------------------------------------------------
 *
 * Status:  
 *     La representation gnrique d'un graphe (utilis dans les Graphes et les Delaunays)
 *
 *=========================================================================
 *)

open Global;;
open Def_graph_generique;;
open Valeur;;
open Auxfct;;
open Printf;;
open Printer;;
open Dispatch_sig;;

Global.__register_version "$RCSfile: core_graph_generique.ml,v $" "$Revision: 1.2 $" "$Date: 2005-11-23 11:10:57 $";;

(* **************************************************************************************************** *)
(* **************************************************************************************************** *)

(* => pour affichage de l'usage
   <= 
*)
external nauty_usage : unit -> unit = "nauty_stub_usage"

(* => nombre de sommets max
      le tableau des voisins du premier graphe
      le tableau des voisins du second  graphe
   <= int : 0 egaux, -1 ou 1 selon que le premier est > ou <
*)
   
external nauty_compare : int -> (int list) array -> (int list) array -> int = "nauty_compare"

(* **************************************************************************************************** *)
(* **************************************************************************************************** *)


(** alias
let seq_desc = Type_seq.seq_desc
and set_desc = Type_set.set_desc
and bag_desc = Type_bag.bag_desc
;;
 **)

let setify  g fn_val = (Array.fold_right (fun e suite -> Core_set.add (fn_val e) suite) g Core_set.empty)
and bagify  g fn_val = (Array.fold_right (fun e suite -> Core_bag.add_b (fn_val e) suite) g Core_bag.empty_b)
and sequify g fn_val = (Array.map fn_val g)

and count g fn_val arg = (let nb = ref 0
			  in
			    Array.iter (function e -> if (0 = compare_mgs_value (fn_val e) arg) then (incr nb)) g;
			    VAL_int(!nb))

and member g fn_val arg = (try
			     for i = 0 to (Array.length g)-1
			     do
			       if (0 = compare_mgs_value arg (fn_val g.(i)))
			       then raise BreakLoop
			     done;
			     VAL_bool false
			   with BreakLoop -> VAL_bool true)

and hd g fn_val = (if (0 < Array.length g)
		   then (fn_val g.(0))
		   else raise (Erreur "hd : cannot be applied on an empty graph"))

and exists g fn_val f = VAL_bool (Global.array_exists (fun x -> bool_from_val (f (fn_val x))) g)
and forall g fn_val f = VAL_bool (Global.array_forall (fun x -> bool_from_val (f (fn_val x))) g)


and fold         f acces tab zero = Array.fold_right (fun v s -> (app_2 f (acces v) s)) tab zero

and iter fonc fn_val g = (Array.iter (fun x -> fonc (fn_val x); ()) g); VAL_undef "iter"

and iterindexed fonc fn_val g = (Array.iteri (fun idx x -> app_2 fonc (VAL_int idx) (fn_val x); ()) g); VAL_undef "iter"

and remove_one_from_array idx f tab = 
  let len = Array.length tab
  in
    if ((idx >= len) or (idx < 0))
    then raise (Erreur (Printf.sprintf "remove : cannot remove the %dth element of an array %d" idx len))
    else 
      Array.init 
	(len - 1)
	(fun index -> (let src = if (index >= idx) then (index+1) else index 
		       in
		       f tab.(src)))

and last tab fn_val = (let len = Array.length tab
		       in
			 if (len = 0)
			 then raise (Erreur "last : empty array")
			 else fn_val tab.(len - 1))


and compare pval pvois (t1, d1) (t2, d2) = 
  let tdiff = compare_type d1 d2 in 
    if (tdiff <> 0) then tdiff
    else 
      begin
	let len_t1 = Array.length t1
	and len_t2 = Array.length t2 in
	let cmp    = Pervasives.compare len_t1 len_t2
	in if (cmp <> 0) then cmp
	  else
	    begin
	      let res = ref 0 in
		try
		  for i = 0 to len_t1 - 1 do
		    (** comparaisons qu'un sommets a bien les mmes voisins **)
		    res := Pervasives.compare (pvois t1.(i)) (pvois t2.(i));
		    if 0 <> !res  then raise BreakLoop;
		    (** comparaisons de svaleurs portes **)
		    res := compare_mgs_value (pval t1.(i)) (pval t2.(i));
		    if 0 <> !res  then raise BreakLoop;
		  done;
		  0
		with BreakLoop -> !res
	    end
      end


and compare_iso pvois (t1, d1) (t2, d2) = (let tdiff = compare_type d1 d2
					   in 
					     if (tdiff <> 0)
					     then tdiff
					     else (let len_t1 = Array.length t1
						   and len_t2 = Array.length t2 in
						   let cmp    = Pervasives.compare len_t1 len_t2
						   in
						     if (cmp <> 0)
						     then cmp
						     else nauty_compare len_t1 (Array.map pvois t1) (Array.map pvois t2)))
					     
and neighbors pval pvois t i = (assert ((i >= 0) && (i < (Array.length t)));
				let vois       = Array.of_list (pvois t.(i))
				in
				  Array.init (Array.length vois) (fun idx -> pval (t.(vois.(idx)))))
