(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: graph.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-11-23 11:10:57 $
 * Version: $Revision: 1.2 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Dispatch_sig;;
open Core_graph;;

__register_version "$RCSfile: graph.ml,v $" "$Revision: 1.2 $" "$Date: 2005-11-23 11:10:57 $";;


let seq_desc = Type_seq.seq_desc
and set_desc = Type_set.set_desc
and bag_desc = Type_bag.bag_desc
;;


let _size_grf ((_, tab), _)  = VAL_int(Array.length tab)

and _hd_grf ((_, tab), td)   = Core_graph_generique.hd tab deco
and _tl_grf ((tbl, tab), td) = 
  let ntab = Core_graph_generique.remove_one_from_array 
	       0 
	       (fun (d, vois) -> d, (List.fold_right (fun v s -> if (v = 0) then s else v::s) vois [])) tab 
  in
    VAL_grf ((tbl, ntab), td)

and _last_grf ((_, tab), _) = Core_graph_generique.last tab deco

and _exists_funct_grf (f, ((_, tab), _)) = Core_graph_generique.exists tab deco f 
and _forall_funct_grf (f, ((_, tab), _)) = Core_graph_generique.forall tab deco f

and _count_any_grf (a, ((_, tab), _)) = Core_graph_generique.count  tab deco a

and _shape_grf ((tbl, tab), d) = (let taille  = Array.length tab
				  in
				    VAL_funct (function VAL_seq (s', _) -> (if (taille <> Array.length s')
									    then raise (Erreur "shape (graph) : argument is not of the same size of the original")
									    else (let tab_fin = Array.mapi (fun idx v -> v, vois(tab.(idx))) s'
										  in
										    VAL_grf ((tbl, tab_fin), d)))
						 | _                    -> raise (Erreur "shape (graph) : a sequence as argument is required")))


and _member_any_grf (a, ((_, tab), _)) = Core_graph_generique.member  tab deco a

and _compareMgsVal_grf_grf (((_, t1), d1), ((_, t2), d2)) = VAL_int (Core_graph_generique.compare deco vois (t1, d1) (t2, d2))

and _isomorphism_grf_grf (((_, t1), d1), ((_, t2), d2))   = VAL_bool (0 = Core_graph_generique.compare_iso vois (t1, d1) (t2, d2))

and _neighbors_grf_int (((_, t), _), i)                   = VAL_seq ((Core_graph_generique.neighbors deco vois t i), seq_desc)

and _neighborspos_grf_int (((_, t), _), i) = (assert ((i >= 0) && (i < (Array.length t)));
					      let _, voisins = t.(i)
					      in
						VAL_seq ((Array.of_list (List.map (fun x -> VAL_int x) voisins)), seq_desc))


and _map_funct_grf (fonc, ((tbl, tab), deco)) = (let new_tbl = Hash.copy tbl
						 in
						   VAL_grf ((new_tbl, (Array.map (function (v, vois) -> (fonc v), vois )) tab), deco))

and _mapindexed_funct_grf (fonc, ((tbl, tab), deco)) = (let new_tbl = Hash.copy tbl
							in
							  VAL_grf ((new_tbl, (Array.mapi (fun index  (v, vois) -> (app_2 fonc (VAL_int index) v), vois) tab)), deco))

and _iter_funct_grf (fonc, ((_, tab), _))                  = Core_graph_generique.iter        fonc deco tab
and _iterindexed_funct_grf (fonc, ((_, tab), _))           = Core_graph_generique.iterindexed fonc deco tab
and _fold_funct_any_grf (fonc, zero, ((_, tab), _))        = Core_graph_generique.fold        fonc deco tab zero
and _foldindexed_funct_any_grf (fonc, zero, ((_, tab), _)) = (let idx = ref ((Array.length tab) - 1)
							      in
								Array.fold_right (fun v s -> (let local = VAL_int !idx
											      in
												decr idx;
												(app_3 fonc local (deco v) s))) tab zero)


and _setify_grf  ((_, tab), _)         = VAL_set(Core_graph_generique.setify  tab deco, Type_set.set_desc)
and _bagify_grf  ((_, tab), _)         = VAL_bag(Core_graph_generique.bagify  tab deco, Type_bag.bag_desc)
and _sequify_grf ((_, tab), _)         = VAL_seq(Core_graph_generique.sequify tab deco, Type_seq.seq_desc)
and _graphify_grf (x, deco)            = VAL_grf (x, deco)


and _empty_grf   ((_, tab), _)         = VAL_bool (0 = Array.length tab)
;;



