(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: topo_graph.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-03-07 17:24:52 $
 * Version: $Revision: 1.1 $
 * -------------------------------------------------------------------
 *
 * Status: voir le commentaire general dans topology.ml
 *          
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Printer;;
open Topology;;
open Topo_graph_generique;;
open Graph;;

__register_version "$RCSfile: topo_graph.ml,v $" "$Revision: 1.1 $" "$Date: 2005-03-07 17:24:52 $";;



exception FoundIntPos of int;;

class topo_grf ((((tbl, tab) as c) : 'collection), couleur) mgsval =
object(self)
  inherit [mgs_value Def_graph_generique.grf_tab_element] topo_graphe_generique tab c couleur mgsval


  method get_decoration_from_tab = Core_graph.deco
  method get_voisins_from_tab    = Core_graph.vois

  method build_result =
    let res = Array.copy tab in                                         (* Reecriture de sommet pour l'instant => copie le tableau initial *)
    let update p v = res.(p) <- v, (self#get_voisins_from_tab res.(p))  (* et on ne met a jour que les sommets modifies, conservant la liste des voisins*)
    in
      List.iter (function (kind, chemin, rhs) -> 
(*		   Printf.printf "Le chemin est %s\n" (Gbf.clean_comma_string_of_general_list string_of_int chemin); *)
		   let len_filtree, positions = List.fold_right (fun p (len, pos) -> (len+1), (p::pos)) chemin (0, [])
		   and idx = ref 0
		   in
		     match rhs
		     with 
			 VAL_seq (t, _) -> (if (len_filtree = 1)
					    then update (List.hd positions) rhs
					    else 
					      if (len_filtree = Array.length t)
					      then List.iter (function p -> (* Printf.printf "En position %d on place %s\n" p (to_string(prsvalue t.(!idx))); *) 
								update p t.(!idx); incr idx) positions
					      else raise (Erreur (Printf.sprintf "not the same length in graph. Waiting %d, got %d" len_filtree (Array.length t))))

		       | VAL_undef _    -> raise (Erreur "undef in graphs not yet implemented")

		       | _              -> (if ((* (not (is_collection rhs)) & *) (len_filtree = 1))
					    then (update (List.hd positions) rhs)
					    else raise (Erreur (Printf.sprintf "non compatible value in graphs %d : %s" len_filtree (to_string (prsvalue rhs) )))))
	_batch;
      VAL_grf ((tbl, res), couleur)		     
    
end
;;
