(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: topo_prox.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-03-07 17:24:53 $
 * Version: $Revision: 1.1 $
 * -------------------------------------------------------------------
 *
 * Status: voir le commentaire general dans topology.ml
 *          
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Printer;;
open Topology;;
open Topo_graph_generique;;
open Prox;;

__register_version "$RCSfile: topo_prox.ml,v $" "$Revision: 1.1 $" "$Date: 2005-03-07 17:24:53 $";;

module IntSet = Set.Make(struct type t = int let compare=compare end);;

exception FoundIntPos of int;;

class topo_prox (((tab as c) : 'collection), couleur) mgsval =
object(self)
  inherit [mgs_value Def_prox.prox_tab_element] topo_graphe_generique tab c couleur mgsval

  val mutable _fonction  = (fun x -> x) 

  method get_voisins_from_tab    = Core_prox.vois
  method get_decoration_from_tab = Core_prox.deco


  method private un_match (kind, chemin, rhs) (fold_chemin, fold_valeur, fold_len_chemin, fold_nb_valeur) =
    let len_filtree = List.length chemin in
    let liste_valeurs, nb_valeurs = (match rhs
				     with 
					 VAL_seq (t, _) -> (let r = (Array.to_list t)  (* Une sequence represente la liste des elements filtres *)
							    in
							      r, (Array.length t))
					   
				       | VAL_undef _    -> [], 0                       (* Un <undef> represente la suppression d'un element     *)

				       | x              -> (if (len_filtree = 1) 
							    then [x], 1
							    else raise (Erreur "internal error on proximal collections with un_match")))
    in
      (List.fold_right (fun e s -> IntSet.add e s) chemin fold_chemin), (liste_valeurs@fold_valeur), (len_filtree+fold_len_chemin), (nb_valeurs+fold_nb_valeur)

  method build_result =
    let chemins, valeurs, len_chemin, nb_valeurs = List.fold_right self#un_match _batch (IntSet.empty, [], 0, 0) in (* Ensemble des chemins filtres et des valeurs calculees   *)
    let valeurs_tableau = Array.of_list valeurs in
    let taille_initiale = Array.length tab in
    let quantite_finale = (let nombre_non_filtre = taille_initiale - len_chemin   (* Si les chemins representent 12 valeurs, 5 n'ont pas ete filtree *)
			   in
			     nombre_non_filtre + nb_valeurs) in                   (* La taille finale est le nombre de non filtre + le nombre de valeurs produites *)     
    let tab_valeur      = Array.make quantite_finale (VAL_undef "") 
    and base            = ref 0
    and check           = ref 0
    in
      assert((IntSet.cardinal chemins) = len_chemin);
      assert((Array.length valeurs_tableau) = nb_valeurs);
      for i = 0 to (taille_initiale - 1)                                      (* On recupere deja tous ceux qui n'ont pas etes filtres *)
      do
	if (IntSet.mem i chemins)                                             (* Si l'index est dans la liste des chemins, c'est qu'il a ete filtre -> ne rien faire *)
	then ()
	else (tab_valeur.(!base) <- Core_prox.deco (tab.(i)); incr base)       (* Sinon, c'est qu'au contraire il doit etre conserve                                  *)
      done;

      for i = 0 to (nb_valeurs - 1)                                               (* On ajoute ensuite tous les elements qui ont etes produits                           *)
      do
	tab_valeur.(!base + i) <- valeurs_tableau.(i);
	assert(incr check; true)
      done;
      assert(!base + !check = quantite_finale);
      Core_prox.proximal_from_all_sommets tab_valeur _fonction couleur


  initializer 
  let fonc = Core_prox.get_defs_from_proximal couleur.name
  in
    _fonction  <- fonc

end
;;
