(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: gsl_random.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2008-01-15 09:08:39 $
 * Version: $Revision: 1.3 $
 * -------------------------------------------------------------------
 *
 * Status: ajout a partir de la version 1.23 de Doligez de la lib ocaml
 *         pour avoir la taille de la hashtbl + fct supplementaires
 * 
 * 
 *
 *=========================================================================
 *)

open Gsl_rng;;
open Gsl_qrng;;
open Array;;

Global.__register_version "$RCSfile: gsl_random.ml,v $" "$Revision: 1.3 $" "$Date: 2008-01-15 09:08:39 $";;


print_string "(* loading and initialisation of GSL funs into user space ... ";
flush Pervasives.stdout;;

(* -------------------------------------------------------------------------------- *)
(* Initialisation *)

(* *** *)
(* RNG *)
(* *** *)

let _ = 
  Gsl_error.init () ;
  Gsl_rng.env_setup () (* lis dans l'environnement la valeur des variables *)
;;                     (* GSL_RNG_TYPE et GSL_RNG_SEED pour definir        *)
                       (* le type de RNG et son seed                       *)

(*
On a droit a

           borosh13               cmrg            coveyou          fishman18
          fishman20          fishman2x              gfsr4           knuthran
          knuthran2       knuthran2002          lecuyer21             minstd
                mrg            mt19937       mt19937_1999       mt19937_1998
               r250               ran0               ran1               ran2
               ran3               rand             rand48      random128-bsd
   random128-glibc2    random128-libc5      random256-bsd   random256-glibc2
    random256-libc5       random32-bsd    random32-glibc2     random32-libc5
       random64-bsd    random64-glibc2     random64-libc5        random8-bsd
     random8-glibc2      random8-libc5         random-bsd      random-glibc2
       random-libc5              randu               ranf             ranlux
          ranlux389            ranlxd1            ranlxd2            ranlxs0
            ranlxs1            ranlxs2             ranmar             slatec
               taus              taus2            taus113         transputer
              tt800                uni              uni32                vax
         waterman14                zuf
*)



let rng = Gsl_rng.make (Gsl_rng.default ());; (* Par defaut, si rien n'est specifie c'est le Mersenne Twister MT19937 *)


(*
let _ = 
  Printf.printf "# generator type: %s\n" 
    (Gsl_rng.name rng) ;
  Printf.printf "# seed = %nu\n" 
    (Gsl_rng.default_seed ()) ;
  Gsl_rng.set_default_seed (Nativeint.of_int 120);
    Printf.printf "# set seed to 120\n"; 
  Printf.printf "# seed = %nu\n" 
    (Gsl_rng.default_seed ()) ;
  Printf.printf "# min value   = %nu\n"  
    (Gsl_rng.min rng) ;
  Printf.printf "# max value   = %nu\n"  
    (Gsl_rng.max rng) ;
  Printf.printf "# first value = %nu\n"  
    (Gsl_rng.get rng)
;;
*)


let rng_name ()          = Gsl_rng.name rng
and rng_default_seed     = Gsl_rng.default_seed
and rng_set_default_seed = Gsl_rng.set_default_seed (* Par defaut, si c'est 0, c'est celui de la def originelle de l'algo *)
and rng_min ()           = Gsl_rng.min rng
and rng_max ()           = Gsl_rng.max rng
and rng_get ()           = Gsl_rng.get rng          (* un entier dans [min, max] *)
and rng_uniform ()       = Gsl_rng.uniform rng      (* un flottant dans [0, 1 [  *)
and rng_uniformpos  ()   = Gsl_rng.uniform_pos rng  (* un flottant dans ]0, 1[   *)
and rng_uniformint       = Gsl_rng.uniform_int rng  (* un entier dans [0, i-1]   *)


(* **** *)
(* QRNG *)
(* **** *)

let local_random_tbl = (Hashtbl.create 9  : (int, (Gsl_qrng.t * (float array))) Hashtbl.t);;
let local_random_nb  = ref 0;;

let make nom dim = (let t = Gsl_qrng.make (if (((String.uppercase nom) = "NIEDERREITER2") && (dim > 0) && (dim <= 12))
					   then NIEDERREITER_2
					   else 
					     if (((String.uppercase nom) = "SOBOL") && (dim > 0) && (dim <= 40))
					     then SOBOL
					     else raise Not_found) dim
		    in
		      Hashtbl.add local_random_tbl !local_random_nb (t, (Array.make dim 0.0));
		      incr local_random_nb;
		      !local_random_nb - 1)

(* Raises Not_found if the argument is not an existing t *)
and init i = (let (t, _) = Hashtbl.find local_random_tbl i in
	      let _      = Gsl_qrng.init t
	      in
		())

(* Raises Not_found if the argument is not an existing t *)
and get i = (let (t, a) = Hashtbl.find local_random_tbl i
	     in
	       Gsl_qrng.get t a;
	       a)

(* Raises Not_found if the argument is not an existing t *)
and sample i = (let (t, _) = Hashtbl.find local_random_tbl i
		in
		  Gsl_qrng.sample t)

(* Raises Not_found if the argument is not an existing t *)
and name i = (let (t, _) = Hashtbl.find local_random_tbl i
	      in
		Gsl_qrng.name t)

(* Raises Not_found if the argument is not an existing t *)
and dimension i = (let (t, _) = Hashtbl.find local_random_tbl i
		   in
		     Gsl_qrng.dimension t)
;;

(* -------------------------------------------------------------------------------- *)
print_string "done *)";;
print_newline();;
