(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: kernel_qmf.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG, AS
 * Modified: $Date: 2005-09-07 18:37:43 $
 * Version: $Revision: 1.8 $
 * -------------------------------------------------------------------
 *
 * Status: Cf. Qmf/def_qmf.ml pour les infos sur les types qmf et ncell
 *          
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Environnement;;
open Dispatch;;

__register_version "$RCSfile: kernel_qmf.ml,v $" "$Revision: 1.8 $" "$Date: 2005-09-07 18:37:43 $";;

print_string "(* loading kernel functions on QMF into user-space ... ";
flush Pervasives.stdout;;

(*!*)

(***
  @module qmf
  The qmf collection (chain on ncell)
*)


(**
  @module qmf
  @name 'new_gmap
  @type unit_gmap

  Creates a new empty gmap.
  [new_gmap()] creates a new empty gmap [g].
*)
addenv_impure "'new_gmap" (VAL_funct (fun _ -> VAL_gmap(Core_qmf.create_gmap())));;

(**
  @module qmf
  @name 'copy_gmap
  @type gmap_gmap

  Returns a copy of a gmap.
  [copy_gmap(g)] returns a copy of the gmap [g].
*)
addenv_impure "'copy_gmap" (fun_1 __copyGmap);;

(**
  @module qmf
  @name 'get_gmap_from
  @type ncell_gmap
  @type qmf_gmap

  Returns the gmap of a ncell or a qmf.
  [get_gmap_from(x)] returns the gmap containing the ncell [x], or the gmap the qmf [x] is built on.
*)
addenv_impure "'get_gmap_from" (fun_1 __getGmapFrom);;

(**
  @module qmf
  @name 'get_coord_from_vertex
  @type ncell_seq

  Returns the embedding of a vertex in libgmapkernel.
  [get_coord_from_vertex(v)] returns the sequence [(x,y,z)] of float that is the embedding
  associated to [v] in libgmapkernel. This embedding is usefull to manipulate ASCCII exchange
  formted files of Moka.

  @voir 'save_moka save_moka
  @voir 'load_moka load_moka
*)
addenv_impure "'get_coord_from_vertex" (fun_1 __getCoordFromVertex);;

(**
  @module qmf
  @name 'set_coord_from_vertex
  @type ncell_seq

  Modifys the embedding of a vertex in libgmapkernel.
  [set_coord_from_vertex(v,s)] updates the embedding associated to the vertex [v]. The new value is
  given by the sequence [s] ; [s] must be a sequence of 3 numbers (int or float) like [(x,y,z)].
  Theses value represent the coordinates of s in a 3-dimensionnal Euclidean space. This embedding
  is usefull to manipulate ASCCII exchange formted files of Moka.

  @voir 'save_moka save_moka
  @voir 'load_moka load_moka
*)
addenv_impure "'set_coord_of_vertex" (fun_2 __setCoordOfVertex);;


(**
  @module qmf
  @name 'load_moka
  @type gmap_string_seq

  Loads in a gmap an ASCII Moka file
*)
addenv_impure "'load_moka" (fun_2 __loadMoka);;

(**
  @module qmf
  @name 'save_moka
  @type gmap_string

  Saves a gmap in an ASCII Moka file
*)
addenv_impure "'save_moka" (fun_2 __saveMoka);;

addenv_impure "'normal_form" (fun_1 __normalForm);;

addenv_impure "'norm" (fun_1 __norm);;

(**
  @module qmf
  @name 'add_ncell
  @type gmap_int_ncell

  Adds a new ncell.
  [add_ncell(g, n)] creates a new ncell [c] of dimension [n] in the gmap [g].
*)
addenv_impure "'add_ncell" (fun_2 __addNcell);;

(**
  @module qmf
  @name 'add_vertex
  @type gmap_ncell

  Adds a new vertex.
  [add_vertex(g)] creates a new ncell [c] of dimension 0 in the gmap [g].
*)
addenv_impure "'add_vertex"   (fun_1 __addVertex);;

(**
  @module qmf
  @name 'add_edge
  @type gmap_ncell

  Adds a new edge.
  [add_edge(g)] creates a new ncell [c] of dimension 1 in the gmap [g].
*)
addenv_impure "'add_edge"   (fun_1 __addEdge);;

(**
  @module qmf
  @name 'add_face
  @type gmap_ncell

  Adds a new face.
  [add_face(g)] creates a new ncell [c] of dimension 2 in the gmap [g].
*)
addenv_impure "'add_face"   (fun_1 __addFace);;

(**
  @module qmf
  @name 'add_volume
  @type gmap_ncell

  Adds a new volume.
  [add_volume(g)] creates a new ncell [c] of dimension 3 in the gmap [g].
*)
addenv_impure "'add_volume" (fun_1 __addVolume);;

(**
  @module qmf
  @name 'add_polygon
  @type gmap_int_ncell

  Adds a new polygon.
  [add_polygon(g, n)] creates a new polygon [c] with [n] sides in the gmap [g].
*)
addenv_impure "'add_polygon" (fun_2 __addPolygon);;

(**
  @module qmf
  @name 'add_sphere
  @type gmap_int_int_seq

  Adds a new sphere.
  [add_sphere(g, m, p)] creates a new sphere [s] with [m] meridians ans [p] parallels in the gmap [g].
*)
addenv_impure "'add_sphere" (fun_3 __addSphere);;

(**
  @module qmf
  @name 'add_square
  @type gmap_int_int_int_ncell

  Adds a new square.
  [add_square(g, ax, ay, dim)] creates a new eventually meshed square. [ax] is the number of subdivions along the
  first dimension, and [ay] the number of subdivions along the second one. [dim] (= 0 or 2) is the dimension of the
  mesh ; if [dim] is null, the values [ax] and [ay] don't matter.
*)
addenv_impure "'add_square" (fun_4 __addSquare);;

(**
  @module qmf
  @name 'add_polyline
  @type gmap_int_ncell

  Adds a new polyline.
  [add_polyline(g, n)] creates a new polyline [c] with [n] verteces in the gmap [g].
*)
addenv_impure "'add_polyline" (fun_2 __addPolyline);;

(**
  @module qmf
  @name 'insert_vertex
  @type ncell_seq

  Splits an edge into two edges separated by a new vertex.
  [insert_vertex(e)] splits into two edges [e1] and [e2] separated by a new vertex v, and returns
  the sequence [(v,e1,e2)].
*)
addenv_impure "'insert_vertex" (fun_1 __insertVertex);;

(**
  @module qmf
  @name 'insert_edge
  @type ncell_ncell_seq

  Splits a face into two faces separated by a new edge.
  [insert_edge(v1,v2)] splits into two faces [f1] and [f2] separated by a new vertex [e] between [v1]
  and [v2] ; theses vertices must belong to the same face. Then it returns the sequence
  [(e,v1,v2,f1,f2)] where [v1] and [v2] have been updated.
*)
addenv_impure "'insert_edge" (fun_2 __insertEdge);;

(**
  @module qmf
  @name 'insert_face
  @type ncell_seq_seq

  Splits a volume into two volumes separated by a new face.
  [insert_face(v,(e1,e2,...,en))] splits the volume [v] into two volumes [v1] and [v2] separated by
  a new face [f] whose faces are the sequence [(e1,e2,...,en)] ; theses edges must belong to the
  same volume. Then it returns the sequence [(f,v1,v2,e1,e2,...en)] where [e1,e2,...,en] have been
  updated.
*)
addenv_impure "'insert_face" (fun_2 __insertFace);;

(**
  @module qmf
  @name 'create_edge
  @type ncell_ncell_seq

  Links 2 vertices by a new edge when it's possible.
  [create_edge(v1,v2)] creates a new edge between [v1] and [v2] when it's possible.
  Then it returns the sequence [(e,v1,v2)] where [v1] and [v2] have been updated.
*)
addenv_impure "'create_edge" (fun_2 __createEdge);;

(**
  @module qmf
  @name 'create_face 
  @type seq_seq

  Creates a new face from given indepdendant edges.
  [create_face(((v1,v2)::(v3,v4)::...::(vn-1,vn+1)::seq:()))] creates the face whose faces
  are the following edges : [(v2,v3)], [(v4,v5)], ..., [(vn,v1)]. Theses edges, defined here
  by their vertices, must exist in the gmap and be alpha-2 and alpha-3 free. Note of the vertices
  in the sequence is important to create the face.
*)
addenv_impure "'create_face" (fun_1 __createFace);;
  
(**
  @module qmf
  @name 'create_volume
  @type seq_unit

  ???
*)
addenv_impure "'create_volume" (fun_2 __createVolume);;

(**
  @module qmf
  @name 'remove_ncell
  @type ncell_ncell

  Remove a ncell.
  [remove_ncell(c)] removes a ncell of dimension 0, 1 or 2 that has exactly 2 cofaces. This remove
  is done by merging the 2 cofaces. It returns the newly created face. This operation is the reverse
  one of insert_(vertex|edge|face).

  @voir 'insert_vertex insert_vertex
  @voir 'insert_edge insert_edge
  @voir 'insert_face insert_face
*)
addenv_impure "'remove_ncell" (fun_1 __removeNcell);;
  
(**
  @module qmf
  @name 'delete_ncell
  @type ncell_seq

  Deletes a ncell.
  [delete_ncell(c)] deletes a ncell of dimension 1, 2 or 3 that has exactly 1 coface. This deletion
  is done by searching all faces and separating them to the volume not exist. At the end, each
  face is independant of the others. It returns the sequence of faces. This operation is the reverse
  one of create_(edge|face).

  @voir 'create_edge create_edge
  @voir 'create_face create_face
*)
addenv_impure "'delete_ncell" (fun_1 __deleteNcell);;
  
(**
  @module qmf
  @name 'glue2FacesBy4Verteces
  @type ncell_ncell_ncell_ncell_ncell_ncell

  ???
*)
addenv_impure "'glue2FacesBy4Verteces" (fun_6 __glue2FacesBy4Verteces);;

(* --- fin de l'installation des fonctions --- *)

print_string "done *)";
print_newline();;

(* ======================================================= *)
