(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: ptopo_qmf.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG, AS
 * Modified: $Date: 2005-09-30 14:26:50 $
 * Version: $Revision: 1.8 $
 * -------------------------------------------------------------------
 *
 * Status: voir le commentaire general dans topology.ml
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Ptopology;;
open Def_qmf;;
open Core_qmf;;
open Printer_qmf;;

__register_version "$RCSfile: ptopo_qmf.ml,v $" "$Revision: 1.8 $" "$Date: 2005-09-30 14:26:50 $";;


exception FoundPos of ncell ;;

let in_place_shuffle_array len a = 
  let nb  = ref (if (len <= 1)
		 then 0
		 else 
		   if (len = 2) 
		   then 1 
		   else len) (* Pour eviter que sur un tableau de longueur 2 on fasse 2 permutations inverses. *)
  in                         (* Le risque est moins grand sur un tableau de taille superieure.                 *)
    while (!nb > 0)
    do
      let src = Random.int len
      and dst = Random.int len
      in
	if (src = dst)
	then ()
	else (let tmp = a.(src)
	      in
		a.(src) <- a.(dst);
		a.(dst) <- tmp;
		decr nb)
    done;
    a
;;

type lattice_node =
    New of string * int * (int list ref) * (int list ref) * int * (mgs_value ref)
  | Old of ncell  * int * (int list ref) * (int list ref) * int * (mgs_value ref)
;;

let affiche_lnode = function
  | New(id,_,lf,lcf,pos,_) ->
      Printf.printf "\tNew: %s, [ " id ;
      List.iter (fun i -> Printf.printf "%i " i) (!lf);
      Printf.printf "], [ " ;
      List.iter (fun i -> Printf.printf "%i " i) (!lcf);
      Printf.printf "] @ %i\n" pos 
  | Old(id,_,lf,lcf,pos,_) ->
      Printf.printf "\tOld: %s, [ " (string_of_ncell id) ;
      List.iter (fun i -> Printf.printf "%i " i) (!lf);
      Printf.printf "], [ " ;
      List.iter (fun i -> Printf.printf "%i " i) (!lcf);
      Printf.printf "] @ %i\n" pos 
;;

let affiche_llnode l = List.iter affiche_lnode l ; Printf.printf "\n" ;;

let rec find_new id l = match l with
  | [] -> raise Not_found
  | (New (s, dim, rlf, rlcf, pos, rval))::q ->
      if (s = id) then
	(pos,rlf,rlcf,rval)
      else
	find_new id q
  | _::q -> find_new id q
;;

let rec find_old id l = match l with
  | [] -> raise Not_found
  | (Old (s, dim, rlf, rlcf, pos, rval))::q ->
      if (s = id) then
	(pos,rlf,rlcf,rval)
      else
	find_old id q
  | _::q -> find_old id q
;;

let rec add_without_double p l = match l with
    [] -> [p]
  | t::q -> if (t=p) then l else t::(add_without_double p q)
;;

let rec inter l1 l2 = match l1 with
  | [] -> []
  | t::q -> if (List.mem t l2) then t::(inter q l2) else inter q l2
;;

let include_new lattice dim id =
  let (pos,lat) = lattice.(dim) in
    try
      find_new id lat
    with Not_found ->
      let rlf = ref [] and rlcf = ref [] and rval = ref (VAL_undef "") in
	lattice.(dim) <- (pos+1,(New (id, dim, rlf, rlcf, pos, rval))::lat) ;
	(pos,rlf,rlcf,rval)
;;

let include_old lattice dim c =
  let c = normal_form c in
  let (pos,lat) = lattice.(dim) in
    try
      find_old c lat
    with Not_found ->
      let rlf = ref [] and rlcf = ref [] and rval = ref (VAL_undef "") in
	lattice.(dim) <- (pos+1,(Old (c, dim, rlf, rlcf, pos, rval))::lat) ;
	(pos,rlf,rlcf,rval)
;;

exception Dart_found of dart ;;

class ptopo_qmf ((c : 'collection), couleur) mgsval =
object(self)
  inherit [ncell] generic_ptopo 
    string_of_ncell                   (** afficheur de position    **)
    (c, couleur)                      (** collection a transformer **)
    mgsval                            (** l'argument complet       **)
    (Core_qmf.get_gmap_from_qmf c,Core_qmf.dart_null (), -1)

  val _avl  = c
  val _avl_size = Core_qmf.size c
  val _avl_gmap = get_gmap_from_qmf c
  val _avl_usage = (let tbl = Core_qmf.create_qmf (Core_qmf.get_gmap_from_qmf c) (Core_qmf.size c)
		    in
		      Core_qmf.iter (fun k v -> Core_qmf.add tbl k PosFree) c;
		      tbl)
  val mutable _batch = ([] : (path_type * 'position path * mgs_value) list)
  val mutable _last_index = Global.crandom (Core_qmf.size c)


  method common_position fl cfl tfl tcfl = [||]

  method iter_all fct = Core_qmf.iter (fun k _ -> (fct k)) c
  method iter_faces fct pos = List.iter
				(fun k -> if (Core_qmf.mem c k) then (fct k))
				(Core_qmf.cofaces pos)
  method iter_cofaces fct pos = List.iter
				  (fun k -> if (Core_qmf.mem c k) then (fct k))
				  (Core_qmf.faces pos)

  method isFaceOf p1 p2 = List.mem (normal_form p1) (List.map normal_form (Core_qmf.faces p2))
  method isCoFaceOf p1 p2 = List.mem (normal_form p1) (List.map normal_form (Core_qmf.cofaces p2))
	  
  method use_transitive_faces_cofaces s =
    let tf  = Core_qmf.transitive_faces s in
    let tcf = Core_qmf.transitive_cofaces s in
    let ret = ref [] in
      List.iter (fun f -> try if (self#usable f) then self#use s f ; ret := f::!ret with Not_found -> ()) tf;
      List.iter (fun cf -> try if (self#usable cf) then self#use s cf ; ret := cf::!ret with Not_found -> ()) tcf;
      !ret

  method dimension s = (get_dim_from_ncell s)

  method state s = Core_qmf.find _avl_usage s
  method update_state f s = Core_qmf.replace _avl_usage s (f (Core_qmf.find _avl_usage s))
  method next_usable = (Core_qmf.get_gmap_from_qmf c, dart_null (), -1 )
  method next_filterable = (Core_qmf.get_gmap_from_qmf c, dart_null (), -1 )
  method unvisit_all =
    Core_qmf.iter (fun s v -> if (pos_visited v) then (Core_qmf.replace _avl_usage s (pos_unvisit v))) _avl_usage

  method get_value_from_pos p = Core_qmf.find _avl p
  method mgs_value_from_pos p = VAL_ncell(p)
  method pos_from_mgs_value m = match m with | VAL_ncell(p) -> p | _ -> raise (Erreur "ptopoQMF: pos_from_mgs_value: wait for a ncell")

  val mutable _mark_f = None
  method get_mark_f = match _mark_f with
    | None -> _mark_f  <- Some (getNewMark(_avl_gmap)) ; self#get_mark_f
    | Some m -> m

  val mutable _mark_local_f = None
  method get_mark_local_f = match _mark_local_f with
    | None -> _mark_local_f  <- Some ([|getNewMark(_avl_gmap);getNewMark(_avl_gmap);getNewMark(_avl_gmap);getNewMark(_avl_gmap)|]) ;
	self#get_mark_local_f
    | Some m -> m

  val mutable _mark_u = None
  method get_mark_u = match _mark_u with
    | None -> _mark_u  <- Some ([|getNewMark(_avl_gmap);getNewMark(_avl_gmap);getNewMark(_avl_gmap);getNewMark(_avl_gmap)|]) ;
	self#get_mark_u
    | Some m -> m

  method add_match rt (p_f,p_u) v =
    _batch <- (rt, (p_f,p_u), v)::_batch

  val mutable _copieDI = None
  method set_copieDI p = _copieDI <- Some p
  method get_copieDI = match _copieDI with
      Some p -> p
    | None -> raise (Erreur "Ptopo_qmf: get_copieDI: this DirectInfo fields doesn't exist")

  val _assoc_ncell_dartlist = Hash.create (Core_qmf.size c)
  method set_assoce (pos:'position) (ds:dart list) = Hash.replace _assoc_ncell_dartlist pos ds
  method get_assoce (pos:'position) =
    try
      Hash.find _assoc_ncell_dartlist pos
    with
	Not_found -> []
  method add_assoce (pos:'position) (d:dart) =
    assert begin if !trace_opt then (
      Printf.printf "################### add_assoce: %i %s\n" (Hash.hash d) (string_of_ncell pos);
      flush stdout);
      true
    end;
    self#set_assoce pos (d::(self#get_assoce pos))


  method include_with_tf lattice dim c (pos,rlf,rlcf,rval) =
    let lf = faces c
    and lcf = cofaces c in
      List.iter
	(fun f ->
	   let fflag = isMarkedNcell f (self#get_mark_local_f).(get_dim_from_ncell f) in
	   let uflag = isMarkedNcell f (self#get_mark_u).(get_dim_from_ncell f) in
	     if ((not fflag)) then (*|(fflag && uflag)) then*)
	       let (f_pos,f_rlf,f_rlcf,f_rval) = include_old lattice (dim-1) f in
		 self#include_with_tf lattice (dim-1) f (f_pos,f_rlf,f_rlcf,f_rval);
		 rlf := add_without_double  f_pos (!rlf) ;
		 f_rlcf := add_without_double pos (!f_rlcf)
	)
	lf ;
      List.iter
	(fun cf ->
	   try
	     let (cf_pos,cf_rlf,cf_rlcf,cf_rval) = find_old cf (snd (lattice.(dim+1))) in
	       rlcf := add_without_double  cf_pos (!rlcf) ;
	       cf_rlf := add_without_double pos (!cf_rlf)
	   with Not_found -> ()
	)
	lcf

  method include_with_tcf lattice dim c (pos,rlf,rlcf,rval) =
    let lcf = cofaces c
    and lf = faces c in
      List.iter
	(fun cf ->
	   let fflag = isMarkedNcell cf (self#get_mark_local_f).(get_dim_from_ncell cf) in
	   let uflag = isMarkedNcell cf (self#get_mark_u).(get_dim_from_ncell cf) in
	     if ((not fflag)) then (*|(fflag && uflag)) then*)
	       let (cf_pos,cf_rlf,cf_rlcf,cf_rval) = include_old lattice (dim+1) cf in
		 self#include_with_tcf lattice (dim+1) cf (cf_pos,cf_rlf,cf_rlcf,cf_rval);
		 rlcf := add_without_double  cf_pos (!rlcf) ;
		 cf_rlf := add_without_double pos (!cf_rlf)
	)
	lcf ;
      List.iter
	(fun f ->
	   try
	     let (f_pos,f_rlf,f_rlcf,f_rval) = find_old f (snd (lattice.(dim-1))) in
	       rlf := add_without_double  f_pos (!rlf) ;
	       f_rlcf := add_without_double pos (!f_rlcf)
	   with Not_found -> ()
	)
	lf

  method include_old_with_tfcf lattice dim c =
    let (pos,rlf,rlcf,rval) = include_old lattice dim c in
      self#include_with_tf lattice dim c (pos,rlf,rlcf,rval);
      self#include_with_tcf lattice dim c (pos,rlf,rlcf,rval);
      (pos,rlf,rlcf,rval)

  method include_with_fcf lattice dim c (pos,rlf,rlcf,rval) =
    let lcf = cofaces c
    and lf = faces c in
      List.iter
	(fun cf ->
	   try
	     let (cf_pos,cf_rlf,cf_rlcf,cf_rval) = find_old (normal_form cf) (snd (lattice.(dim+1))) in
	       rlcf := add_without_double  cf_pos (!rlcf) ;
	       cf_rlf := add_without_double pos (!cf_rlf)
	   with Not_found -> ()
	)
	lcf;
      List.iter
	(fun f ->
	   try
	     let (f_pos,f_rlf,f_rlcf,f_rval) = find_old (normal_form f) (snd (lattice.(dim-1))) in
	       rlf := add_without_double  f_pos (!rlf) ;
	       f_rlcf := add_without_double pos (!f_rlcf)
	   with Not_found -> ()
	)
	lf

  method include_with_ic lattice dim c =
    let ics = Core_qmf.icells c dim in
      List.iter
	(fun ic ->
	   let fflag = isMarkedNcell ic (self#get_mark_local_f).(dim) in
	   let uflag = isMarkedNcell ic (self#get_mark_u).(dim) in
	     if ((not fflag)) then (*|(fflag && uflag)) then*)
	       let (pos,rlf,rlcf,rval) = include_old lattice dim ic in
		 self#include_with_fcf lattice dim ic (pos,rlf,rlcf,rval)
	)
	ics

  method include_old_with_aic lattice dim c =
      let (pos,rlf,rlcf,rval) = include_old lattice dim c in
	self#include_with_ic lattice 0 c;
	self#include_with_ic lattice 1 c;
	self#include_with_ic lattice 2 c;
	self#include_with_ic lattice 3 c;
	(pos,rlf,rlcf,rval)

  method find_dart d0 d1 d2 d3 =
    let cov = ncell_coverage d0 in
      try
	while (cont_coverage cov) do
	  let d = current_dart cov in
	    if(not( isMarkedDart _avl_gmap d self#get_mark_f )
	       &(normal_form (_avl_gmap,d,1) = d1 )
	       &(normal_form (_avl_gmap,d,2) = d2 )
	       &(normal_form (_avl_gmap,d,3) = d3 ))
	    then
	      raise (Dart_found d) ;
	    next_dart cov;
	done ;
	delete_coverage cov ;
	raise Not_found
      with Dart_found d -> getDirectInfo _avl_gmap d self#get_copieDI;

  method common_darts gmap d0 d1 d2 =
    let dim0 = get_dim_from_ncell d0
    and dim1 = get_dim_from_ncell d1
    and dim2 = get_dim_from_ncell d2
    and ds0 = self#get_assoce d0
    and ds1 = self#get_assoce d1
    and ds2 = self#get_assoce d2
    and m0 = getNewMark gmap
    and m1 = getNewMark gmap
    and ret = ref [] in
      List.iter (fun b -> if (not (isMarkedDart gmap b m0)) then markNcell (gmap,b,dim0) m0) ds0 ;
      List.iter (fun b -> if (not (isMarkedDart gmap b m1)) then markNcell (gmap,b,dim1) m1) ds1 ;
      List.iter
	(function b ->
	   let c = (gmap,b,dim2) in
	   let cov = ncell_coverage c in
	     while (cont_coverage cov) do
	       let d = current_dart cov in
		 if ((isMarkedDart gmap d m0) && (isMarkedDart gmap d m1)) then
		   ret := add_without_double d (!ret) ;
		 next_dart cov;
	     done ;
	     delete_coverage cov
	)
	ds2 ;
      List.iter (fun b -> if (isMarkedDart gmap b m0) then unmarkNcell (gmap,b,dim0) m0) ds0 ;
      List.iter (fun b -> if (isMarkedDart gmap b m1) then unmarkNcell (gmap,b,dim1) m1) ds1 ;
      freeMark gmap m0;
      freeMark gmap m1;
      !ret ;

  method add_one_instance gmap ret (rt, (p_f,p_u), v) =
    match v with
      | VAL_seq(a,t) when (t == Type_seq.pRhsSpec_desc) -> (
	  (* On marque les cellules filtres et utilises *)
	  let mark_used = self#get_mark_u in
	    List.iter (fun f -> markNcell f mark_used.(get_dim_from_ncell f) ) p_u;
	  let rhs = self#trad a in
	  let lattice = Array.make 4 (0,[]) in
	    List.iter
	      (function
		 | Filt (cell,s_v) -> ()
		 | (Symb (id, (dim, lf, lcf), s_v)) as s ->
		     let (pos,rlf,rlcf,rval) = include_new lattice dim id in
		       rval := s_v ;

		       (* On ajoute ensuite les faces et cofaces... *)
		       if (dim <> 0) then
			 List.iter
			   (function
			      | VAL_symbol(fname) ->
				  let (f_pos,f_rlf,f_rlcf,f_rval) = include_new lattice (dim-1) fname in
				    f_rlcf := add_without_double pos (!f_rlcf) ;
				    rlf := add_without_double  f_pos (!rlf)
			      | VAL_ncell(c) ->
				  let (f_pos,f_rlf,f_rlcf,f_rval) = self#include_old_with_aic lattice (dim-1) c in
				    f_rlcf := add_without_double pos (!f_rlcf) ;
				    rlf := add_without_double  f_pos (!rlf)
			      | _ -> ()
			   )
			   lf ;
		       if (dim <> 3) then
			 List.iter
			   (function
			      | VAL_symbol(fname) ->
				  let (f_pos,f_rlf,f_rlcf,f_rval) = include_new lattice (dim+1) fname in
				    f_rlf := add_without_double pos (!f_rlf) ;
				    rlcf := add_without_double f_pos (!rlcf)
			      | VAL_ncell(c) ->
				  let (f_pos,f_rlf,f_rlcf,f_rval) = self#include_old_with_aic lattice (dim+1) c in
				    f_rlf := add_without_double pos (!f_rlf) ;
				    rlcf := add_without_double f_pos (!rlcf)
			      | _ -> ()
			   )
			   lcf
	      )
	      rhs ;

	    assert begin if !trace_opt then (
	      Array.iteri (fun i (nb_elt, l) ->
			     Printf.printf "Dim %i: %i elt(s)\n" i nb_elt ;
			     affiche_llnode l ;
			     Printf.printf "\n"
			  ) lattice ;
	      flush stdout );
	      true
	    end;

	    (* On transforme le treillis en tableau de tableaux... *)

	    let lat = Array.map (fun (_,l) -> (Array.of_list (List.rev l))) lattice in

	    (* On construit les chemins du treillis, ie les brins... *)
	    let chem = ref [] in

	      (*Array.iteri (fun i v -> chem := (i,[[i]])::!chem ) lat.(3) ;*)
	      for d = 3 downto 0 do
		let chem_coface = !chem in
		  chem := [] ;
		  Array.iteri
		    (fun i c ->
		         match c with
			 | Old(_,0,rlf,rlcf,_,_) when (!rlcf = []) -> ()
			 | Old(_,1,rlf,rlcf,_,_) when ((!rlf = [])||(!rlcf = [])) -> ()
			 | Old(_,2,rlf,rlcf,_,_) when ((!rlf = [])||(!rlcf = [])) -> ()
			 | Old(_,3,rlf,rlcf,_,_) when (!rlf = []) -> ()
			 | _ ->
			     let rlcf = match c with
				 New(_,_,_,rlcf,_,_) -> rlcf
			       | Old(_,_,_,rlcf,_,_) -> rlcf
			     in
			       match !rlcf with
				 | [] -> chem := (i,[[i]])::!chem
				 | _ ->
				     chem := (i,List.fold_left
						(fun acc cf ->
						   try
						     (List.map
							(fun ch -> i::ch)
							(List.assoc cf chem_coface)
						     )@acc
						   with Not_found -> acc
						)
						[]
						!rlcf) :: !chem
		    )
		    lat.(d);

		  assert begin if !trace_opt then (
		    List.iter
		      (fun (i,lc) ->
			 Printf.printf "chem(%i,%i)= " i d;
			 List.iter (fun ch -> Printf.printf "[ " ; List.iter (fun pos -> Printf.printf "%i " pos) ch ;
				      Printf.printf "]  " ) lc; Printf.printf "\n" 
		      )
		      !chem;
		    flush stdout );
		    true
		  end;
		    
	      done ;

	      (* On cherche si les brins construits prcdemment sont dj dans la Gcarte. Si
	         c le cas, on prend le brin dans la Gcarte de retour. Dans le cas contraire, celui
		 d'un brin qui contient une cellule New, on cre un nouveau brin...*)

	      let chem_old, chem_new =
		List.fold_left
		  (fun (acc_old,acc_new) ch ->
		     if ((List.length ch) <> 4) then (
(*		       let sss = snd (List.fold_left (fun (nb,acc) i -> nb+1,lat.(nb).(i)::acc) (0,[]) ch) in
			 if (List.for_all (function Old(_) -> true | _ -> false) sss) then
			   ()
			 else*)
			   raise (Erreur "Ptopo_qmf: add_one_instance: chem length <> 4")
		     ) ;
		     match snd (List.fold_left (fun (nb,acc) i -> nb+1,lat.(nb).(i)::acc) (0,[]) ch) with
		       | Old(d3,_,_,_,_,_)::
			   (Old(d2,_,_,_,_,_)::
			      (Old(d1,_,_,_,_,_)::
				 (Old(d0,_,_,_,_,_)::[]))) ->
			   (try
			      ((ch, self#find_dart d0 d1 d2 d3)::acc_old, acc_new)
			    with Not_found ->
			      Printf.printf "ca chie...\n" ;
			      (acc_old,(ch, newDart gmap 0. 0. 0.)::acc_new)
			   )
		       | _ -> (acc_old,(ch, newDart gmap 0. 0. 0.)::acc_new)
		  )
		  ([],[])
		  (List.concat (List.map (fun (i,qlq) -> qlq) !chem))
	      in
	      let rec cmp_dart d1 d2 i l1 l2 = match (l1,l2) with
		| t1::q1,t2::q2 ->
		    if (t1=t2) then
		      cmp_dart d1 d2 (i+1) q1 q2
		    else
		      if (q1=q2) then
			if (isFree gmap d1 i) && (isFree gmap d2 i) then
			  Core_qmf.sew gmap d1 d2 i
			else
			  raise (Erreur "Ptopo_qmf: cmp_dart: not a gmap")
		      else
			()
		| [],[] -> raise (Erreur "Ptopo_qmf: cmp_dart: equal darts ?!?")
		| _ -> raise (Erreur "Ptopo_qmf: cmp_dart: not a gmap")
	      in
	      let ch = ref chem_new in
		while (!ch <> []) do
		  let one_ch,d1 = List.hd !ch in
		    ch := List.tl !ch;
		    List.iter (fun (snd_ch,d2) -> cmp_dart d1 d2 0 one_ch snd_ch) !ch;
		    List.iter (fun (snd_ch,d2) -> cmp_dart d1 d2 0 one_ch snd_ch) chem_old
		done ;

		(* Il faut essayer des construires les liens entre deux parties du treillis nouvellement
		   construites qui n'ont pas de liens entre elles par des brins prexhistant, copis au
		   dbut de la reconstruction. En effet, Si l'ensemble de la carte est filtr, il faut
		   solidariser les parties que l'on reconstruit. On se base sur les cellules utilises
		   qui sont le ciment de cette solidarit.

		   Le principe est simple. On cherche  lier les chem possant exactement 1 New. En effet, 
		   ceux n'en possdant pas sont grs  l'tape prcdente, et ceux en ayant au moins deux
		   ne peuvent pas tre mis en relation avec d'autre chem d'une autre instance reconstruite
		   (la diffrence entre deux brins correspond  cette cellule New. S'il y a deux cellules
		   New, les brins ne peuvent pas tre adjacent. Il faut en effet au plus une diffrence
		   entre deux brins pour qu 'ils le soient et tre lis par un alpha).
		*)

		let to_add_to_assoce = ref [] in
		List.iter
		  (function (ch,new_d) ->		     
		     match snd (List.fold_left (fun (nb,acc) i -> nb+1,lat.(nb).(i)::acc) (0,[]) ch) with
		       | New(_,i,_,_,_,_)::(Old(d3,_,_,_,_,_))::(Old(d2,_,_,_,_,_))::(Old(d1,_,_,_,_,_))::[]
		       | Old(d3,_,_,_,_,_)::(New(_,i,_,_,_,_))::(Old(d2,_,_,_,_,_))::(Old(d1,_,_,_,_,_))::[]
		       | Old(d3,_,_,_,_,_)::(Old(d2,_,_,_,_,_))::(New(_,i,_,_,_,_))::(Old(d1,_,_,_,_,_))::[]
		       | Old(d3,_,_,_,_,_)::(Old(d2,_,_,_,_,_))::(Old(d1,_,_,_,_,_))::(New(_,i,_,_,_,_))::[] ->(
			   match (self#common_darts gmap d1 d2 d3) with
			     | [] -> to_add_to_assoce := (d1,d2,d3,new_d)::!to_add_to_assoce ;
				 assert begin if !trace_opt then (
				   Printf.printf " ajouter %i\n" (Hash.hash new_d);
				   flush stdout );
				   true
				 end
			     | [cd] ->
				 if (isFree gmap cd i) && (isFree gmap new_d i) then (
				   assert begin if !trace_opt then (
				     Printf.printf "alpha: dart %i " (Hash.hash cd) ; 
				     Printf.printf "dimension %i\n" i ; 
				     flush stdout) ;
				     true ;
				   end ;
				   Core_qmf.sew gmap cd new_d i
				 ) else
				   raise (Erreur "Ptopo_qmf: add_one_instance: not a gmap. alpha is not an involution (1)")
			     | [d1;d2] ->
				 if (((d1 = new_d)||(d2 = new_d)) && (isFree gmap d1 i) && (isFree gmap d2 i)) then (
				   assert begin if !trace_opt then (
				     Printf.printf "alpha: dart %i " (Hash.hash d1) ; 
				     Printf.printf "dimension %i\n" i ;
				     flush stdout );
				     true
				   end;
				   Core_qmf.sew gmap d1 d2 i )
				 else
				   raise (Erreur "Ptopo_qmf: add_one_instance: not a gmap. alpha is not an involution (2)")
			     | _ -> raise (Erreur "Ptopo_qmf: add_one_instance: not a gmap. alpha is not an involution (3)"))
		       | _ ->  ()
		  )
		  chem_new ;

		  List.iter
		    (fun (d1,d2,d3,new_d) -> 
		       self#add_assoce d1 new_d ;
		       self#add_assoce d2 new_d ;
		       self#add_assoce d3 new_d
		    )
		    !to_add_to_assoce ;
		  

		(* ajouter le plongement *)

		let marked_cells0 = ref []
		and marked_cells1 = ref []
		and marked_cells2 = ref []
		and marked_cells3 = ref []
		and mark0 = getNewMark(gmap)
		and mark1 = getNewMark(gmap)
		and mark2 = getNewMark(gmap)
		and mark3 = getNewMark(gmap) in
		  List.iter
		    (fun (ch,d) ->
		       assert begin if !trace_opt then (
			 Printf.printf "%i\n" (Hashtbl.hash d) ;
			 flush stdout );
			 true
		       end;

		       (try 
			  if (not (isMarkedNcell (gmap,d,0) mark0 )) then (
			    markNcell (gmap,d,0) mark0;
			    marked_cells0 := (gmap,d,0)::!marked_cells0 ;
			    Core_qmf.replace_force ret (gmap,d,0)
			      (match lat.(0).(List.nth ch 0) with
				 | New(_,_,_,_,_,rval) -> !rval
				 | Old(cell,_,_,_,_,_) -> Core_qmf.find _avl cell
			      )) 
			with Failure "nth" -> () );

		       (try
			  if (not (isMarkedNcell (gmap,d,1) mark1 )) then (
			    markNcell (gmap,d,1) mark1;
			    marked_cells1 := (gmap,d,1)::!marked_cells1 ;
			    Core_qmf.replace_force ret (gmap,d,1)
			      (match lat.(1).(List.nth ch 1) with
				 | New(_,_,_,_,_,rval) -> !rval
				 | Old(cell,_,_,_,_,_) -> Core_qmf.find _avl cell
			      ))
			with Failure "nth" -> () );
			 
		       (try
			  if (not (isMarkedNcell (gmap,d,2) mark2 )) then (
			    markNcell (gmap,d,2) mark2;
			    marked_cells2 := (gmap,d,2)::!marked_cells2 ;
			    Core_qmf.replace_force ret (gmap,d,2)
			      (match lat.(2).(List.nth ch 2) with
				 | New(_,_,_,_,_,rval) -> !rval
 				 | Old(cell,_,_,_,_,_) -> Core_qmf.find _avl cell
			      )) 
			with Failure "nth" -> () );

		       (try
			  if (not (isMarkedNcell (gmap,d,3) mark3 )) then (
			    markNcell (gmap,d,3) mark3;
			    marked_cells3 := (gmap,d,3)::!marked_cells3 ;
			    Core_qmf.replace_force ret (gmap,d,3)
			      (match lat.(3).(List.nth ch 3) with
				 | New(_,_,_,_,_,rval) -> !rval
				 | Old(cell,_,_,_,_,_) -> Core_qmf.find _avl cell
			      ))
			with Failure "nth" -> () );


		    )
		    (chem_new@chem_old) ;
		  List.iter (fun c -> unmarkNcell c mark0) !marked_cells0 ;
		  List.iter (fun c -> unmarkNcell c mark1) !marked_cells1 ;
		  List.iter (fun c -> unmarkNcell c mark2) !marked_cells2 ;
		  List.iter (fun c -> unmarkNcell c mark3) !marked_cells3 ;
		  freeMark gmap mark0;
		  freeMark gmap mark1;
		  freeMark gmap mark2;
		  freeMark gmap mark3;
		  List.iter (fun f -> unmarkNcell f mark_used.(get_dim_from_ncell f)) p_u;
		  freeMark _avl_gmap mark_used.(0) ;
		  freeMark _avl_gmap mark_used.(1) ;
		  freeMark _avl_gmap mark_used.(2) ;
		  freeMark _avl_gmap mark_used.(3) ;
		  _mark_u <- None


	)
      | _ ->
	  if ((List.length p_f) = 1) then (
	    ()
	  ) else
	    raise (Erreur "mouai, c bof ca !!!")

(*
 * Ides:
 *  - Rajouter ds le dpart l'ensemble des brins n'appartenant pas  une cellule filtre OK
 *  - rajouter dans le treillis les cellules cofaces ou faces des Old (et donc utilises)
 *    pour ddoubler les volumes par exemples...
 *     Procdure rcursive sur l'insertion d'un lment dans le treillis.
 *     Pour les brins du treillis, si toutes les cellules sont des Old, il ne faut pas le
 *     crer, mais rcuprer celui qui existe dj dans la nouvelle carte (et il existe
 *     forcment). Qst, comment le trouver rapidement dans l'ancienne carte ?
 *)

  method build_result =
    let g = create_gmap () in
    let q = create_qmf g _avl_size in

      (* On rajoute l'ensemble des brins non-filtrs par copie *)
      (* On marque les cellules filtres dans la Gcarte... *)
      List.iter
	(fun (_, (p_f,_), _) ->
	   let m_f = self#get_mark_f in
	   let mark_local_filtered = self#get_mark_local_f in
	     List.iter (fun f -> markNcell f m_f ; markNcell f mark_local_filtered.(get_dim_from_ncell f) ) p_f;
	)
	_batch ;
      self#set_copieDI (copy_notmarked_dart _avl_gmap g self#get_mark_f) ;

      List.iter (self#add_one_instance g q) _batch ;
      qmfy g ;
      let q' = create_qmf g _avl_size in
	Core_qmf.iter (fun c v -> Core_qmf.replace q' c v) q;
	freeDirectInfo _avl_gmap self#get_copieDI;
	List.iter (fun (_, (p_f,_), _) -> 
		     let m_f = self#get_mark_f in
		     let mark_local_filtered = self#get_mark_local_f in
		       List.iter (fun f -> unmarkNcell f m_f ;
				    unmarkNcell f mark_local_filtered.(get_dim_from_ncell f)) p_f
		  ) _batch ;
	freeMark _avl_gmap self#get_mark_f;
	freeMark _avl_gmap (self#get_mark_local_f).(0);
	freeMark _avl_gmap (self#get_mark_local_f).(1);
	freeMark _avl_gmap (self#get_mark_local_f).(2);
	freeMark _avl_gmap (self#get_mark_local_f).(3);
	_mark_local_f <- None;
	VAL_qmf (q', couleur)

end
;;












(*
	  let lattice = Array.make 4 (0,[]) in
	    List.iter
	      (function
		 | Filt _ -> ()
		 | (Symb (id, (dim,lf,lcf), _)) as s ->
		     let (pos,l) = lattice.(dim) in
		       lattice.(dim) <- (pos+1,s::lattice.(dim)) ;
		       List.iter
			 (function
			    | VAL_ncell c ->
				let d = get_dim_from_ncell c in
				  if (not (List.mem (Filt(c,VAL_undef "")) lattice.(d) ))
				  then lattice.(d) <- (Filt(c,VAL_undef ""))::lattice.(d)
			    | _ -> ()
			 )
			 lf;
		     List.iter
		       (function
			  | VAL_ncell c  ->
			      let d = get_dim_from_ncell c in
				if (not (List.mem (Filt(c,VAL_undef "")) lattice.(d) ))
				then lattice.(d) <- (Filt(c,VAL_undef ""))::lattice.(d)
			  | _ -> ()
		       )
		       lcf;
	      )
	      rhs ;
	    Printf.printf "dim 0: %d\n" (List.length lattice.(0)) ;
	    Printf.printf "dim 1: %d\n" (List.length lattice.(1)) ;
	    Printf.printf "dim 2: %d\n" (List.length lattice.(2)) ;
	    Printf.printf "dim 3: %d\n" (List.length lattice.(3)) ;
	    let chem = ref [] in
	      chem := List.fold_left (fun acc c -> (c,[c])::acc) (!chem) (lattice.(0)) ;
	      chem := List.fold_left (fun acc c -> (c,[c])::acc) (!chem) (lattice.(0)) ;
	      *)
