(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: topo_qmf.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG, AS
 * Modified: $Date: 2007-04-22 17:04:34 $
 * Version: $Revision: 1.4 $
 * -------------------------------------------------------------------
 *
 * Status: voir le commentaire general dans topology.ml
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Printer;;
open Printer_term;;
open Topology;;
open Def_qmf;;
open Core_qmf;;
open Printer_qmf;;
open Type_qmf;;
open Qmf;;

__register_version "$RCSfile: topo_qmf.ml,v $" "$Revision: 1.4 $" "$Date: 2007-04-22 17:04:34 $";;


exception FoundPos of ncell ;;

let in_place_shuffle_array len a = 
  let nb  = ref (if (len <= 1)
		 then 0
		 else 
		   if (len = 2) 
		   then 1 
		   else len) (* Pour eviter que sur un tableau de longueur 2 on fasse 2 permutations inverses. *)
  in                         (* Le risque est moins grand sur un tableau de taille superieure.                 *)
    while (!nb > 0)
    do
      let src = Random.int len
      and dst = Random.int len
      in
	if (src = dst)
	then ()
	else (let tmp = a.(src)
	      in
		a.(src) <- a.(dst);
		a.(dst) <- tmp;
		decr nb)
    done;
    a
;;


class topo_qmf ((c : 'collection), couleur) mgsval (vdim:int) (ndim:int) =
object(self)
  inherit [ncell] generic_topo 
    Core_qmf.string_of_ncell          (** afficheur de position    **)
    (c, couleur)                      (** collection a transformer **)
    mgsval                            (** l'argument complet       **)
    (Core_qmf.get_gmap_from_qmf c,Core_qmf.dart_null (),-1)
    
  method dummy () = Printf.fprintf stderr "#### creation d'une topo_qmf\n"
  val _gmap = Core_qmf.get_gmap_from_qmf c
  val _avl  = Core_qmf.get_ncells_of_dim_in_qmf vdim c
  val _avl_usage = (let tbl = Hash.create (Core_qmf.size_d vdim c)
		    in
		      Core_qmf.iter_d vdim (fun k v -> Hash.add tbl k PosFree) c;
		      tbl
		   )
  val mutable _batch = ([] : (Terme.path_type * 'position path * mgs_value) list)
  val mutable _last_index = Global.crandom (Core_qmf.size_d vdim c)
  
  method state ncell = (Hash.find _avl_usage ncell)
  method update_state f ncell = Hash.replace _avl_usage ncell (f(Hash.find _avl_usage ncell))
  method next_usable _ =
    if (!match_random <= 0) then
      try
	Hash.iter_from
	  _last_index 
	  (fun ((_,_,dim) as k) v -> if (( dim = vdim)&&(v = PosFree)) then raise (FoundPos k))
	  _avl_usage;
	raise NoMoreUsable
      with FoundPos p ->
	(assert
	   begin 
	     if !trace_opt then
	       (Printf.printf "---- topoQMF: next_usable: found %s\n" (Core_qmf.string_of_ncell p)); 
	     true
	   end;
	 _last_index <- Hash.last_iter_index();
	 p)
    else
      try
	Hash.iter_from_random_key
	  (fun ((_,_,dim) as k) v -> if (( dim = vdim)&&(v = PosFree)) then raise (FoundPos k))
	  _avl_usage;
	raise NoMoreUsable
      with FoundPos p ->
	(assert
	   begin 
	     if !trace_opt then
	       (Printf.printf "---- topoQMF: next_usable: found %s\n" (Core_qmf.string_of_ncell p)); 
	     true
	   end;
	 _last_index <- Hash.last_iter_index();
	 p)

  method unvisit_all =
    Hash.iter (fun k v -> if (pos_visited v) then (Hash.replace _avl_usage k (pos_unvisit v))) _avl_usage
  method direction_from_to p1 p2 = VAL_undef "direction"
  method some_neighbors p = function
    | AllDirections    -> (let verteces = List.map normal_form (Core_qmf.ccells p ndim) in
			   let ans = List.filter
				       (function v -> (pos_usable (self#state v)))
				       verteces
			   in
			   let n = List.length ans in
			     (n, if (!match_random > 0) then
				   (in_place_shuffle_array n (Array.of_list ans))
			         else
				   (Array.of_list ans)
			     )
			  )

    | SomeDirections d -> (let err = "Error: cannot use direction in qmf: "
				     ^(Array.fold_left (fun s x -> s^"\t"^(to_string (prsvalue x))^"\n") "" d)
			   in 
			     prerr_endline err;
			     raise (Erreur err))

  method get_value_from_pos p = (Hash.find _avl p)
  method mgs_value_from_pos p = VAL_ncell(p)

  method add_match pt p v = _batch <- (pt, p, v)::_batch

  method private ajoute_tous h (kind, chemin, rhs) = 
    let idx = ref 0
    and longueur_chemin = List.length chemin
    in
      (match rhs
       with
	   VAL_seq (seq, col) ->
	     (if (longueur_chemin <> (Array.length seq))
	      then raise (Erreur "in QMF(??), replacement must be of same arity")
	      else (List.iter
		      (fun pos -> 
			 ((match seq.(!idx) with 
(*			       VAL_undef _ -> ()   (* C'est que l'on souhaite le supprimer *)*)
			     | x           -> Hash.replace h pos x);
			  incr idx
			 )
		      )
		      chemin
		   )
	     )
	 | VAL_undef _ ->
	     if (longueur_chemin = 1) then
	       ()
	     else
	       (raise (Erreur "in QMF(??), replacement must be of same arity"))

	 | x -> if (longueur_chemin = 1) then (Hash.replace h (List.hd chemin) x)
      )

  method build_result =
    let tbl = Hash.create (Hash.size _avl)
    in
      List.iter (self#ajoute_tous tbl) _batch;
      Hash.iter (fun k v -> (if (pos_usable v)
				then Hash.replace tbl k (Hash.find _avl k))) _avl_usage;
      let ans = Core_qmf.copy c in
	Core_qmf.set_ncells_of_dim_in_qmf vdim ans tbl ;
	VAL_qmf (ans, couleur)




(*
  method private ajoute_tous new_q h (kind, chemin, rhs) = 
    let idx = ref 0
    and longueur_chemin = List.length chemin
    in
      (match rhs
       with
	   VAL_seq (seq, col) ->
	     (if (longueur_chemin <> (Array.length seq))
	      then raise (Erreur "in QMF(??), replacement must be of same arity")
	      else (List.iter                               (*: ('a -> 'b -> 'b) -> 'a list -> 'b -> 'b*)
		      (fun pos -> 
			 ((match seq.(!idx) with 
			      VAL_undef _ -> ()   (* C'est que l'on souhaite le supprimer *)
			    | x           -> Hash.replace h pos x);
			  incr idx
			 )
		      )
		      chemin
		   )
	     )
	 | VAL_undef _ ->
	     if (longueur_chemin = 1) then
	       ()
	     else
	       (raise (Erreur "in QMF(??), replacement must be of same arity"))

	 | VAL_qmf(replace_q,coul) ->
	     if (coul = couleur) then
	       if (longueur_chemin = 1) then
		 let old_s = (List.hd chemin) in (
		   Core_qmf.iter
		     (fun nc v ->
			if ( (Core_qmf.ncelldim nc > vdim) ) then
			  raise (Erreur "on le remplace par un truc plus petit")
			else
			  try (
			    match ((Core_qmf.find new_q nc), v) with
			      | (VAL_undef _, VAL_undef t) -> ()
			      | (x, VAL_undef t)           -> ()
			      | (VAL_undef t, x)           -> (Core_qmf.replace new_q nc x)
			      | (x1, x2)                   -> (Core_qmf.replace new_q nc x2)
			  ) with
			      Not_found -> (
				(Core_qmf.replace new_q nc v)
			      )
		     )
		     replace_q)
	       else
		 raise (Erreur "Je sais pas encore trop koi faire l :)")
	     else
	       if (longueur_chemin = 1)
	       then(Core_qmf.replace new_q (List.hd chemin) rhs)
	       else raise (Erreur "in QMF(??), replacement must be of same arity... at this time !!")

	 | x -> if (longueur_chemin = 1) then (Hash.replace h (List.hd chemin) x)
      )

  method build_result =
    let ans = Core_qmf.copy c in
    let tbl       = Hash.create (Hash.size _avl) in
      Core_qmf.set_ncells_dim vdim ans tbl ;
      Hash.iter (fun k v -> (if (pos_usable v)
				     then Hash.replace tbl k (Hash.find _avl k))) _avl_usage;
      List.iter (self#ajoute_tous ans tbl) _batch;
      VAL_qmf (ans, couleur)
      *)
end
;;

