(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: type_qmf.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-11-23 11:10:57 $
 * Version: $Revision: 1.6 $
 * -------------------------------------------------------------------
 *
 * Voir les fichier Common/type.ml et Qmf/def_qmf.ml
 * 
 *=========================================================================
 *)


open Global;;
open Valeur;;
open Type;;
open Type_scalar;;

__register_version "$RCSfile: type_qmf.ml,v $" "$Revision: 1.6 $" "$Date: 2005-11-23 11:10:57 $";;


(*!*)(** 
  @module scalar
  @name 'ncell
  @type any_bool
  Returns [true] if the argument is a ncell and [false] for any other value. 

  [ncell] is also the type of the QMF topological cells.
*)(*!*)
let ncell_desc = create_type
	        { name = "'ncell";
		  concrete = true;
		  leibniz = false;
		  predicate = wrap_pred(is_ncell);
		  empty = VAL_undef "dummy for default value field in type descriptor 'ncell";
		  pospredicate = scalar_desc.pospredicate;
		  make = scalar_desc.make;
		  xxxify = VAL_undef "Type_scalar: undef_desc: not yet implemented on ncell";
		  derive_from = [scalar_desc];
		  father_of = [];
		  uid = get_new_uid();
		}
;;

(*!*)(** 
  @module scalar
  @name 'gmap
  @type any_bool
  Returns [true] if the argument is a gamap and [false] for any other value. 

  [gmap] is also the type of the Generalized Maps containing the QMF topological cells.
*)(*!*)
let gmap_desc = create_type
	        { name = "'gmap";
		  concrete = true;
		  leibniz = false;
		  predicate = wrap_pred(is_gmap);
		  empty = VAL_undef "dummy for default value field in type descriptor 'gmap";
		  pospredicate = scalar_desc.pospredicate;
		  make = scalar_desc.make;
		  xxxify = VAL_undef "Type_scalar: undef_desc: not yet implemented on gmap";
		  derive_from = [scalar_desc];
		  father_of = [];
		  uid = get_new_uid();
		}
;;

(*!*)(** 
       @module collections
       @name 'qmf
       @type any_bool
       Qmf type and predicate.
*)(*!*)

let qmf_desc = create_type
		 { name = "'qmf";
		   concrete = false;
		   leibniz = true;
		   predicate = wrap_pred(is_leibniz);
		   empty = VAL_undef "dummy for empty field in type descriptor 'qmf";
		   pospredicate = pospredicate_ncell;
		   make = make_dummy;
		   xxxify = xxxify_dummy "'qmf";
		   derive_from = [leibniz_desc];
		   father_of = [];
		   uid = get_new_uid();
		 }
;;

qmf_desc.predicate <- wrap_pred(coll_pred qmf_desc);;

let rec extension_qmf g desc a =
  Type.check_vplist desc.name (to_funct desc.pospredicate) a;
  VAL_qmf(Array.fold_left add_elem (Core_qmf.new_qmf g) a, desc)

and  add_elem q =  function
  | VAL_seq(aa, _) -> Core_qmf.add_ncell q (to_ncell aa.(0)) aa.(1)
  | _ -> raise (Erreur "Internal Error: extension: add_elem in extension_qmf")
;;

let rec create_new_qmf nom gmap =
  try
    let t = Hash.find type_tbl nom
    in let err = "Error: type  "^nom^"  already defined."
    in prerr_endline err; prftype stderr t; raise (Erreur err)
  with Not_found -> 
    (if Hash.mem Environnement.var_tbl nom
     then prerr_endline ("Warning: type predicate  "^nom^"  hiddes a function with same name");
     let nt = create_type { name = nom;
		 	    concrete = true;
			    leibniz = true;
			    predicate = qmf_desc.predicate;
			    empty =  VAL_qmf ((gmap,[|Hash.create 11;Hash.create 11;Hash.create 11;Hash.create 11|],3), qmf_desc);
			    pospredicate = pospredicate_ncell;
			    make = make_dummy;
			    xxxify = xxxify_dummy nom;
			    derive_from = [qmf_desc];
			    father_of = [];
			    uid = get_new_uid();
			  }
     in 
       begin
	 Core_qmf.qmfy gmap ;
	 nt.empty <- change_type nt nt.empty;
	 nt.predicate <- wrap_pred(coll_pred nt);
	 nt.make <- extension_qmf gmap nt;
	 make_xxxify nt;
	 ignore(Environnement.addfct nom nt.predicate);
	 nt
       end)
