(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: printer_rec.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2004-09-23 20:17:13 $
 * Version: $Revision: 1.3 $
 * -------------------------------------------------------------------
 *
 * Les fonctions d'impressions sur les scalaires.
 *
 * Il faut implmenter par type Xxx:
 *
 *  -  _uprbval_'outBuffer_xxx : impression non-formatte de xxx
 *  -  _pfbval_'outBufferF_xxx : impression formatte de xxx
 *
 *
 *=========================================================================
 *)

open Buffer;;
open Global;;
open Valeur;;

open Dispatch_sig;;
open Environnement;;
open Printer;;
open Auxfct;;



__register_version "$RCSfile: printer_rec.ml,v $" "$Revision: 1.3 $" "$Date: 2004-09-23 20:17:13 $";;


(** Attention: les noms collisionnent dans Printf et Format.
 ** Aussi on n'ouvre pas ces modules mais on dfinit ici 2 alias
 **)
let uprintf = Printf.bprintf (** impression standard dans un buffer **)
and fprintf = Format.fprintf (** impression formatt dans un buffer **)
;;





(** uprval b x : est une fonction qui imprime une valeur mgs x dans son buffer b.
 ** Cette fonction est la fonction principale d'affichage. L'impression dans une 
 ** string ou bien sur la sortie standard ou bien dans un fichier derivent de 
 ** cette fonction
 **)

let upr_field flag b id v =
  (if !flag then uprintf b ", " else flag := true);
  (match id with
     | VAL_symbol s ->  uprintf b "%s = %a" s uprbval2 v
     | k ->  uprintf b "%a = %a" uprbval2 id  uprbval2 v)
;;

let _uprbval_'outBuffer_rec (b, (r, color)) = 
  let flag = ref false 
  in
    uprintf b "{ ";
    Core_rec.iter (upr_field flag b) r;
    uprintf b " }";
    valundef
;;

let rec _upobval1_'outBuffer_string_string_string_funct_rec (b,pre,inter, post,f, (r,color)) = let flag = ref false in
  uprintf b "%s " pre;
  Core_rec.iter (fun id v -> uprintf b "%s%a = %a" (if !flag then inter else "") uprbvalue2 id uprbvalue2 v; flag := true) ( r);
  uprintf b " %s" post;
  valundef
;;


(** _pfbval_'outBufferF_XXX (b, x) prend un buffer format [b] et une valeur [x] de type XXX. 
 ** Elle crit de faon formate [x] dans le buffer format [b], puis renvoie valundef
 **
 ** On le fait "de manire imprative" car la mise en place des ordre de formattage est "dlicate".
 ** On ne peut mettre l'ordre de formattage que directement dans la string de fprintf, pas dans
 ** un argument qui sera passer  un %s: on ne peut donc pas utiliser le Aux.plist
 **)

let pr_field flag b id v =
  (if !flag then fprintf b ",@ " else flag := true);
  (match id with
     | VAL_symbol s ->  fprintf b "%s = %a" s prbval2 v
     | k ->  fprintf b "%a = %a" prbval2 id  prbval2 v)
;;

let rec _pfbval_'outBufferF_rec (b, (r, color)) = 
  let flag = ref false in
    fprintf b "{ @[<hov>";
    Core_rec.iter (pr_field flag b) r;
    fprintf b "@]@ }@,";
    valundef
;;

let rec _pobval1_'outBufferF_int_string_string_string_funct_rec (b,t,pre,inter, post,f, (r,color)) = let flag = ref false in
  Format.pp_open_box b t;
  Format.fprintf b "%s " pre;
  Core_rec.iter (fun id v -> Format.fprintf b "%s@,%a = %a" (if !flag then inter else "") pfbvalue2 id pfbvalue2 v; flag := true) ( r);
  Format.fprintf b " %s@]@," post;
  valundef
;;


(* --- fin du fichier ----------------------------------- *)

