(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: topo_rec.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2007-06-27 16:55:53 $
 * Version: $Revision: 1.1 $
 * -------------------------------------------------------------------
 *
 * Status: voir le commentaire general dans topology.ml
 *          
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Terme;;
open Printer;;
open Topology;;

__register_version "$RCSfile: topo_rec.ml,v $" "$Revision: 1.1 $" "$Date: 2007-06-27 16:55:53 $";;



(* =================================================================
 * === Record ======================================================
 * =================================================================
 *)


exception PosDepart of mgs_value ;;


(** --- La topologie d'une sequence ------------------------------ **)

class topo_record (c, couleur) mgsval =
object(self)
  inherit [mgs_value] generic_topo 
    (Auxfct.rond to_string prsvalue)   (** afficheur de position    **)
    (c, couleur)                       (** collection a transformer **)
    mgsval                             (** la valeur mgs complete sur laquelle on travaille **)
    (let nb = ref (Global.crandom (Core_rec.size c)) in
       try
	 Core_rec.iter (fun pos v -> if (!nb=0) then raise (PosDepart pos) else (decr nb)) c ;
	 raise (Erreur "Topo_rec: pos: oups!")
       with PosDepart v -> v
    )                                  (** position de depart       **)

  val _lg = Core_rec.size c

  val _used = Core_rec.map (fun v -> (ref PosFree)) c

  val mutable _batch = ([] : (rule_property * 'position path * mgs_value) list)

  method add_match pt p v = _batch <- (pt, p, v)::_batch

  method state p          = !(Core_rec.find p _used)

  method update_state f p = let st = (Core_rec.find p _used) in st := f (!st)

  method unvisit_all =
    Core_rec.iter (fun _ st -> if pos_visited(!st) then st := pos_unvisit(!st)) _used

  method some_neighbors p _ = (0, [||])
				
  method mgs_value_from_pos p = p

  method get_value_from_pos p = Core_rec.find p c

  method direction_from_to p1 p2 = VAL_undef "direction_from_to in rec"

  method next_usable p = (
    
    let nb = if !Global.match_random >= 2 then (ref (Global.crandom (Core_rec.size c))) else (ref 0) in
    let nb' = ref (!nb) in
       try
	 
	 (* A partir d'une certaine pos *)
	 Core_rec.iter (fun pos st ->
			  if (!nb <= 0)
			  then
			    begin
			      assert begin if !trace_opt then 
				(Printf.printf "---- topoREC: next_usable: try %s\n" (self#prpos pos); flush stdout);
				true
			      end;

			      if not(pos_used !st) & not(pos_visited !st) then raise (PosDepart pos)

			    end ;
			  (decr nb)
		       ) _used ;

	 (* Du dbut  la pos *)
	 Core_rec.iter (fun pos st ->
			  if (!nb' > 0)
			  then
			    begin
			      assert begin if !trace_opt then 
				(Printf.printf "---- topoREC: next_usable: try %s\n" (self#prpos pos); flush stdout);
				true
			      end;

			      if not(pos_used !st) & not(pos_visited !st) then raise (PosDepart pos)

			    end
			  else
			    begin
			      raise NoMoreUsable
			    end ;
			  (decr nb')
		       ) _used ;

	 raise NoMoreUsable

       with
	 | PosDepart pos -> (
	     assert begin if !trace_opt then 
	       (Printf.printf "---- topoREC: next_usable: found %s\n" (self#prpos pos)); 
	       true 
	     end;
	     pos
	   )
  )


  method build_result = (
    let rec funct = function
      | [] -> (
	  Core_rec.fold
		 (fun pos st acc ->
		    if not (pos_used !st)
		    then Core_rec.add pos (Core_rec.find pos c) acc
		    else acc
		 )
		 _used
		 Core_rec.empty
	)
	  
      | (rp, [pos], VAL_seq(a, _))::b when
	  (Array.length a = 1)
	  && (rp.a_flat = FlatPath) -> (
	    let acc = funct b in
	      Core_rec.add pos a.(0) acc
	  )

      | (rp, [pos], v)::b -> (
	  let acc = funct b in
	    Core_rec.add pos v acc
	)
	  
      | _ -> raise (Erreur "Topo_rec: build_result: path is too long")
    in
      VAL_rec(funct _batch, couleur)
  )


end;;

