(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: seq.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2007-09-28 09:59:12 $
 * Version: $Revision: 1.9 $
 * -------------------------------------------------------------------
 *
 * Status:  
 *    Implantation au niveau "representation MGS" des fonctions sur les 
 *    sequences.
 *
 *    Dans un rpertoire Xxx correspondant  une collection xxx, 
 *    le type xxx est defini dans def_xxx.ml et les operations sur 
 *    ce type sont dfinies dans core_xxx.ml. Le fichier kernel_xxx.ml
 *    contient la declaration dans la librairie MGS des fcts dfinies 
 *    dans xxx.ml
 *
 *    L'organisation de xxx.ml fichier suit l'ordre suivant:
 *
 *       1/ Fonctions de construction: ... -> seq (sauf extension)
 *             cons, join, append, shape, xxxify, iota, extension
 *
 *       2/ Prdicats, accs et observateur
 *             compare, val2bool, size, empty, count, 
 *             member, random, hd, tl, take
 *
 *       3/ Fold et autre itrations
 *             fold, map, iter, foldindexed, mapindexed, iterindexed,
 *             forall, exists, filter
 *
 *       4/ Neighbors, neighbor
 *
 *
 *       5/ Fonctions spcifiques
 *
 *
 *       6/ mise  jour du descripteur du type
 *              champs: make, xxxify et empty
 *
 * Ici :   xxx = seq
 *
 *=========================================================================
 *)


open Global;;
open Def_seq;;
open Valeur;;
open Type;;
open Dispatch_sig;;



__register_version "$RCSfile: seq.ml,v $" "$Revision: 1.9 $" "$Date: 2007-09-28 09:59:12 $";;

(** alias **)
let seq_desc = Type_seq.seq_desc
;;
let empty = [||]
;;
let empty_seq = VAL_seq(empty, seq_desc)
;;


(** === Constructeurs ======================================================- **)

let _cons_any_seq (e, s) = VAL_seq(Array.append [|e|] (fst s), snd s)
and _cons_undef_seq (_, s) = VAL_seq s
;;


let _append_seq_seq (s1, s2) = VAL_seq(Array.append (fst s1) (fst s2), commontype (snd s1) (snd s2))
;;



let _join_seq_seq (s1, s2) = VAL_seq(Array.append (fst s1) (fst s2), commontype (snd s1) (snd s2))
;;

(* Les joins entre collections diffrentes conduisent  une squence 
 * - sauf si l'un des lments est un record
 * - sauf si l'un des lments est leibnizien et l'autre undef.
 * On met le code ici, faute de mieux.
 *)

let _join_any_any (a, b) = 
  (* reste donc le cas: 2 scalaire ou bien 2 collections. Le cas ou l'un des deux arguments est undef
   * est trait par ailleurs.
   *)
  VAL_seq([|a; b|], seq_desc)
;;

(* on sait que le any n'est pas un undef car il est traite ci-dessous *)
let _join_any_seq (e, s) = if true_coll e then _join_any_any (e, VAL_seq(s)) else VAL_seq(Array.append [|e|] (fst s), snd s)			       
and _join_seq_any (s, e) = if true_coll e then _join_any_any (VAL_seq(s), e) else VAL_seq((Array.append (fst s) [|e|]), snd s)
;;

(* On met le join de ces scalaire particulier ici, car on a le descripteur de type des squences *)
let _join_undef_undef (_, _) = VAL_seq([||], seq_desc)
and _join_undef_any   (_, e) =  VAL_seq([|e|], seq_desc)
and _join_any_undef   (e, _) =  VAL_seq([|e|], seq_desc)

and _join_seq_undef (s, _) = VAL_seq s
and _join_undef_seq (_, s) = VAL_seq s

;;




let _shape_seq (s, deco) = 
  let taille_s = Array.length s
  in
    VAL_funct (function 
		 | VAL_seq (s', _) -> 
		     (if (taille_s <> Array.length s')
		      then raise (Erreur "shape (seq) : argument is not of the same size of the original")
		      else VAL_seq (s', deco))
		 | _  -> raise (Erreur "shape (seq) : a sequence as argument is required"))
;;



let _sequify_seq s = VAL_seq(fst s, seq_desc)

and _sequify_string s = 
  let lg =  String.length s in
  let tab = Array.make lg mgsundef in
    begin
      for i = 0 to lg - 1 do 
	tab.(i) <- VAL_string(String.make 1 s.[i])
      done;      
      VAL_seq(tab, seq_desc)
    end

and _sequify_any x = VAL_seq([|x|], seq_desc)
;;


let _iota_int_seq (n, (v, col)) = 
  if (n < 0)
  then raise (Erreur "iota_seq with a negative argument")
  else (let res = Array.make n (VAL_int 0)
	in
	  for i = 1 to (n-1)
	  do
	    res.(i) <- VAL_int i
	  done;
	  VAL_seq ((Array.append res v), col))
;;



(** Extension --------------------------------- **)

let rec extension_seq seq_desc a = 
  check_vplist seq_desc.name (to_funct seq_desc.pospredicate) a;
  let sup = Array.fold_left maxsize 0 a in
  let tab = Array.make (1+sup) (VAL_undef "default value in seq extension")
  in Array.iter (set_tab tab) a;
    VAL_seq(tab, seq_desc)

and maxsize m = function
| VAL_seq(aa, _) -> max m (to_int aa.(0))
| _ -> raise (Erreur "Internal Error: extension: maxsize in extension_seq")

and set_tab tab = function
  | VAL_seq(aa, _) -> tab.(to_int aa.(0)) <- aa.(1)
  | _ -> raise (Erreur "Internal Error: extension: set_tab in extension_seq")
;;



(** === Prdicats, accs et observateurs =========================================== **)

let rec _compareMgsVal_seq_seq ((x,tx), (y,ty)) = 
  let tdiff = compare_type tx ty
  in if tdiff <> 0 then VAL_int(tdiff) else
      begin
	let lx = Array.length x
	and ly = Array.length y in
	let diff = lx - ly 
	in (if diff < 0 then VAL_int(-1)
	    else if diff > 0 then VAL_int(1)
	    else compare_seq_aux lx x y)
      end

and compare_seq_aux lg x y =
  let res = ref 0 in
  try
    for i = 0 to lg - 1 do
      res := compare_mgs_value x.(i) y.(i);
      if 0 <> !res then raise BreakLoop
    done;
    mgszero
  with BreakLoop -> VAL_int(!res)


and _val2bool_seq x = VAL_bool(0 <> Array.length (fst x))
;;


let _size_seq x = VAL_int(Array.length (fst x))
and _empty_seq x = VAL_bool(0 = Array.length (fst x))
;;


exception Found of mgs_value;;
exception BreakLoop;;

let _count_any_seq (a, (s, _)) = 
  let nb = ref 0
  in
    Array.iter (function e -> if (0 = compare_mgs_value e a) then (incr nb)) s;
    VAL_int (!nb)

let _member_any_seq (a, (s, _)) = 
  try
    Array.iter (function e -> if (0 = compare_mgs_value e  a) then raise BreakLoop) s;
    VAL_bool false
  with BreakLoop -> VAL_bool true
;;



let _random_seq (s, _) = 
  let n = Array.length(s)
  in if n == 0 then VAL_undef "random: bad argument: empty seq"  else s.(Random.int(n))
;;



let _hd_seq s = 
  let ss = fst s in if (0 = Array.length ss) then VAL_undef("hd on empty seq") else ss.(0)

and _tl_seq s = 
  let ss  = fst s in
  let len = Array.length ss
  in
    if (0 = len)
    then VAL_undef("tl on empty seq") 
    else VAL_seq(Array.sub ss 1 (len - 1), snd s)
;;

let _take_seq_int ((s, col), n) =
  begin
    if n >= 0 then 
      (if n < Array.length s then s.(n) else VAL_undef "take out of range")
    else
      let idx = (Array.length s) + n in
	(if idx <  Array.length s then s.(idx) else VAL_undef "take out of range")
  end
;;


(** le take generaliste :
 ** si n >= 0:  take(c, n) = hd(tl^n(c))
 ** si n < 0 :  take(c, n) = hd(tl^(N+n)(c)) avec N = size(c)
 ** pour une sequence, un n ngatif correspond  prendre les lments  partir de la fin.
 **)
let rec _take_any_int (c, n) =
  if not(is_collection(c)) then raise (Erreur "take: first argument is not a collection")
  else (if n >= 0 then take_aux c n else take_aux c (to_int(size(c)) + n))

and take_aux c n = 
  if n < 0 then VAL_undef "take: out of bound"
  else begin
    if n = 0 then hd(c)
    else take_aux (tl c) (n-1)
  end
;;


(** le take generalis:
 ** take(seq, coll) ou coll est une collection d'entier, remplace chaque element i (un entier) 
 ** de la collection coll par l'element d'index i de seq
 **)
let _take_seq_any ((s, _), col) =
  if not(is_collection(col)) then raise (Erreur "take: second argument must be an integer or a collection")
  else
    let fct = function i ->
      if is_int(i) then 
	let j = to_int i in (if j < 0 or j >= Array.length s then VAL_undef "take: out of bound" else s.(j))
      else VAL_undef "take: index not an integer"
    in Dispatch_sig.map (VAL_funct fct) col
;;





(* === Fold et autre itrations ================================ *)

let _fold_funct_any_seq (fonc, zero, (vect, _)) = Array.fold_right (fun v suite -> (app_2 fonc v suite)) vect zero
and _map_funct_seq (fonc, (vect, c)) = VAL_seq ((Array.map fonc vect), c)
and _iter_funct_seq (fonc, (vect, _)) = (Array.iter  (fun x   -> fonc x; ()) vect); VAL_undef "iter seq"
and _mapindexed_funct_seq (fct, (vect, t)) =  VAL_seq ((Array.mapi (fun i x -> (app_2 fct (VAL_int i) x)) vect), t)
and _iterindexed_funct_seq (fonc, (vect, _)) = (Array.iteri (fun idx x -> ignore(app_2 fonc (VAL_int idx) x)) vect); VAL_undef "iter"
and _foldindexed_funct_any_seq (fct, zero, (vect, t)) = 
  let acc = ref zero 
  and sz = Array.length vect in
    begin
      for i = sz-1 downto 0 do
	acc := app_3 fct (VAL_int i) vect.(i) !acc
      done;
      !acc
    end  

and _forall_funct_seq (p, s) = VAL_bool(Global.array_forall (fun x -> Auxfct.bool_from_val (p x)) (fst s))
and _exists_funct_seq (p, s) = VAL_bool(Global.array_exists (fun x -> Auxfct.bool_from_val (p x)) (fst s))
and _filter_funct_seq (p, s) = 
  VAL_seq(Array.of_list (List.filter (fun x -> Auxfct.bool_from_val(p x)) (Array.to_list (fst s))),
	  snd s)
;;







(* === Neighbors, neighbor ================================ *)

let _neighbors_seq_int ((t, _), i) = 
  let sz         = Array.length t 
  and retourne v = VAL_seq (v, seq_desc)
  in
    assert((i >= 0) && (i < sz));             (* Acces hors des bornes    *)
    if (sz = 1)                               (* Taille de 1 => 0 voisins *)
    then (retourne [||])
    else 
      if (i = 0)                              (* Index 0     => 1 voisins *)
      then (retourne [| t.(1) |])
      else 
	if (i = (sz-1))                       (* Index max   => 1 voisins *)
	then (retourne [| t.(i-1) |])
	else (retourne [| t.(i-1) ; t.(i+1) |])


and _left_seq_int ((t, _), i) = 
  let sz = Array.length t
  in
    assert((i >= 0) && (i < sz));
    if (i > 0)
    then t.(i-1)
    else VAL_undef "left out of bound"

and _right_seq_int ((t, _), i) = 
  let sz = Array.length t
  in
    assert((i >= 0) && (i < sz));
    if (i < sz-1)
    then t.(i+1)
    else VAL_undef "right out of bound"




(* === Fonctions spcifiques ======================================================- *)

let _sort_funct_seq (f, (s, col)) =
  let cmp = fun x y -> (match (pf1 (f x))(y) with 
			  | VAL_int(n) -> n
			  | _ -> let err = "Error: 'sort: the comparison function must return an integer"
			         in prerr_endline err; raise (Erreur err))
  and scopy =  Array.copy s
  in Array.sort cmp scopy; VAL_seq(scopy, col)
;;

let _sort_funct_any (f, col) =
  if not(is_collection(col)) then raise (Erreur "sort: second argument must be a collection")
  else
    let s = to_seq(Dispatch_sig.sequify col)
    in _sort_funct_seq (f, (s, seq_desc))
;;


let _last_seq (v, col) =
  let len = Array.length v in 
    if (len > 0) 
    then v.(len-1)
    else VAL_undef "last seq"
;;


(* Flatten ----------------------------------------------------------- *)


let rec _flatten_seq (s, t) =  VAL_seq((flatten_seq_aux s), t)

and flatten_seq_aux s =
   let l = Array.map (function VAL_seq (a,_) -> Array.to_list a | y -> [y]) s in
   let ll = Array.fold_right (fun x acc -> x@acc) l []
   in Array.of_list ll
;;



(** Une fonction partition sur les monoides ----------------------------------- *)


let _partition_int_seq (n, (s, t)) =
  if n <= 0 then VAL_undef "partition: bad negative argument" else
    begin
      let lg = Array.length s in
      let sz = 1+(lg-1)/n in
      let a = Array.init sz 
		(fun i -> 
		   let debut = i*n in let len = (Pervasives.min lg (debut+n)) - debut 
		   in VAL_seq(Array.sub s debut len, t))
      in VAL_seq(a, t)
    end
;;



(** Partition en 2 classes suivant un predicat ----------------------------------------------- *)


let _partition_funct_seq (f, (s, t)) = 
  let (l1, l2) = Array.fold_left 
		   (fun (ll1, ll2) x -> 
		      if Auxfct.bool_from_val (f x) 
		      then (x::ll1, ll2)
		      else (ll1, x::ll2))
		   ([], [])
		   s
  in let a1 = Array.of_list (List.rev l1)
     and a2 = Array.of_list (List.rev l2)
  in let a = Array.make 2 (VAL_undef "dummy _partition_funct_seq")
  in 
    a.(0) <- VAL_seq(a1, t); 
    a.(1) <- VAL_seq(a2, t); 
    VAL_seq(a, t)
;;
  




(** Une fonction partition par quivalence sur les monoide ----------------------------------- *)

let rec _partitioneq_funct_seq (f, (s, t)) =
  let eql = ref [] in (* eql contient une liste de classe d'equivalence. Cf. Auxfct.update_equivalence *)
    begin             
      for i = 0 to (Array.length s) - 1 do
	eql := Auxfct.update_equivalence f s.(i) !eql
      done;
      VAL_seq(Array.of_list(List.map (function (e, l) -> VAL_seq(Array.of_list !l, t)) !eql), t)
    end
;;



(** Fonctions reverse et rotate sur les sequences ----------------------------------- *)

let _reverseseq_seq (s, t) =
  let lg = (Array.length s) - 1
  in VAL_seq(Array.init (lg+1) (function i -> s.(lg - i)), t)

and _rotateleft_seq (s, t) =
  let lg = (Array.length s)
  in VAL_seq(Array.init lg (function i -> if i+1 < lg then s.(i+1) else s.(0)), t)

and _rotateright_seq (s, t) =
  let lg = (Array.length s)
  in VAL_seq(Array.init lg (function i -> if i >= 1 then s.(i-1) else s.(lg-1)), t)
;;


(** Fonctions pad sur les sequences ----------------------------------- *)

let _padLeft_seq_int_any ((s, t), n, v0) =
  let lg = Array.length s in
    if lg == n then VAL_seq(s, t)
    else if lg > n then let deb = lg - n in VAL_seq(Array.init n (function i -> s.(i+deb)), t)
    else (let deb = n - lg 
	  in VAL_seq(Array.init n (function i -> if i < deb then v0 else s.(i - deb)), t))

let _padRight_seq_int_any ((s, t), n, v0) =
  let lg = Array.length s in
    if lg == n then VAL_seq(s, t)
    else if lg > n then VAL_seq(Array.init n (function i -> s.(i)), t)
    else VAL_seq(Array.init n (function i -> if i >= lg then v0 else s.(i)), t)
;;


(* --- Matrice ------------------------------------------------------- *)

exception BadSize
;;


(** Nombre max d'imbrication de squences (profondeur) *)

let rec _matrixDim_seq (s, t) =
  VAL_int(1 + Array.fold_left (fun acc x -> max acc (to_int (matrixDim x))) 0 s)

and _matrixDim_any _ = VAL_int(0)


(** Rank = le nbre d'lments dans chaque dimensiuons d'une imbrication de squences *)

and _rank_seq (s, t) = 
  let rk =  Array.fold_left (fun acc x -> maxRank acc (rank x)) (VAL_seq ([||], t)) s
  in VAL_seq(Array.append [|VAL_int(Array.length s)|] (to_seq rk), t)

and _rank_any _ = VAL_seq([||], seq_desc)

and _maxRank_seq_seq ((s1, t1), (s2, t2)) =
  let sz1 = Array.length s1 
  and sz2 = Array.length s2 in
  let sz = max sz1 sz2 in
  let v = Array.init sz (initrk sz1 sz2 s1 s2)
  in VAL_seq(v, t1)

and initrk sz1 sz2 s1 s2 i =
  if (i < sz1) & (i < sz2) then VAL_int(max (to_int s1.(i)) (to_int s2.(i)))
  else if (i < sz1) then s1.(i)
  else s2.(i)


and srank m = Array.map to_int (to_seq (rank m))


(** transformer une imbrication de squences en matrice  n dimensions en homognisant les manques *)

and matrixify m = 
  let rk = Array.to_list (srank m)
  in matrixify2 rk m

and _matrixify2_'intlist_seq (rk, (s, t)) = 
  match rk with
    | [] -> VAL_seq(s, t)
    | a::l -> let sz = Array.length s in
	VAL_seq(Array.init a (fun i -> if i < sz then matrixify2 l s.(i) else VAL_undef "matrixify"), t)

and _matrixify2_'intlist_any (rk, x) = 
  match rk with
    | [] -> x
    | a::l -> VAL_seq(Array.make a  (matrixify2 l x), seq_desc)



(** Verifier qu'on a bien quelque chose qui peut passer pour une matrice *)

and _matrixCheck_seq (s, _) =
  let size_row = Array.length s in
    if size_row = 0 then mgsfalse
    else if not(is_seq(s.(0))) then mgsfalse
    else
      let size_col = Array.length (to_seq s.(0))
      in try checksize s size_row size_col with BadSize -> mgsfalse

and _matrixCheck_any _ = mgsfalse

and checksize s n m =
  for i = 0 to n-1 do
    let e = s.(i) in
      if not(is_seq(e)) or (m <> Array.length (to_seq e)) then raise BadSize
  done;
  VAL_seq([|VAL_int(n); VAL_int(m)|], seq_desc)


(** Transposer une matrice *)

and _transpose_seq (s, t) =
  match _matrixCheck_seq (s, t) with
    | VAL_seq([|VAL_int(n); VAL_int(m)|], _) -> 
	let a = Array.make n (VAL_undef "transpose")  in
	  for i = 0 to n-1 do
	    a.(i) <- VAL_seq(Array.init m (function j -> (to_seq s.(j)).(i)), t)
	  done;
	  VAL_seq(a, t)
    | _ -> let err = "Error: transpose: argument is not a matrix" in
	Printf.fprintf stderr "%s:\n\t%a\n" err Printer.prfvalue2 (VAL_seq(s,t));
	raise (Erreur err)


(** produit externe generalise *)

and _outerproduct_funct_funct_seq_seq (fplus, fprod, (s1, t1), (s2, t2)) =
  match _matrixCheck_seq (s1, t1), _matrixCheck_seq (s2, t2) with
    |  VAL_seq([|VAL_int(n1); VAL_int(m1)|], _), VAL_seq([|VAL_int(n2); VAL_int(m2)|], _)
	 -> prod_matrix_matrix fplus fprod n1 m1 s1 n2 m2 s2
    |  VAL_seq([|VAL_int(n1); VAL_int(m1)|], _), _ ->  prod_matrix_vector fplus fprod n1 m1 s1 s2
    |  _, VAL_seq([|VAL_int(n2); VAL_int(m2)|], _) ->  prod_matrix_vector fplus fprod n2 m2 s2 s1
    | _ -> prod_vector_vector fplus fprod s1 s2

and prod_vector_vector fplus fprod s1 s2 =
  let n1 = Array.length s1
  and n2 = Array.length s2 
  in if n1 <> n2 then VAL_undef "outerproduct(vector,vector): bad size"
    else if n1 = 0 then VAL_seq(s1, seq_desc)
    else let acc = ref (app_2 fprod s1.(0) s2.(0)) in
      begin
	for i = 1 to n1 - 1 do
	  acc := app_2 fplus !acc (app_2 fprod s1.(i) s2.(i))
	done;
	!acc
      end

and prod_matrix_vector fplus fprod n m s v = 
  if m <> Array.length v then VAL_undef "outerproduct(matrix,vector): bad size"
  else
    VAL_seq(Array.init n (function i -> prod_vector_vector fplus fprod (to_seq s.(i)) v),
	    seq_desc)


and prod_matrix_matrix fplus fprod n1 m1 s1 n2 m2 s2 = 
  if m1 <> n2 then VAL_undef "outerproduct(matrix,matrix): bad size"
  else let a = Array.init n1 
		 (fun _ ->  VAL_seq(Array.make m2 (VAL_undef "internal:_outerproduct(matrix, matrix)"),
				    seq_desc))
  in
    for i = 0 to n1 - 1 do
      for j = 0 to m2 - 1 do
	let acc = ref (app_2 fprod (to_seq s1.(i)).(0) (to_seq s2.(0)).(j)) in
	  for k = 1 to m1 - 1 do
	    acc := app_2 fplus !acc (app_2 fprod (to_seq s1.(i)).(k) (to_seq s2.(k)).(j))
	  done;
	  (to_seq a.(i)).(j) <- !acc
      done
    done;
    VAL_seq(a, seq_desc)


(** Produit externe classique sur des matrices d'entiers ou de floatant *)

and _scalarproduct_seq_seq ((s1, _), (s2, _)) =
  prod_vector_vector
    (function x -> VAL_funct(function y -> add x y))
    (function x -> VAL_funct(function y -> mult x y))
    s1 s2

and prodMatrixVector m v =
  outerproduct 
    (VAL_funct (function x -> VAL_funct(function y -> add x y)))
    (VAL_funct (function x -> VAL_funct(function y -> mult x y)))
    m v    

and prodMatrixMatrix m1 m2 =
  outerproduct 
    (VAL_funct (function x -> VAL_funct(function y -> add x y)))
    (VAL_funct (function x -> VAL_funct(function y -> mult x y)))
    m1 m2   
;;



(* ================================================================ *)

(*** !!! Ne pas oublier de mettre  jour les champs
 ***     _ make 
 ***     _ xxxify
 ***     _ empty
 *** du descripteur de type 
 ***)
seq_desc.make <- extension_seq seq_desc;;  
seq_desc.xxxify <- VAL_funct(sequify);;
seq_desc.empty <- empty_seq;;


