(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: completion.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2004-09-24 00:17:02 $
 * Version: $Revision: 1.1 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *=========================================================================
 *)

open Global;;
open Set;;


__register_version "$RCSfile: completion.ml,v $" "$Revision: 1.1 $" "$Date: 2004-09-24 00:17:02 $";;




module Words = Set.Make (struct type t = string let compare=Pervasives.compare end)
;;
let words = ref Words.empty
;;

let rec add_word w = 
  begin
    words := Words.add w !words;
    if w.[0] = '\'' then add_word (String.sub w 1 ((String.length w) - 1))
  end
;;



let substring p w =
  let lp = String.length p
  and lw = String.length w in
    if lp > lw then false else
      begin
	try
	  for i = 0 to lp-1 do
	    if p.[i] <> w.[i] then raise BreakLoop
	  done;
	  true
	with BreakLoop -> false
      end
;;




(* Prefixe commun  une liste de mots
 *)

exception CommonIndex of int;;

let rec common_prefixe = function
  | [] -> ""
  | w::l -> commonpl w l

and commonpl w = function
  | [] -> w
  | ww::l -> commonpl (common_ww w ww) l

and common_ww w1 w2 =
  let lg = min (String.length w1) (String.length w2) in
  let cindex = 
    try
      for i = 0 to lg-1 do
	if w1.[i] <> w2.[i] then raise (CommonIndex i)
      done;
      lg
    with CommonIndex idx -> idx
  in String.sub w1 0 cindex
;;


let reverse_string w = 
  let lg = String.length w in
  let rw = String.create lg in
    begin
      for i = 0 to lg-1 do
	rw.[lg - i - 1] <- w.[i]
      done;
      rw
    end
;;


exception Ok of string list
;;
     
(** on ne retient pas tous les elements qui viennent de words 
 ** - on enleve des noms systemes qui commencent par ' ou '$ ou bien $ ou 
 ** - on enleve aussi 1_PI, 2_PI, etc. qui ne peuvent s'ecrire que comme '1_PI, etc.
 ** - on ne liste pas la forme 'xxx et xxx si xxx refere a 'xxx (cela n'arrive que quand
 **   on demande la liste avec un prefixe vide !)
 **)
let filtre_completion prefixe w = 
  match w.[0] with
  | ''
  | '$'  
  | '0'..'9'-> false
  | '\'' ->  let c = w.[1] in (c <> '') & (c <> '$')
  | _ -> ((0 <> (String.length prefixe)) 
	  or not(Environnement.is_exactly_defined_var(String.concat "" ["'"; w])))
;;

let cherche_completion w = 
  List.filter (filtre_completion w) (Words.elements(fst (Words.partition (substring w) !words)))
;;


let max_char_in_line = 80
and separe_col = "  "
;;

let lg_separe_col = String.length separe_col
;;

let rec affiche_colone lw = 
  let lg = List.length lw in
    match lg with
      | 0 -> ()
      | n when n < 4  -> List.iter (fun x ->  prerr_endline ("\t"^x))  lw
      | n when n < 9  -> arrange_col (Array.of_list lw) lg 2
      | n when n < 13 -> arrange_col (Array.of_list lw) lg 3
      | n when n < 17 -> arrange_col (Array.of_list lw) lg 4
      | n when n < 21 -> arrange_col (Array.of_list lw) lg 5
      | n when n < 25 -> arrange_col (Array.of_list lw) lg 6
      | n -> arrange_col (Array.of_list lw) lg 10

(** arrange_col ws l n : essaie d'arranger le tableau ws de l strings en n colonnes,
 ** et si ce n'est pas possible, il diminue le nombre de colonnes.
 **)
and arrange_col ws l n = 
  if n < 2 then Array.iter (fun x ->  prerr_endline ("\t"^x)) ws
  else acol ws l n

(** idem arrange_col mais on sait que le nombre de colone <> 1 **)
and acol ws lg n = 
  let nb_in_col = if lg mod n = 0 then lg/n else lg/n+1 in (* nb d'elements max dans une colone *)
  let cols = Array.make_matrix n nb_in_col "" in           (* tableau de colonnes *)
    begin
      fill_cols cols ws n lg;                    (* on remplit le tableau des colones *)
      if check_fit cols then print_cols cols     (* on verifie que ca tient dans une ligne *)
      else arrange_col ws lg (n-1)               (* sinon on essaye avec moins de colonne *)
    end

and fill_cols cols ws nbcol lg =
  let i = ref 0 and j = ref 0 in
    for k = 0 to lg-1 do
      (if !i = nbcol then (i := 0; incr j));
      cols.(!i).(!j) <- ws.(k); 
      incr i
    done
    

and check_fit cols = 
  max_char_in_line > (Array.fold_left (fun acc col -> acc + lg_separe_col + (max_in_col col)) 0 cols)

(* largeur max d'un mot dans une colonne *)
and max_in_col col = Array.fold_left (fun acc x -> max acc (String.length x)) 0 col

(* Impression du tableau de colonne *)
and print_cols cols =
  (** on pade les chaines **)
  for i = 0 to (Array.length cols) - 1 do
    let mcol = max_in_col cols.(i) in
      for j = 0 to (Array.length cols.(i)) - 1 do
	cols.(i).(j) <- pad_name mcol cols.(i).(j)
      done
  done;
  (** on imprime **)
  for j = 0 to (Array.length cols.(0)) - 1 do
    for i = 0 to  (Array.length cols) - 1 do
      Printf.fprintf stderr "%s%s" separe_col cols.(i).(j)
    done;
    prerr_newline()
  done
;;





let start_completion () = 
  (* On rajoute les mots cles *)
  Hash.iter (fun c v -> add_word c) Keyword.keyword_table;
  Hash.iter (fun c v -> add_word c) Keyword.commande_table;
  (* On rajoute les fonctions prdfinies du noyau *)
  Hash.iter (fun c v -> add_word c) Environnement.var_tbl;
  (* les parametres *)
  add_word "trace_opt";
  add_word "delaunay_opt";
  add_word "trace_input";
  add_word "abort_on_error";
  add_word "match_random" ; 
  add_word "exit_on_error" ; 
  add_word "silent_error" ; 
  add_word "top_level" ; 
  (* On rajoute a pour le fun *)
  add_word "giavitto"; add_word "Giavitto";
  add_word "michel";   add_word "Michel";
  add_word "Jean-Louis"; add_word "Olivier";
;;


(** On permet d'enregister des mots supplementaire dans le dico de completion 
 **)
Global.add_completion_word := add_word
;;
