(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: eval.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2008-04-08 21:05:41 $
 * Version: $Revision: 1.7 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *          
 *          
 *
 *=========================================================================
 *)


open Lexing;;
open Unix;;
open List;;
open Global;;
open Valeur;;
open Terme;;
open Printer;;
open Environnement;;
open Combinatorise;;
open Optimise_combinateur;;
open Compile;;
open Dispatch_sig;;
open Parser;;

__register_version "$RCSfile: eval.ml,v $" "$Revision: 1.7 $" "$Date: 2008-04-08 21:05:41 $";;


let fun_input_char = ref Ledit.input_char;;




let trap_error s = 
  Pervasives.flush_all();
  if !Global.exit_on_error 
  then
    begin
      prerr_endline ("\nErrors are fatal ("^s^").\nAbort."); 
      exit(1)
    end
  else 
    begin
      (if not !Global.silent_error 
       then Printf.eprintf "\nError:\n\t%s\nis ignored. Jump at top-level.\n\n" s); 
      false
    end
;;



let top_level_exception_handler with_end_message toplevel =
  try 
    toplevel()
  with
    | Lexer.Eof               -> if (with_end_message) then prerr_endline "Good bye.\n"; true
    | Lexer.Unknown_character -> prerr_endline "Unknown character \n"; false
    | Parsing.Parse_error     -> prerr_endline "Parser error. Please try again. \n"; trap_error "Error: parser error" 
    | Sys.Break               -> prerr_endline "\n\nRestart.\n"; false
    | Erreur s                -> trap_error s
    | BreakTransformation _   -> trap_error "return not catched (do you have used a return statment outside a transformation ?)"
    | MgsAssertionFailed      -> prerr_endline ("\nAssertion failed.\nAbort."); exit(1)
    | MGSExn (s,v)            -> trap_error ("MGS exception not catched: "^s^" "^(to_string(prsvalue v))^"\n")
    | exc                     -> 
	Printf.eprintf "\nInternal error (?):\n\tException not catched: %s\n" 
	(Printexc.to_string exc);
	flush Pervasives.stdout; 
	raise exc; 
	true
;;

let rec main () = 
  let finito = ref false
  and arg_set = ref false
  in 
    if (97 <> Char.code 'a')  (* 129 en EBCDIC  *)
    then (Printf.eprintf "BAD ERROR - Not running on an ASCII based computer...\n"; exit(-1));

    (* FIXME Stub_delaunay.usage();*)
    (* FIXME Stub_nauty.usage();*)
    (* FIXME Core_gmap.usage();*)

    banner();
    Arg.parse 
	[("-f"    ,       Arg.String (function s -> ((try
							eval_a_file s
						      with _ -> ());
						     exit 0)),
	  "\t to evaluate a file");
	 ("-v"    ,       Arg.Unit   (function () -> Printf.printf "%s\n" (try
									     fst (Hashtbl.find (__get_versions()) "valeur.ml")
									   with Not_found -> (prerr_string "\nInternal error on requesting version of mgs\n";
												exit (-1)));
					  exit 0),
	    "\t current version of mgs");
	 ("-gui"    ,       Arg.Unit   (function () -> without_gui := false),
	    "\t running with GUI");

	 ("-args",          Arg.String (function s -> (let s = s^" " in 
						       let nb_blancs = ref 0 
						       in
							 begin
							   for i = 0 to ((String.length s)-1)
							   do
							     if (s.[i] = ' ') then incr nb_blancs
							   done;                                      (* (!nb_blancs+1)/2 est le nombre de couples -opt opt_val *)
							   let tab   = Array.make !nb_blancs (VAL_undef "")
							   and debut = ref 0
							   and ptr   = ref 0
							   in
							     for i = 0 to !nb_blancs - 1
							     do
							       begin
								 try
								   let pos_fin = String.index_from s !debut ' '
								   in
								     tab.(!ptr) <- VAL_string (String.sub s !debut (pos_fin - !debut));
								     incr ptr;
								     debut := pos_fin + 1
								 with Not_found -> (Printf.eprintf "Error: Unatended error in string parsing...";
										    exit(-1))
							       end
							     done;
							     Environnement.addenv_impure "'args" (VAL_seq (tab, Type_seq.seq_desc));
							     arg_set := true;
							 end))								
	    
	 , 
	 " giving the mgs value args some optional value");

	]
	(fun _ -> ())
	"Available options : ";

    if (not !arg_set) then Environnement.addenv_impure "'args" (VAL_undef "args");
    if !without_gui 
    then
      begin
	Ledit.set_prompt "mgs> ";
	Ledit.set_edit();
	Ledit.set_max_len 132;
	Completion.start_completion()
      end
    else fun_input_char := Pervasives.input_char;

    let termine = ref false
    and lexbuf = Lexing.from_function read_a_line
    in
      (try
	 let f = ".mgs"
	 in
	   Pervasives.close_in (Pervasives.open_in f); (* Ceci teste l'existence du fichier ... *)
	   eval_a_file f
       with _ -> (try
		    let f = "/etc/mgsrc"
		    in
		      Pervasives.close_in (Pervasives.open_in f);
		      eval_a_file f
		  with _ -> ()));
      while (!termine = false)
      do
	termine :=
	match !Valeur.top_level with
	  | None -> analyse_et_exec true lexbuf
	  | Some f -> top_level_exception_handler true (function () -> Auxfct.bool_from_val(f (Valeur.mgsundef)))
      done;
      if !without_gui  then Ledit.unset_edit();
      Input_output.close_all();
      exit 0

and eval_a_file n = 
  try
    let chan = Pervasives.open_in n 
    and fin  = ref false in
    let buf  = Lexing.from_channel chan
    in
      while (!fin = false)
      do
	fin := analyse_et_exec false buf
      done;
      close_in chan
  with Sys_error e -> raise (Erreur ("file "^n^" cannot be opened for input : "^e))

and read_a_line str nb=
  begin
    try
      let c = ref ' '
      and i = ref 0
      in while (((c := (!fun_input_char Pervasives.stdin); !c) <> '\n') & (!i < nb))
	 do
	   String.set str !i !c;
	   incr i
	 done;
	if ((!c = '\n') & (!i < nb)) then (String.set str !i '\n'; !i+1) else 	!i
    with Sys.Break | End_of_file -> 0
  end


and analyse_et_exec with_end_message lexbuf =
  let default_toplevel () = 
    begin
      affiche_from_parse (Parser.reader Lexer.token lexbuf);
      print_newline();
      Pervasives.flush_all();
      false
    end
  in top_level_exception_handler with_end_message default_toplevel

and affiche_from_parse = function

  | Ceval e -> 
      (* remettre explicitement  0 les tabulations si jamais on s'etait arret en erreur *)
      Environnement.raz_tabulation(); 
      (* traduction en combinateur et evaluation *)
      traduit_evalue e; ()

  | Cquit   -> raise Lexer.Eof

  | Chelp -> show_help()

  | Cliste  -> (** Listing de toutes les constantes **)
      let file, out = Stub_unix.mkstemp "/tmp/tempXXXXXX" in
      let chan      = Unix.out_channel_of_descr out
      in 
	begin
	  (**
	    Printf.fprintf out "\n(** --- Pure Constantes --- *)\n\n";
	    lst_cte out;
	    Printf.fprintf out "\n(** --- Variable --- *)\n\n";
	    lst_var out;
	  **)
	  Pervasives.set_binary_mode_out chan false;
	  Printf.fprintf chan "\n(** --- %d Bindings --- *)\n\n" (Hash.size var_tbl);
	  Printer_term.listing_bindings chan;
	  Pervasives.close_out chan;
	  Unix.system(more_commande^file);
	  Unix.system("/bin/rm -f "^file);
	  ()
	end

  | Ctype 1     (** Listing des types **)
  | Ctype 2 ->  (** Listing des types collections **)
      let file, out = Stub_unix.mkstemp "/tmp/tempXXXXXX" in
      let chan      = Unix.out_channel_of_descr out
      in 
	begin
	  Printf.fprintf chan "\n(** --- %d Collection Types --- *)\n\n"(Type.number_defined());
	  Type.listing chan;	
	  Pervasives.close_out chan;
	  Unix.system(more_commande^file);
	  Unix.system("/bin/rm -f "^file);
	  ()
	end

  | Ctype 3 ->  (** Listing des record **)
      let file, out = Stub_unix.mkstemp "/tmp/tempXXXXXX" in
      let chan      = Unix.out_channel_of_descr out
      in 
	begin
	  Printf.fprintf chan "\n(** --- %d Record Types --- *)\n\n" (Type_rec.number_defined());
	  Type_rec.listing chan;	
	  Pervasives.close_out chan;
	  Unix.system(more_commande^file);
	  Unix.system("/bin/rm -f "^file);
	  ()
	end

  | Ctype _ -> raise (Erreur "Error: internal eroor: eval: 'Ctype n' not defined")
      
  | Cversion ->  (** Listing des versions des fichiers **)
      Printf.printf 
      "\n(** --- Version:\n ** ---      `%s'\n ** --- of the MGS implementation files\n **)\n" 
      major_version_name;
      let liste = Hashtbl.fold (fun n (v, d) s -> (n, v, d, (Checksum.find n))::s) (__get_versions()) []
      in
	List.iter
	  (function  (nom, vers, date, checksum) -> Printf.printf "  %s\t%s\t%s\t%s\n" (pad_name 23 nom) vers date checksum)
	  (List.sort (fun (n1, _, _, _) (n2, _, _, _) ->  compare n1 n2) liste)


  | Cgc     -> prerr_newline();
               Gc.print_stat Pervasives.stderr;
               prerr_newline();
               prerr_string "Force gc."; flush Pervasives.stderr;
               Gc.compact();
               prerr_newline()

  | Csetparam(s, t) -> 
      begin
	match s with
	  | "abort_on_error"   -> Global.abort_on_error := Auxfct.bool_from_val (eval t)
	  | "exit_on_error"    -> Global.exit_on_error := Auxfct.bool_from_val (eval t)
	  | "silent_error"     -> Global.silent_error := Auxfct.bool_from_val (eval t)
	  | "trace_input"      -> Global.trace_input := Auxfct.bool_from_val (eval t)
	  | "trace_opt"        -> Global.trace_opt := Auxfct.bool_from_val (eval t)
	  | "opt_delaunay"     -> Global.delaunay_opt := (match (eval t)
							with
							    VAL_string s -> s
							  | x            -> (Printf.eprintf  "\nError: parameter delaunay_opt expects a string, got:\n\t";
									     prfvalue Pervasives.stderr x; prerr_endline("\ndelaunay_opt is now set to the empty string");
									     ""));
          | "opt_ac"           -> Instancie_pat_ac.opt_ac := Auxfct.bool_from_val (eval t)
          | "opt_bag_matching" -> Topo_bag.opt_bag_matching := Auxfct.bool_from_val (eval t)
          | "top_level"        -> Valeur.set_top_level (eval t); ()
	  | "match_random"     -> Global.match_random :=
	      begin
		match (eval t) with
		  | VAL_int(i) -> i
		  | x -> Printf.eprintf  "\nError: parameter match_random expects an int, got:\n\t";
		         prfvalue Pervasives.stderr x; prerr_endline("\nmatch_random is now set to 1"); 
		      1
	      end
	  | _ -> Printf.eprintf "Error: commande !set: parameter %s not recognized\n" s
      end

  | Clegal -> prvalue (eval (Id "'copyright")); ()

  | Ccoldef t       -> Type.prtype t; print_newline()
  | Ccoldefl lt     -> List.iter (fun t -> Type.prtype t; print_newline()) lt
  | Cinclude f      -> eval_a_file f
  | Ctrace idlist   -> List.iter Environnement.trace_fct idlist
  | Cuntrace idlist -> List.iter Environnement.untrace_fct idlist

  | Cexp _        -> print_string "!! commande Cexp nyi\n"
  | Carrowdef _   -> print_string "!! commande Carrowdef nyi\n"
  | Cnothing      -> print_string "!! commande Cnothing nyi\n"
  | Cclear _      -> print_string "!! commande Cclear nyi\n"
  | Coutput _     -> print_string "!! commande Coutput nyi\n"
  | Cpositionprinter -> print_string "!! commande Cpositionprinter nyi\n"
  | Ccompil(file, id, t) ->  print_string "!! commande Ccompil nyi\n"

(***
  | _ -> raise (Erreur
		  "Internal error: file eval.ml: construction grammaticale non prise en compte")
***)



and banner () = Printf.printf "%s\n" "
(*--------------------------------------------------------------------
 *
 * MGS : a General Model of Systems              (copyright 2001-2008)
 *
 *                 Olivier Michel      : michel@ibisc.univ-evry.fr
 *                 Jean-Louis Giavitto : giavitto@ibisc.univ-evry.fr
 *                 Antoine Spicher     : aspicher@ibisc.univ-evry.fr
 *                 Julien Cohen        : jcohen@ibisc.univ-evry.fr
 *                 Mailing-list        : mgs@ibisc.univ-evry.fr
 *                 WWW                 : http://mgs.ibisc.univ-evry.fr 
 *
 * IBISC FRE 3190 CNRS, Universite d'Evry Val d'Essonne, Genopole Evry
 *
 *
 * MGS uses third party libraries: gsl, ocamlgraph, qhull, nauty,
 * gmap and ocaml. Refer to the documentation for further informations.
 *
 *
 * (Type   !help;;  + enter  if you are lost.)
 * (Type   esc+tab  to have the completion of the current word.)
 *--------------------------------------------------------------------
*)"

and show_help () =
      let file, out = Stub_unix.mkstemp "/tmp/tempXXXXXX" in
      let chan      = Unix.out_channel_of_descr out
      in
	Printf.fprintf chan "\n
==========================================================================
                                  HELP
--------------------------------------------------------------------------

There is an online manual available at url: 
          http://mgs.lami.univ-evry.fr/Online_Manual
that provides more complete informations about all functions available 
in MGS.


The prompt of the interpreter accepts emacs-like line-editing command 
  ^A for the begining of line, 
  ^E to go to the end of line,
  ^P for the previous line in the history
  ^N for the next in the history, 
  esc+tab for the completion of the current word, 
etc.


Three kinds of sentences are possible at top-level:

   ! <command> ;;

   <type definition (collection or arrow or record or gbf)> ;;

   <expression (including function and transformation definition)> ;; 


--------------------------------------------------------------------------
Commands: (begin with an exclamation mark and end with a double semicolon)

   include <identifier>         include a file
   include \"file name\"        include a file

   help           this help
   legal          legal disclaimer and copyright notice
   quit           command to quit mgs
   exit           command to quit mgs

   list           lists the environment (named functions and variables)
   type           lists all collections definitions 
   trace idlist   trace the calls of the functions in idlist
   untrace idlist untrace the calls of the functions in idlist

   gc             prints the current gc status and forces a new cycle

   version        lists the name and the version of the implementation
                  files of this mgs interpreter

   set <identifier> := <expr>    
                set an interpreter parameter. Currently you can set:

     [name]           [default] [purpose]
      opt_ac           true      Optimization by patterns rewriting for AC matching
      opt_bag_matching true      Optimization topoloical structure for AC matching
      trace_opt        false     Debug trace 
      trace_input      false     Verbose read-eval loop
      top_level        <undef>   set a user-defined top_level
      silent_error     false     No message are emitted by the default 
                                 error handler
      exit_on_error    true      Exit the interpreter on error
      abort_on_error   true      On error, abort evaluation and jump to top-level
      match_random     0         Randomize matching: 
                                  0 = deterministic 
                                  1 = random starting point for path
                                  2 = 1+more random choices in path construction

--------------------------------------------------------------------------\n\n";
	Printf.fprintf chan "\n(** ----- List of current bindings ----- *)\n\n";
	Printer_term.listing_bindings chan; 
	Printf.fprintf chan "\n
--------------------------------------------------------------------------
Copyright and legal notice\n\n";
	prfvalue chan (eval (Id "'copyright"));
	Printf.fprintf chan "\n\n
==========================================================================\n\n\n";
	Pervasives.close_out chan;
	Unix.system(Global.more_commande^file);
	Unix.system(Global.rm_commande^file);
	()
;;

(* ******************************************************************************** *)

Global.analyse_et_exec := analyse_et_exec;;

Random.self_init();;
Sys.catch_break true;;
(* Printf.printf "Le new_gmap retourne : %d\n" (Stub_gmap.new_gmap());; *) 
Printexc.print main ();; 

(*
 let fibo = \x.(if x then 1 else fibo(x-1)+fibo(x-2) fi);;
 
 let acker = \x,y.(if (x == 0) 
                   then (y+1)
                   else (if (y == 0) 
                         then acker((x-1), 1) 
                         else acker((x-1), acker(x, (y-1))) fi) fi);;

;;
*)
