/*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: parser.mly,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2008-05-20 10:13:51 $
 * Version: $Revision: 1.33 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *=========================================================================
 */


%{
  open Global;;
  open Valeur;;
  open Terme;;
  open Type;;
  open Def_gbf;;
  open Type_rec;;
  open Compile;;
  open Dispatch_sig;;
  open Dispatch;;
  open Printer;;
  open Printer_term;;
  open Environnement;;
  open Options;;
  open Topology;;
  open Transformation;;
  open Pattern;;
  open Macro;;

(*  open Type_description;;*)
(*  open Record;;*)


__register_version "$RCSfile: parser.mly,v $" "$Revision: 1.33 $" "$Date: 2008-05-20 10:13:51 $";;


let warning_keyword_state = ref true
;;

%}



%token <int> INT ACELL
%token <string> ID STRING IDENT_ARROW GENID GENID_INV GENDIR QID BQID UNDEF
%token <float> REAL


%token ARROW ARROW_BEGIN ARROW_END PLUS_ARROW ABSTRACT_ARROW PLUS_ABSTRACT_ARROW 
%token DPLUS DTIME MULTMULT
%token ARROW_BEGIN ARROW_END ARROW_SPEC
%token UNDEF ASSERTION_MARK
%token ON FLAT PRIORITY ABSTRACT PARALL
%token ARROW_BEGIN ARROW_END

%token LET ANDDEF IN UPDATE FOLLOWING WHEN NOTIN
%token WHILE DO
%token SWITCH CASE DEFAULT ENDSWITCH
%token SEMI_COLUMN TERMINATOR COMMA COMMA2 COMMA3 
%token INF2 INF3 SUP2 SUP3
%token DOWNARROW UPARROW
%token STATE RECORD TRACE UNTRACE 
%token TRANSFORM TRANSFORM2 FUN PATCH 
%token ID  COLLECTION CONSTRAINT GBF QMF DELAUNAY PROXIMAL
%token EQUAL AT
%token TRUE FALSE
%token PLUS TIME DIV MOD MINUS UNDERSCORE HAT
%token IF THEN ELSE ENDIF
%token TRY WITH
%token AND PIPE PIPEPIPE NOT
%token RAISE
%token CONS 
%token LE LT GE GT EQ EQ3 NEQ LTB GTB
%token DOT COLUMN
%token AS  LIST_COMMA 
%token LBRACKET RBRACKET LPARENT RPARENT LCROCHET RCROCHET BEGIN END
%token LENUMER RENUMER ATEQUAL SHARP
%token LAMBDA RETURN
%token QUIT HELP LIST COMPILE TYPE SETPARAM VERSION CLEAR POSITION OUTPUT GC EVAL RUN TRACE INCLUDE LEGAL


/** Du plus faible au plus fort **/

/* Y a un probleme dans :
 *    f({...}, {...}, {...}) 
 * est-ce un appel d'une fonction a 3 arg, ou bien un appel 
 * d'une fonction a 1 arg (qui est une liste) ?
 * On tranche pour :
 * C'est un appel d'une fonction a 3 arg. Si on veut dire le 
 * contraire, suffit d'ecrire:
 *     f( ({...}, {...}, {...}) ) 
 * Pour trancher il suffit que le bind d'une liste d'arguments soit 
 * plus fort que le bind de la construction d'une liste. Ce qu'on fait 
 * en utilisant %prec ci dessous, et en utilisant le token LIST_COMMA 
 * juste pour indiquer une precedence plus faible que le COMMA
 */


/* %right EQUAL ici si fun (x) = x+ 1; 2 veut dire fun (x) = (x+ 1; 2) */

%right IN

%right B_SEMI_COLUMN
%right SEMI_COLUMN
%right A_SEMI_COLUMN

%right EQUAL /*  ici si fun (x) = x+ 1; 2 veut dire (fun (x) = x+ 1); 2 */
%nonassoc WHILE 
%nonassoc DO
%right TRY
%right WITH
%nonassoc ARROW ABSTRACT_ARROW
%nonassoc CASE
%nonassoc SWITCH
%nonassoc ENDSWITCH
%right UPDATE
%nonassoc AT

%right  DOT_LAMBDA_MARK

%right ASSERTION_MARK

%left FOLLOWING
%nonassoc LTB

%left  INF2 INF3 SUP2 SUP3
%left  OR AND PIPE PIPEPIPE
%nonassoc A_PIPE A_PIPEPIPE
%left  EQ EQ3 NEQ

%right      DOWNARROW

%right     B_COMMA
%right     COMMA3
%right     COMMA2
%right     COMMA
%right     GENDIR
%right     GENID
%right     GENID_INV
%right     A_COMMA 
  
%left      AS


%left      LT LE GT GE

%left      COLUMN
%right     COLUMN2
%right     CONS 

%left      PLUS MINUS DPLUS UPARROW
%left      TIME DIV MOD DTIME
%nonassoc  NOT
%nonassoc  RAISE

%left  SIGN_OP     /* operateur de signe d'un nombre */

%left      LAMBDA

%nonassoc  LBRACKET RBRACKET
%nonassoc  LPARENT  RPARENT
%nonassoc  LCROCHET RCROCHET

%nonassoc  HAT

%left      DOT SHARP


/******************************************************************************/
/******************************************************************************/
 
/* Il faut donner explicitement le module, car c'est un texte
 * qui sera inclus dans parser.mli qui est utilise ailleurs.
 */

%start reader
%type <Terme.commande> reader

%%

reader: prompt user_input { $2 } ;

prompt:  { (*print_string "\n=> ";*) flush stdout } ;

user_input:  
             rec_type_declarations TERMINATOR { Ccoldefl $1 }
          |  expr TERMINATOR                  { Ceval $1    }
          |  commande TERMINATOR              { $1          }
          |  TERMINATOR                       { Cnothing    }
          |  type_declaration TERMINATOR      { $1          }

type_declaration: arrow_spec      { Cexp $1    }
//                | collection      { Ccoldef $1 }
//                | record_spec     { Ccoldef $1 }
//                | gbf_spec        { Ccoldef $1 }
//                | qmf_spec        { Ccoldef $1 }
//                | delaunay_spec   { Ccoldef $1 }
//                | proximal_spec   { Ccoldef $1 }
//                | constraint_spec { Ccoldef $1 }

rec_type_declarations: rec_type_declaration_list  { Macro.make_rec_type_declarations $1 }

rec_type_declaration_list: rec_type_declaration                                  { [$1] }
                         | rec_type_declaration_list ANDDEF rec_type_declaration { $3::$1 }

rec_type_declaration: rec_collection      { $1 }
                    | rec_constraint_spec { $1 }
                    | rec_gbf_spec        { $1 }
                    | rec_proximal_spec   { $1 }
                    | rec_delaunay_spec   { $1 }
                    | rec_qmf_spec        { $1 }
                    | rec_record_spec     { $1 }

user_id: ID { $1 }

id:  user_id { $1 }
   | QID     { $1 }

/*****************************************************************************/
/*****************************************************************************/
/* COLLECTION                                                                */
/*****************************************************************************/
/*****************************************************************************/


//collection: COLLECTION user_id EQUAL id { Type.create_new_color $2 $4 }

rec_collection: COLLECTION user_id EQUAL id { RecTypeDef_coll ($2, $4) }

/*****************************************************************************/
/*****************************************************************************/
/* CONTRAINTES (TYPES DE DONNEES ABSTRAITS)                                  */
/*****************************************************************************/
/*****************************************************************************/

rec_constraint_spec: CONSTRAINT user_id EQUAL opt_pipe constraint_list { RecTypeDef_cstrt ($2, $5) }

//constraint_spec: CONSTRAINT user_id EQUAL opt_pipe constraint_list
//                            { if not (check_unknown_names $5 [$2]) then
//				raise (Erreur "Parser: constraint_spec: unknown !!!");
//			      let t = create_dummy_constraint $2 in
//			      let lcstrt = replace_unknown_type $5 t in
//			      let f = build_constraint_pred lcstrt in
//				t.predicate <- f ;
//				ignore(addfct $2 t.predicate);
//				t
//			    }

opt_pipe: { } | PIPE { }

constraint_list: constraint_elt                      { [$1] }
               | constraint_list PIPE constraint_elt { ($3)::$1 }


constraint_elt: scalar_value                         { CSTRT_scalar (match $1 with Cte(x) -> x | _ -> raise (Erreur "Parser: constraint_elt: impossible 1 !!!")) }
	      | genid                                { CSTRT_scalar (VAL_posgbf(Core_gbf.posgbf_from_genid $1)) }
	      | constraint_contents id constraint_size
		  { if (Hash.mem type_tbl $2)
		    then 
		      begin
		      let t = Hash.find type_tbl $2 
		      in
			if ( (not (coll_type_pred t)) & (($1 <> None) || ($3 <> None)))
			then raise (Erreur "Parser: constraint_elt: constraint_contents and constraint_size only on collection type 1")
			else CSTRT_type (t, $1, $3)
		      end
		    else
		      if (Hash.mem type_tbl ("'"^$2))
		      then 
			begin
			  let t = Hash.find type_tbl ("'"^$2) 
			  in
			    if ( (not (Type.coll_type_pred t)) & (($1 <> None) || ($3 <> None)))
			    then raise (Erreur "Parser: constraint_elt: constraint_contents and constraint_size only on collection type 2")
			    else CSTRT_type (t, $1, $3)
			end
		      else
			if (($1 <> None) || ($3 <> None))
			then CSTRT_unknown ($2, $1, $3)
			else
			  if (Hash.mem var_tbl $2)
			  then CSTRT_var ($2, (!var_env).(fst (Hash.find var_tbl $2)))
			  else 
			    if (Hash.mem var_tbl) (String.concat "" ["'"; $2])
			    then CSTRT_var ($2, (!var_env).(fst (Hash.find var_tbl (String.concat "" ["'"; $2]))))
			    else CSTRT_unknown ($2, $1, $3)
		  }

constraint_contents: 
		                                  { None    } 
	      | LCROCHET constraint_tree RCROCHET { Some $2 }

constraint_size: 
                                    { None    } 
	      | LPARENT INT RPARENT { Some $2 }

constraint_tree: constraint_elt                            { TCSTRT_elt $1       }
	       | NOT constraint_tree                       { TCSTRT_not $2       }
	       | constraint_tree PIPE constraint_tree      { TCSTRT_or  ($1, $3) }
	       | constraint_tree PIPE PIPE constraint_tree { TCSTRT_or  ($1, $4) }
	       | constraint_tree AND constraint_tree       { TCSTRT_and ($1, $3) }
	       | constraint_tree AND AND constraint_tree   { TCSTRT_and ($1, $4) }
	       | LPARENT constraint_tree RPARENT           { $2                  }

/*****************************************************************************/
/*****************************************************************************/
/* Les GBF                                                                   */
/*****************************************************************************/
/*****************************************************************************/


rec_gbf_spec: GBF user_id EQUAL LT  gbf_body GT   { RecTypeDef_gbf ($2, $5) }
            | GBF user_id EQUAL LTB gbf_body GTB  { raise (Erreur "Non abelian GBF not yet implemented") }

//gbf_spec: GBF user_id EQUAL LT  gbf_body GT   { let ret = Type_gbf.create_new_gbf $2
//                                                in
//                                                  Core_gbf.process Abelian $2 $5;
//                                                  ret
//                                              }
//        | GBF user_id EQUAL LTB gbf_body GTB  { raise (Erreur "Non abelian GBF not yet implemented") }


gbf_body: arg_list                                  { ((List.rev $1), [])            }
        | arg_list SEMI_COLUMN gbf_equation_list    { ((List.rev $1), (List.rev $3)) }


gbf_equation_list: equation                         { [$1]   }
                 | gbf_equation_list COMMA equation { $3::$1 }



equation: gbf_expr EQUAL gbf_expr { GBF_equal($1, $3) }
        | gbf_expr                { $1                }


gbf_expr: user_id                  { GBF_gen (Positif, $1)               }
        | MINUS user_id            { GBF_gen (Negatif, $2)               }
        | integer user_id          { GBF_mult($1, GBF_gen (Positif, $2)) }
        | integer TIME user_id     { GBF_mult($1, GBF_gen (Positif, $3)) }
        | LPARENT gbf_expr RPARENT { $2                                  }
        | gbf_expr PLUS gbf_expr   { GBF_op(GBF_plus, $1, $3)            }
        | gbf_expr MINUS gbf_expr  { GBF_op(GBF_moins, $1, $3)           }
        | integer                  { if ($1 <> 0) then raise (Erreur ("Only 0 is permitted in a GBF expression, not "^(string_of_int $1))) else GBF_zero }

/*****************************************************************************/
/*****************************************************************************/
/* Les QMF                                                                   */
/*****************************************************************************/
/*****************************************************************************/

rec_qmf_spec: QMF user_id ON expr { RecTypeDef_qmf($2, to_gmap(Compile.eval $4)) }
	    | QMF user_id         { RecTypeDef_qmf($2, Core_qmf.create_gmap ())  }

//qmf_spec: QMF user_id ON expr { let gmap = to_gmap(Compile.eval $4) in
//				  (Type_qmf.create_new_qmf $2 gmap)
//			      }
//	| QMF user_id         { let gmap = Core_qmf.create_gmap () in
//				  (Type_qmf.create_new_qmf $2 gmap)
//			      }

/********************************/
/* Les expressions de Delaunay  */
/********************************/

rec_delaunay_spec: DELAUNAY LPARENT INT COMMA bool_value RPARENT user_id EQUAL expr
				  { RecTypeDef_del($7, $3, to_bool(val2bool(Compile.eval $5)), (Compile.eval $9)) }

	         | DELAUNAY LPARENT INT RPARENT user_id EQUAL expr
		                  { RecTypeDef_del($5, $3, false, (Compile.eval $7)) }


//delaunay_spec: DELAUNAY LPARENT INT COMMA bool_value RPARENT user_id EQUAL expr { let voronoi = (match $5
//												 with
//												     Cte(VAL_bool x) -> x
//												   | _               -> raise (Erreur "severe error in parser on delaunay...")) in
//										  let ret = Type_del.create_new_del $7
//										  in
//										    Core_del.process $7 $3 (Compile.eval $9) voronoi;
//										    ret 
//										}
//	     | DELAUNAY LPARENT INT RPARENT user_id EQUAL expr { let ret = Type_del.create_new_del $5
//								 in
//								   Core_del.process $5 $3 (Compile.eval $7) false;
//								   ret 
//							       }



/********************************/
/* Les expressions de Proximal  */
/********************************/

rec_proximal_spec: PROXIMAL user_id EQUAL expr { RecTypeDef_prox ($2, (Compile.eval $4)) }

//proximal_spec: PROXIMAL user_id EQUAL expr { let ret = Type_prox.create_new_prox $2
//                                             in
//                                               Core_prox.process $2 (Compile.eval $4);
//                                               ret }



/********************************/
/* Graphes generalises          */
/********************************/

graphe: LCROCHET graph_liste RCROCHET { Cte (VAL_grf(Core_graph.process (List.rev $2), Type_graph.graph_desc)) }


graph_liste: base_graph                        { [$1]   }
          | graph_liste SEMI_COLUMN base_graph { $3::$1 }

base_graph: INT UPDATE expr                        { $1, (Compile.eval $3), []            }
          | INT UPDATE expr COLUMN GT sommet_liste { $1, (Compile.eval $3), (List.rev $6) }

sommet: INT                   { $1, "" }
      | INT AS user_id        { $1, $3 }  

sommet_liste: sommet                    { [ $1 ] }
          |   sommet_liste COMMA sommet { $3::$1 }


/*****************************************************************************/
/*****************************************************************************/
/* COMMANDE                                                                  */
/*****************************************************************************/
/*****************************************************************************/

commande: QUIT                              { Cquit               }
        | HELP                              { Chelp               }
        | LEGAL                             { Clegal              }
        | COMPILE STRING id expr            { Ccompil($2, $3, $4) }
        | LIST                              { Cliste              } 
        | TYPE                              { Ctype 1             } 
        | COLLECTION                        { Ctype 2             } 
        | recordlist                        { Ctype 3             } 
        | SETPARAM id UPDATE expr           { Csetparam($2, $4)   } 
        | VERSION                           { Cversion            } 
        | CLEAR user_id                     { Cclear $2           }
        | POSITION                          { Cpositionprinter    }
        | OUTPUT integer                    { Coutput (($2 != 0)) }
        | EVAL expr                         { Ceval $2            }
        | TRACE idlist                      { Ctrace $2           }
        | UNTRACE idlist                    { Cuntrace $2         }
        | INCLUDE expr                      { Cinclude (to_string (Compile.eval $2)) }
        | GC                                { Cgc                 }

idlist: id        { [ $1 ] }
      | idlist id { $2::$1 }

recordlist: RECORD {}
          | STATE  { if !warning_keyword_state then 
		       Printf.printf "\nWARNING: the keyword 'STATE' is now obsolete: use 'RECORD' instead\n\n";
		     warning_keyword_state := false
		   }

/*****************************************************************************/
/*****************************************************************************/
/* RECORD                                                                     */
/*****************************************************************************/
/*****************************************************************************/

rec_record_spec:  RECORD user_id EQUAL rec_state_body { RecTypeDef_rec($2, $4, get_all_dependances $4) }
     
rec_state_body:    id                                 { [RecordType $1] }
              |    LBRACKET  RBRACKET                 { [Fields []]     }
              |    LBRACKET rec_sid_list RBRACKET     { [Fields $2]     }
              |    rec_state_body PLUS rec_state_body { $1 @ $3         }

rec_sid_list: rec_field_specifier                    { [$1] }
            | rec_sid_list COMMA rec_field_specifier { $3::$1 }

rec_field_specifier: user_id                      { ($1, Present) }
               | user_id EQUAL expr %prec A_COMMA { ($1, WithValue (Compile.eval $3)) }
               | user_id COLUMN constraint_tree   { ($1, WithCstrt $3) }
               | NOT user_id                      { ($2, Absent) }
               | user_id NEQ expr                 { ($1, WithoutValue (Compile.eval $3)) }



//record_spec:  RECORD user_id EQUAL state_body { Type_rec.process $2 $4 }

     
//state_body:    id                          { [RecordType $1] }
//          |    LBRACKET  RBRACKET          { [Fields []]     }
//          |    LBRACKET sid_list RBRACKET  { [Fields $2]     }
//          |    state_body PLUS state_body  { $1 @ $3         }

//sid_list:   field_specifier                { [$1] }
//          | sid_list COMMA field_specifier { $3::$1 }

//field_specifier: user_id                          { ($1, Present) }
//               | user_id EQUAL expr %prec A_COMMA { ($1, WithValue (Compile.eval $3)) }
//               | user_id COLUMN constraint_tree   { ($1, WithType  (VAL_funct((function v -> VAL_bool(build_constraint_pred_tree v $3))))) }
//               | NOT user_id                      { ($2, Absent) }
//               | user_id NEQ expr                 { ($1, WithoutValue (Compile.eval $3)) }


/*****************************************************************************/
/*****************************************************************************/
/* FONCTION                                                                  */
/*****************************************************************************/
/*****************************************************************************/

arg_list:            { [] }
        |  id_list   { $1 }


fonction: FUN user_id LPARENT arg_list RPARENT EQUAL expr 
          { 
            check_new_fct_name_before_definition $2;
            let fct = (if $4 = [] then Abs(new_name "dummy_arg_for_void_fct", $7) else make_abs $7 $4)
            in Set($2, fct)
          }

          /* forme:  fun f [a=0](x, y, z) = x*a+y+z  */
          | FUN user_id LCROCHET blist RCROCHET LPARENT arg_list RPARENT EQUAL expr 
          { 
            check_new_fct_name_before_definition $2;
            let body = process_option_fct $2 $4 $10 in
            let fct = (if $7 = [] then Abs(new_name "dummy_arg_for_void_fct", body) else make_abs body $7)
            in Set($2, fct)
          }

          /* forme:   \x, y, z.x*a+y+z */
        | LAMBDA arg_list DOT expr %prec DOT_LAMBDA_MARK {
            if $2 = [] then Abs(new_name "dummy_arg_for_void_lambda", $4) else make_abs $4 $2 }

                


id_list:   id                { [$1] }
        |  id_list COMMA id  { $3 :: $1 }  /* Si on inverse le sens, il faut aussi changer les List.rev dans fonctions */


real: REAL                     { $1     }
    | MINUS REAL %prec SIGN_OP { -.($2) }
/*    | PLUS REAL  %prec SIGN_OP { $2     }     /* Ce cas n'arrive jamais... */


acell: ACELL  { $1 }

integer: INT                     { $1    }
       | MINUS INT %prec SIGN_OP { -($2) }
/*       | PLUS INT  %prec SIGN_OP { $2    }    /* Idem                      */
;


/*****************************************************************************/
/*****************************************************************************/
/* GenID                                                                     */
/*****************************************************************************/
/*****************************************************************************/


genid: GENID       { let v = String.sub $1 1 ((String.length $1)-2) in GBF_gen (Positif, v) }
     | GENID_INV   { let v = String.sub $1 1 ((String.length $1)-2) in GBF_gen (Negatif, v) }


/*****************************************************************************/
/*****************************************************************************/
/* EXPRESSION                                                                */
/*****************************************************************************/
/*****************************************************************************/


bloc: LPARENT expr RPARENT         { $2  }
    | BEGIN expr END               { $2  }

expr: bloc                         { $1  }
    | expr COMMA expr              { op_apply2 "'join"  $1 $3  }
    | expr SEMI_COLUMN expr        { Fby ($1, $3) }
    | WHILE LPARENT expr RPARENT DO expr { op_apply2 "'while"  
					     (Abs(new_name "dummy_arg_for_while_lambda1",$3))
					     (Abs(new_name "dummy_arg_for_while_lambda2", $6)) }

    | switch_expr                  { $1 }
   
    | HAT id                       { op_apply1 "'pos" (Id $2) }
    | NOT expr                     { op_apply1 "'not" $2 }
    | id COLUMN LPARENT RPARENT    { Cte(empty_from_type_name $1) }
    | LPARENT RPARENT COLUMN id    { Cte(empty_from_type_name $4) }
    | id COLUMN PIPE int_list GT   { Cte(VAL_posgbf((Array.of_list $4), (Core_gbf.find_gbf_def $1))) }

    | id COLUMN LENUMER valpos_list RENUMER { op_apply2 "'extension" (Cte (VAL_string $1)) (Macro.make_valposlist $4) }
    | LENUMER valpos_list RENUMER COLUMN id { op_apply2 "'extension" (Cte (VAL_string $5)) (Macro.make_valposlist $2) }

    | expr LPARENT expr_list RPARENT                   { Macro.make_application $1 (OptionList []) $3 }
    | expr LCROCHET expr RCROCHET LPARENT expr RPARENT { Macro.make_application $1 (OptionItere $3) [$6] }
    | expr LCROCHET TIME RCROCHET LPARENT expr RPARENT { Macro.make_application $1 OptionFixPoint [$6] }
    | id LCROCHET optionlist RCROCHET LPARENT expr_list RPARENT { Macro.make_application (Id $1) $3 $6 }

/*  |  expr  expr                                       { Nyi } */

/***************************/
/* Les expressions de GBF  */
/***************************/

    | genid                       { Cte (VAL_posgbf(Core_gbf.posgbf_from_genid $1)) }
    | expr FOLLOWING expr         { op_apply2 "'following" $1 $3                    }
    | expr genid expr %prec GENID { Nyi                                             }

/***************************/
/* Les operateurs binaires */
/***************************/

    | expr PLUS expr                 { op_apply2 "'+"  $1 $3 }
    | expr TIME expr                 { op_apply2 "'*"  $1 $3 } 
    | expr DTIME expr                { op_apply2 "'**" $1 $3 }
    | expr DIV  expr                 { op_apply2 "'/"  $1 $3 }
    | expr MOD  expr                 { op_apply2 "'mod"  $1 $3 }
    | expr MINUS expr                { op_apply2 "'-"  $1 $3 }
    | expr AND AND expr              { If($1, App(Id "'val2bool", [$4]), Cte(VAL_bool false)); }
    | expr AND expr                  { If($1, App(Id "'val2bool", [$3]), Cte(VAL_bool false)); }
    | expr PIPEPIPE  expr            { If($1, Cte(VAL_bool true), App(Id "'val2bool", [$3])); }
    | expr CONS expr                 { op_apply2 "'::"  $1 $3 }
    | expr LE expr                   { op_apply2 "'<="  $1 $3 }
    | expr LT expr                   { op_apply2 "'<"   $1 $3 }
    | expr GE expr                   { op_apply2 "'>="  $1 $3 } 
    | expr GT expr                   { op_apply2 "'>"   $1 $3 } 
    | expr EQ expr                   { op_apply2 "'=="  $1 $3 }
    | expr EQ3 expr                  { op_apply2 "'===" $1 $3 }
    | expr NEQ expr                  { op_apply2 "'!="  $1 $3 }
    | expr INF2 expr                 { op_apply2 "'<<"  $1 $3 }
    | expr INF3 expr                 { op_apply2 "'<<<" $1 $3 }
    | expr DOWNARROW expr            { op_apply2 "'\\/" $1 $3 }
    | expr UPARROW expr              { op_apply2 "'/\\" $1 $3 }

    | expr DOT LPARENT expr RPARENT   { op_apply2 "'take" $1 $4 }
    | expr SHARP  expr                { op_apply2 "'count" $3 $1 }

    | IF expr THEN expr ELSE expr ENDIF { If($2, $4, $6) }

    | RETURN LPARENT expr RPARENT       { Macro.wrap_return($3) }

    | record                            { $1 }
    | graphe                            { $1 }
    | user_id UPDATE expr               { Set($1, $3) }

    | local_def                         { $1 }
    | id %prec IN                       { Id $1 } /* a cause de id:() */
    | expr DOT user_id                  { op_apply2 "'take" $1 (Cte (VAL_symbol ("`"^$3))) }
    | ASSERTION_MARK expr               { op_apply2 "'!!" (Cte(prsterm $2)) $2 }
    | fonction                          { $1 }
    | transformation                    { $1 }
    | transformation2                   { $1 }
    | patch                             { $1 }
    | scalar_value                      { $1 }
    | SHARP INT scalar_value            { (match $3
					   with 
					       Cte v -> Cte (VAL_seq((Array.make $2 v), Type_seq.seq_desc))
					     | _     -> raise (Erreur ("PARSER: case cannot happen...")))}
    | UNDEF                             { Cte(VAL_undef($1))  (* ici et non dans scalar_value: ca resoud un conflit (?) *) }
    | patch_rhs_spec                    { $1 }

    | RAISE LPARENT BQID expr RPARENT   { op_apply2 "'raise" (Cte (VAL_symbol $3)) $4 }
    | RAISE LPARENT BQID RPARENT        { op_apply2 "'raise" (Cte (VAL_symbol $3)) (Cte (VAL_seq([||],Type_seq.seq_desc))) }
    | RAISE BQID                        { op_apply2 "'raise" (Cte (VAL_symbol $2)) (Cte (VAL_seq([||],Type_seq.seq_desc))) }
    | TRY expr WITH pipe_opt try_rules  { make_try $2 $5 }

pipe_opt:      {} 
    |     PIPE {}

try_rules:
    | try_rule                { [$1]   }
    | try_rules PIPE try_rule { $3::$1 }

try_rule: BQID LPARENT id_list RPARENT ABSTRACT_ARROW  expr { ($1,$3,$6) }
        | BQID LPARENT RPARENT ABSTRACT_ARROW  expr         { ($1,[],$5) }
        | BQID ABSTRACT_ARROW  expr                         { ($1,[],$3) }


optionlist: blist   { OptionList $1  }
          | integer { OptionItere (Cte (VAL_int $1)) }
          | TIME    { OptionFixPoint }


expr_list:                                   { []     }
        | expr %prec A_COMMA                 { [$1]   }
        | expr_list COMMA expr %prec A_COMMA { $3::$1 }

valpos_list:                                              { []           }
        | expr EQUAL expr %prec A_COMMA                   { [($1, $3)]   }
        | expr AT    expr %prec A_COMMA                   { [($3, $1)]   }
        | valpos_list COMMA expr EQUAL expr %prec A_COMMA { ($3, $5)::$1 }
        | valpos_list COMMA expr AT    expr %prec A_COMMA { ($5, $3)::$1 }

int_list:
	| INT                { [$1]   }
	| int_list COMMA INT { $3::$1 }


/*****************************************************************************/
/*****************************************************************************/
/* LET ... IN ...                                                            */
/*****************************************************************************/
/*****************************************************************************/



local_def: LET localdef IN expr         { make_let $4 $2 }

localdef: id EQUAL expr                 { [($1,$3)]   }
        | localdef ANDDEF id EQUAL expr { ($3,$5)::$1 }


/*****************************************************************************/
/*****************************************************************************/
/* SWITCH ... CASE xx : .... ENDSWITCH                                       */
/*****************************************************************************/
/*****************************************************************************/

switch_expr: SWITCH expr  list_case ENDSWITCH { make_switch $2 $3 }

list_case: case                               { [$1]              }
	 | list_case case                     { $2::$1            }

case: CASE scalar_value COLUMN  expr          { ((Some $2),$4)    }
    | DEFAULT COLUMN  expr                    { (None,$3)         }

/*****************************************************************************/
/*****************************************************************************/
/* TYPES DE BASE ET CAST                                                     */
/*****************************************************************************/
/*****************************************************************************/

scalar_value: integer    { Cte(VAL_int $1)     }
            | real       { Cte(VAL_float $1)   }
            | STRING     { Cte(VAL_string $1)  }
	    | bool_value { $1                  }
	    | BQID       { Cte(VAL_symbol $1)  }
	    | acell      { Cte(
			     try
			       VAL_acell (Core_achain.get_acell_from_uid $1)
			     with Not_found -> raise (Erreur ("The acell /"^(string_of_int $1)^"/ doesn't exist."))
			   ) }

bool_value:
            | TRUE      { Cte(VAL_bool true)  }
            | FALSE     { Cte(VAL_bool false) }

/*****************************************************************************/
/*****************************************************************************/
/* RECORD
/*****************************************************************************/
/*****************************************************************************/

record:  LBRACKET rblist RBRACKET  { make_record $2 record_desc }

rblist:                                              { []             }  /* x = 2, y = 3 */
        | id EQUAL expr %prec A_COMMA                { [(VAL_symbol ("`"^$1), $3)]     }
        | id  %prec A_COMMA                          { [(VAL_symbol ("`"^$1), Cte(VAL_undef ""))]   }
        | rblist COMMA id EQUAL expr %prec A_COMMA   { (VAL_symbol ("`"^$3), $5)::$1   }
        | rblist COMMA id  %prec A_COMMA             { (VAL_symbol ("`"^$3), Cte(VAL_undef ""))::$1 }

blist:                                               { []             }  /* x = 2, y = 3 */
        | id EQUAL expr %prec A_COMMA                { [($1, $3)]     }
        | id  %prec A_COMMA                          { [($1, Cte(VAL_undef ""))]   }
        | blist COMMA id EQUAL expr %prec A_COMMA    { ($3, $5)::$1   }
        | blist COMMA id  %prec A_COMMA              { ($3, Cte(VAL_undef ""))::$1 }

/*****************************************************************************/
/*****************************************************************************/
/* FLECHES                                                                   */
/*****************************************************************************/
/*****************************************************************************/

arrow_spec: ARROW_SPEC IDENT_ARROW EQUAL arrow_body  { Nyi }

arrow_body: arrow_sep_begin arrow_prop_list arrow_sep_end   { Terme.compile_rule_property $2 }

arrow_sep_begin: ARROW_BEGIN { }
        |        LBRACKET    { }


arrow_sep_end: ARROW_END { }
        |      RBRACKET  { }

arrow_prop_list: arrow_property                         { [$1]   }
               | arrow_prop_list COMMA arrow_property   { $3::$1 } 
;

arrow_property: FLAT                         { RP_flat FlatPath   }
              | FLAT EQUAL TRUE              { RP_flat FlatPath   }
              | FLAT EQUAL FALSE             { RP_flat NoFlatPath }
              | PRIORITY EQUAL integer       { RP_priority $3     }
              | TRACE                        { RP_trace None      }
              | TRACE EQUAL integer          { RP_trace (Some $3) }
              | ON EQUAL expr %prec A_COMMA  { RP_on (Compile.eval $3) }
              | PARALL                       { RP_parall          }
              | ABSTRACT                     { RP_abstract        }
	      | ID EQUAL expr                { match $1 with
						 | ("Ci" | "C") -> RP_gillespie (GilCExp (Abs("$dummy_arg_CExp",$3)))
						 | ("Ai" | "A") -> RP_gillespie (GilAExp (Abs("$dummy_arg_AExp",$3)))
						 | ("P"  | "p") -> RP_probability (StochExp (Abs("$dummy_arg_PExp",$3)))
						 | _            -> raise (Erreur "Parser: arrow_property: only C, Ci, A, Ai, P and p are allowed")
		}
		  
/*****************************************************************************/
/*****************************************************************************/
/* TRANSFORMATION                                                            */
/*****************************************************************************/
/*****************************************************************************/

transformation: TRANSFORM transbody                   { make_transformation $2 }
              | TRANSFORM user_id EQUAL transbody     { check_new_fct_name_before_definition $2; Set($2, make_transformation $4) }
              | TRANSFORM  user_id LCROCHET blist RCROCHET EQUAL transbody 
		  { 
		    check_new_fct_name_before_definition $2;
		    let rules = process_option_trans $2 $4 $7 
		    in Set($2, make_transformation rules)		      
		  }

transformation2: TRANSFORM2 cpldim transbody                   { make_transformation2 $2 $3 }
               | TRANSFORM2 cpldim user_id EQUAL transbody     { check_new_fct_name_before_definition $3; Set($3, make_transformation2 $2 $5) }
               | TRANSFORM2 cpldim user_id LCROCHET blist RCROCHET EQUAL transbody 
		   { 
		     check_new_fct_name_before_definition $3;
		     let rules = process_option_trans $3 $5 $8 
		     in Set($3, make_transformation2 $2 rules)		      
		   }

cpldim:                       {(0,1)}
      | LT INT GT             {($2,0)}
      | LT INT COMMA INT  GT  {($2,$4) }


transbody: LBRACKET regle_list opt_sc RBRACKET   { List.rev $2 }
        |  LPARENT pattern arrow a_expr RPARENT  { [(None, $2, $3, $4)] }

regle_list: regle                                  { [$1] }
        |   transformation                         { 
              print_endline "Warning: nested transformation not yet implemented"; [] } 
        |   transformation2                        { 
              print_endline "Warning: nested transformation2 not yet implemented"; [] } 
        |   regle_list SEMI_COLUMN regle           { $3::$1 }
        |   regle_list SEMI_COLUMN transformation  { 
              print_endline "Warning: nested transformation not yet implemented"; [] }
        |   regle_list SEMI_COLUMN transformation2 { 
              print_endline "Warning: nested transformation2 not yet implemented"; [] }

opt_sc:                   {}
          | SEMI_COLUMN   {}





patch: PATCH patchbody                { make_patch $2 }
     | PATCH user_id EQUAL patchbody  { check_new_fct_name_before_definition $2; Set($2, make_patch $4) }
     | PATCH user_id  LCROCHET blist RCROCHET EQUAL patchbody
	 { check_new_fct_name_before_definition $2;
	   let rules = process_option_patch $2 $4 $7 
	   in Set($2, make_patch rules)
	 }


patchbody: LBRACKET pregle_list opt_sc RBRACKET  { List.rev $2 }
        |  LPARENT ppattern arrow a_expr RPARENT { [(None, Patch.reordonne $2, $3, $4)] }


pregle_list: pregle                           { [$1] }
           | pregle_list SEMI_COLUMN pregle   { $3::$1 }


pregle: ppattern arrow a_expr               { (*Patch.print_parse_motif ($1);*)
					      let c = (Patch.normalise_patch $1) in
						(*Patch.print_sorted_motif c ;*)
						let s = (Patch.reordonne $1) in
						  (*Patch.print_motif s ;*)
						  (None, s, $2, $3)
					    }
      | user_id EQUAL ppattern arrow a_expr { (*Patch.print_parse_motif ($3);*)
					      let c = (Patch.normalise_patch $3) in
						(*Patch.print_sorted_motif c ;*)
						let s = (Patch.reordonne $3) in
						  (*Patch.print_motif s ;*)
						  (Some $1, s, $4, $5)
					    }

/* FIXME:
     Dcrire la structure de la liste,
     remplacer pos par ^ (aussi pour les transformations), DONE
     le ~ n apparait que sur un crochet (on ne filtre les crochets non tild), DONE
*/
ppattern: motif                         { let (n,(g,c),crochet) = $1 in ([(n,crochet,g)], (List.map (fun (con,x) -> (con,n,x) ) c)) }
        | NOT motif                     { let (n,(g,c),crochet) = $2 in ([(n,false,g)], (List.map (fun (con,x) -> (con,n,x) ) c)) }
        | ppattern connecteur motif     { let (mlist, conlist) = $1 in
				          let (n,(g,c),crochet) = $3
				          in ((n,crochet,g)::mlist, ($2, (Patch.fst3 (List.hd mlist)), n)::((List.map (fun (con,x) -> (con,n,x) ) c)@conlist)) }
        | ppattern connecteur NOT motif { let (mlist, conlist) = $1 in
				          let (n,(g,c),crochet) = $4
				          in ((n,false,g)::mlist, ($2, (Patch.fst3 (List.hd mlist)), n)::((List.map (fun (con,x) -> (con,n,x) ) c)@conlist)) }

/* FIXME:
	>* <*, rajouter MULT dans pguard,
	|exp>, <exp| incidence (vrifier que exp est tjrs positif - warning et abort),
	  c1:[] <* _:[dim=dimension(^c1)+k] >* c2:[dim=dimension(^c1)] ????
*/
connecteur: { NoConnection} 
          | LT { FaceOf }   /*| INF2 { TfaceOf   } | INF3 { TfaceOf   }*/
          | GT { CoFaceOf } /*| SUP2 { TcoFaceOf } | SUP3 { TcoFaceOf }*/

/* filtrage d'une seule entit */
motif: UNDERSCORE                                      { (new_name "dummy_patch_variable", ([],[]), false) }
     | user_id                                         { ($1, ([],[]), false)                               }
     | UNDERSCORE COLUMN LCROCHET pguard_list RCROCHET { (new_name "dummy_patch_variable", $4, true) }
     | user_id COLUMN LCROCHET pguard_list RCROCHET    { ($1, $4, true)                               }


pguard_list:                                           { [],[] }
           | pguard                   %prec A_COMMA    { match $1 with
							   | Some g, None -> [g],[]
							   | None, Some c -> [],c
							   | None, None
							   | Some _, Some _ -> raise (Erreur ("Parse error: pguard: beuha !!!"))
						       }
           | pguard_list COMMA pguard %prec A_COMMA    { let (lg,lc) = $1 in
							   match $3 with
							     | Some g, None -> (g::lg),lc
							     | None, Some c -> lg,(c@lc)
							     | None, None
							     | Some _, Some _ -> raise (Erreur ("Parse error: pguard: beuha !!!"))
						       }

/* FIXME: rorganiser tout ca,
   permettre permutation argument (in faces , face of),
   incidence,
   transitif(co)faces,
   
*/
pguard:
      | id IN LPARENT idlist2 RPARENT %prec A_COMMA  { match $1 with
					   | "faces"   -> None, Some (List.map (fun x -> (CoFaceOf, x)) $4)
					   | "cofaces" -> None, Some (List.map (fun x -> (  FaceOf, x)) $4)
					   | _ -> raise (Erreur ("Parse error: pguard: beuha !!! (1)"))
	                               }
      | id EQUAL expr %prec A_COMMA    { if (List.mem $1 (["dim";"val"])) then
					   (Some (Pred($1, $3)), None)
					 else
					   raise (Erreur ("Parse error: pguard: beuha !!! (2)"))
				       }
      | expr %prec A_COMMA             { (Some(Test $1), None)      }

idlist2: id                             { [ $1 ] }
       | idlist2 COMMA id %prec A_COMMA { $3::$1 }

patch_rhs_spec: expr COLUMN LCROCHET patch_new_elem_list RCROCHET { make_patchRhsSpec $1 $4 }

patch_new_elem_list:                                { []     }
              | pnew_elem                           { [$1]   }
	      | pnew_elem COMMA patch_new_elem_list {
		  if (List.mem_assoc (fst $1) $3) then
		    raise (Erreur ("Parse error: patch_new_elem_list: attribute "^(fst $1)^" is specified too many times"))
		  else
		    $1::$3
		}

pnew_elem:
         | id EQUAL expr %prec A_COMMA {
	     match $1 with
	       | "d"  | "dim"     | "dimension" -> ("d",  $3)
	       | "f"  | "faces"   | "face"      -> ("f",  $3)
	       | "cf" | "cofaces" | "coface"    -> ("cf", $3)
	       | "v"  | "val"     | "value"     -> ("v",  match $3 with Cte(VAL_undef(_)) -> Cte(VAL_symbol "PatchUndefVal") | v->v)
	       | _    -> raise (Erreur ("Parse error: pnew_elem: invalid attribute in a patch rhs specification"))
	   }

/*****************************************************************************/
/*****************************************************************************/
/* REGLE                                                                     */
/*****************************************************************************/
/*****************************************************************************/

regle: pattern arrow a_expr                { (None, $1, $2, $3) }
     | user_id EQUAL pattern arrow a_expr  { (Some $1, $3, $4, $5) }


a_expr: expr %prec A_SEMI_COLUMN           { $1 }



arrow: ARROW               { Terme.emtpy_rule_property() } /* =>  */
     | ABSTRACT_ARROW      { Terme.emtpy_rule_property() } /* ->  */
     | PLUS_ARROW          { Terme.emtpy_rule_property() } /* +=> */
     | PLUS_ABSTRACT_ARROW { Terme.emtpy_rule_property() } /* +-> */
     | IDENT_ARROW         { raise (Erreur "PARSER: not yet implemented naming an arrow") }
     | arrow_body          { $1                          }


pattern: UNDERSCORE              { BlankPat }                                        /*  _                */
       | user_id                 { AsPat($1, BlankPat) }                             /*  x                */
       | UNDEF                   { NotYetPat }                                       /*  <undef>          */
       | LPARENT pattern RPARENT { $2     }                                          /*  ( ... )          */
       | pattern AS user_id      { AsPat($3, $1) }                                   /*  ... as x         */
       | pattern DIV expr        { GuardedPat($1, $3) }                              /*  ... / P(...)     */
       | filter_state            { RecordPat $1 }                                    /*  { x, ~y, r = 2 } */
       | pattern pat_iteration   { IterPat($2, $1) }                                 /*  [...] +          */
       | pattern PIPE pattern    { OrPat($1, $3) }                                   /*  [...] | [...]    */
       | SHARP INT scalar_value  { CtePat ((Compile.eval $3), $2) }                  /*  $1 fois un scalaire  */
       | INT scalar_value        { CtePat ((Compile.eval $2), $1) }                  /*  ecriture alternative */
       | scalar_value            { CtePat ((Compile.eval $1), 1) }                   /*  5 ou "abc"       */
       | pattern COLUMN id       { let n = new_name "$patid" in                      
				     GuardedPat(AsPat(n, $1), op_apply1 $3 (Id n)) } /*  ... : P          */

       | pattern COMMA pattern                                 /* ..., [...]                 */
           { CommaPat(AllDirections, None, $1, $3) }
       | pattern COMMA AS user_id pattern                      /* ..., as d [...]            */
           { CommaPat(AllDirections, (Some $4), $1, $5) }
       | pattern LPARENT COMMA AS user_id RPARENT pattern      /* idem                       */ 
	   { CommaPat(AllDirections, (Some $5), $1, $7) }
       | pattern genid pattern  %prec GENID                    /* ... |nord> [...]           */
           { CommaPat(SomeDirections([| VAL_posgbf(Core_gbf.posgbf_from_genid $2) |]), None, $1, $3) }
       | pattern LTB direction GTB  pattern %prec GENID        /* ... <| a, b |>  [...]      */
           { CommaPat(SomeDirections(Array.of_list $3), None, $1, $5) }
       | pattern LTB direction GTB AS user_id pattern          /* ... <| a, b |> as dir [...]*/
           { CommaPat(SomeDirections(Array.of_list $3), Some $6, $1, $7) }
       | pattern DOWNARROW pattern                             /* ... \/ ...                 */
	   { DownPat($1, $3) }
       | NOT pattern { NotPat($2) }                            /* ~p dconsomme p            */


pat_iteration:
          PLUS                    { PatPlus (AllDirections, FirstFound) }
     |    DPLUS                   { raise (Erreur "pattern iteration ++ not yet implemented") }
     |    TIME                    { PatStar (AllDirections, FirstFound) }
     |    DTIME                   { raise (Erreur "pattern iteration ** not yet implemented") } 
     |    LTB direction GTB PLUS  { PatPlus ((SomeDirections (Array.of_list $2)), FirstFound) }
     |    LTB direction GTB DPLUS { raise (Erreur "pattern iteration |g>++ not yet implemented") }
     |    LTB direction GTB TIME  { raise (Erreur "pattern iteration |g>* not yet implemented") }
     |    LTB direction GTB DTIME { raise (Erreur "pattern iteration |g>** not yet implemented") }

direction: gbf_expr                 { [VAL_posgbf(Core_gbf.gbf_pos_from_gbf_expr $1)]     }
         | direction COMMA gbf_expr { (VAL_posgbf(Core_gbf.gbf_pos_from_gbf_expr $3))::$1 }


filter_state: LBRACKET RBRACKET          { [] }
	 |    LBRACKET fid_list RBRACKET { $2 }

/** renvoie un liste de doublet: nom_variable, (nom_field, spec_field) **/
fid_list: field_pat_spec                    { [$1]   }
        | fid_list COMMA field_pat_spec     { $3::$1 }


field_pat_spec: id                    { (Some $1), ($1, Present) }
              | NOT id                { None, ($2, Absent) }
              | id EorAS id           { (Some $3), ($1, Present) }
              | id EQUAL scalar_value { (Some $1), ($1, WithValue (Compile.eval($3)))   }
              | id COLUMN id          { (Some $1), ($1, WithType (Compile.eval(Id $3))) }
              | id COLUMN id EorAS id { (Some $5), ($1, WithType (Compile.eval(Id $3))) }

EorAS: EQUAL {} 
     | AS {}

%%
(* --- End ------------------------------------------------------- *)
