(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Antoine Spicher     : aspicher@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: pattern_ac.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-07-12 15:14:17 $
 * Version: $Revision: 1.2 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *          
 * Une topo est une structure qui embarque
 * - une collection
 * - les deplacements dans cette collection
 * - les elements deja utilise dans le pattern-matching
 * - la position de l'element courant
 *
 * Un pattern accepte ou non la position et met a jour en accord les
 * elements utilise dans topo. On peut lui demander aussi d'annuler la
 * prise des elements filtres.
 * 
 *=========================================================================
 *)

open Printf;;
open Global;;
open Valeur;;
open Terme;;
open Printer;;
open Printer_term;;
open Dispatch_sig;;
open Topology;;
open Pattern;;

exception FoundPath;;


__register_version "$RCSfile: pattern_ac.ml,v $" "$Revision: 1.2 $" "$Date: 2005-07-12 15:14:17 $";;


class ['position] blank_pattern_ac patvar next new_cpos =
object(self)
  inherit ['position] blank_pattern patvar next as super

  method to_full_string = "_"^(self#string_var)^" "^(if new_cpos then "COMMA " else "")^(next#to_full_string)

  method filtre topo =
     assert begin if !trace_opt then 
	(printf "    ~~ blank_pattern_ac: filtre pos: %s\n" (topo#prpos (topo#cpos)); flush stdout); 
	true 
      end;
      let cpos = topo#cpos in
	(*if topo#used(cpos) then None (* le filtre prcdent a pas su me proposer un element libre *)
	else*) begin
	  assert begin if !trace_opt then
	    (printf "    ~~ blank_pattern_ac: use OK\n"; flush stdout); 
	    true
	  end;
	  (** je dis que j'utilise la pos propose **)
	  topo#use cpos;
	  self#update_elem_variables topo [cpos];

	  if new_cpos then  (* next a besoin d'une position libre fraiche *)
	  (
	    (** et je propose une pos. pour la continuation **)
	    let (nb, reste) = topo#neighbors topo#cpos in
	      (*if nb = 0 then 
	      (
	         match next#filtre topo with
		   | None -> topo#unuse cpos; None
		   | Some path -> Some (cpos::path)
	      )
	      else*)
	        let found_path = ref [] in
	  	try
		  for i = 0 to nb-1 do
		    topo#move reste.(i);
		    match next#filtre topo with
		      | None      -> () (* on passe au suivant possible *)
		      | Some path -> found_path := path; raise FoundPath (* on a trouv *)
		  done;
		  topo#unuse cpos;
		  None
		with
		  | FoundPath -> Some (cpos::!found_path)
		  | NoMoreFound -> topo#unuse cpos; raise NoMoreFound
	  )
	  else
	  (
	    try
	      (
		match next#filtre topo with
		  | None -> topo#unuse cpos; None
		  | Some path -> Some (cpos::path)
	      )
	    with NoMoreFound -> topo#unuse cpos; raise NoMoreFound
	  )
	end

(*
  method filtre topo =
     assert begin if !trace_opt then 
	(printf "    ~~ blank_pattern_ac: filtre pos: %s\n" (topo#prpos (topo#cpos)); flush stdout); 
	true 
      end;
     let found_path = ref [] in
     try
       let (nb,usable) = topo#allUsable in
       for i = 0 to (nb-1) do
         topo#use usable.(i);
	 match next#filtre topo with
	   | None      -> topo#unuse usable.(i) (* on passe au suivant possible *)
	   | Some path -> found_path := (usable.(i))::path; raise FoundPath (* on a trouv *)
       done;
       None
     with FoundPath -> Some (!found_path)
*)
end;;

(* ------------------------------------------------------- *)

class ['position] cte_pattern_ac patvar (cte : mgs_value) next new_cpos =
object(self)
  inherit ['position] cte_pattern1 patvar cte next as super

  method to_full_string = (to_string(prsvalue cte))^(self#string_var)^" "^(if new_cpos then "COMMA " else "")^(next#to_full_string)

  method filtre topo =
(*
     assert begin if !trace_opt then 
	(printf "    ~~ record_pattern (set|bag): filtre pos: %s\n" (topo#prpos (topo#cpos)); flush stdout);
	true
      end;
      let cpos = topo#cpos in
	if topo#used(cpos) then None (* le filtre prcdent a pas su me proposer un element libre *)
	else begin
	  assert begin if !trace_opt then
	    (printf "    ~~ record_pattern (set|bag): use OK\n"; flush stdout); 
	    true
	  end;

	  if Dispatch_sig.equal_mgs_value cte (topo#get_value_from_pos cpos) then
	  (
	  (** je dis que j'utilise la pos propose **)
	     topo#use cpos;
	     self#update_elem_variables topo [cpos];

		  if new_cpos then
		  (
		    (** et je propose une pos. pour la continuation **)
		    let (nb, reste) = topo#neighbors topo#cpos in
		      if nb = 0 then 
		        (match next#filtre topo with
			   | None -> topo#unuse cpos; None
			   | Some path -> Some (cpos::path))
		      else
		        let found_path = ref [] in
		  	try
			  for i = 0 to nb-1 do
			    topo#move reste.(i);
			    match next#filtre topo with
			      | None      -> () (* on passe au suivant possible *)
			      | Some path -> found_path := path; raise FoundPath (* on a trouv *)
			  done;
			  topo#unuse cpos;
			  None
			with FoundPath -> Some (cpos::!found_path)
		  )
		  else
		  (
		    match next#filtre topo with
		      | None -> topo#unuse cpos; None
		      | Some path -> Some (cpos::path)
		  )
	  )
	  else
	  	None
	end
*)
    assert begin if !trace_opt then 
      (printf "    ~~ cte_pattern_ac: filtre pos: %s\n" (topo#prpos (topo#cpos)); flush stdout); 
      true 
    end;
    match topo#find cte with
      | None -> raise NoMoreFound
      | Some cpos ->
	  begin
	    assert begin if !trace_opt then 
	      (printf "    ~~ cte_pattern_ac: use OK\n"; flush stdout); 
	      true
	    end;
	    topo#move cpos;
	    topo#use cpos;
	    self#update_elem_variables topo [cpos];
	    if new_cpos then
	    (
	      (** et je propose une pos. pour la continuation **)
	  	let (nb, reste) = topo#neighbors topo#cpos 
	    	and found_path = ref [] in
	        try
	 	  for i = 0 to nb-1 do
		    topo#move reste.(i);
		    match next#filtre topo with
		      | None      -> () (* on passe au suivant possible *)
		      | Some path -> found_path := path; raise FoundPath (* on a trouv *)
		  done;
		  topo#unuse cpos;
		  None
		with
		  | FoundPath -> Some (cpos::!found_path)
		  | NoMoreFound -> topo#unuse cpos; raise NoMoreFound
	    )
	    else
	    (
	      try
		(
	    	  match next#filtre topo with
		    | None      -> topo#unuse cpos; None
		    | Some path -> Some (cpos::path)
		)
	      with NoMoreFound -> topo#unuse cpos; raise NoMoreFound
	    )
	  end

(*
  method filtre topo =
    assert begin if !trace_opt then 
      (printf "    ~~ cte_pattern_ac: filtre pos: %s\n" (topo#prpos (topo#cpos)); flush stdout); 
      true 
    end;
    match topo#find cte with
      | None -> None
      | Some cpos ->
          begin
	    assert begin if !trace_opt then 
	      (printf "    ~~ cte_pattern_ac: use OK\n"; flush stdout); 
	      true
	    end;
	    topo#use cpos;
	    match next#filtre topo with
	      | None      -> topo#unuse cpos; None
	      | Some path -> Some (cpos::path)
	  end
*)

end;;

(* ------------------------------------------------------- *)

class ['position] cteN_pattern_ac patvar (cte : mgs_value) (nbocc:int) (alternatif : 'position pattern) (next : 'position pattern) (new_cpos : bool) =
object(self)
  inherit ['position] cte_patternN patvar cte nbocc alternatif next as super

  method to_full_string = "#"^(string_of_int nbocc)^" "^(to_string(prsvalue cte))^(self#string_var)^" "^(if new_cpos then "COMMA " else "")^(next#to_full_string)

  method private filtre_non_opt topo =
    begin
      assert begin if !trace_opt then 
	(printf "    ~~ cteN_pattern_ac: filtre pos: #%d %s\n" nbocc (topo#prpos (topo#cpos)); flush stdout); 
	true 
      end;

      if topo#dispose_get_NvalueS_from_pos
      then (
	let cpos = topo#cpos in
	  match topo#get_NposS_from_pos cpos nbocc cte with
	    | None                   -> raise NoMoreFound
	    | Some (chemin, lastpos) -> (
		topo#move lastpos;
		List.iter topo#use chemin;
		self#update_elem_variables topo chemin;
		if new_cpos then (
		  (** Je fournis une position pour la suite **)
		  let (nb, reste) = topo#neighbors topo#cpos
		  and found_path = ref [] in
		    try
		      for i = 0 to nb-1 do
			topo#move reste.(i);
			match next#filtre topo with
			  | None      -> () (* on passe au suivant voisin possible *)
			  | Some path -> found_path := path; raise FoundPath (* on a trouv *)
		      done;
		      List.iter topo#unuse chemin;
		      topo#goto cpos;
		      None
		    with
		      | FoundPath   -> Some (chemin@(!found_path))
		      | NoMoreFound -> List.iter topo#unuse chemin; topo#goto cpos; raise NoMoreFound
		) else (
		  (** La suite n a pas besoin d'une position courante libre **)
		  try
		    match next#filtre topo with
		      | None      -> (List.iter topo#unuse chemin;
				      topo#goto cpos;
				      None)
		      | Some path -> Some (chemin@path)
		  with NoMoreFound -> List.iter topo#unuse chemin; topo#goto cpos; raise NoMoreFound
		)
	      )
      ) else alternatif#filtre topo
    end

  method filtre topo =
    assert begin if !trace_opt then 
      (printf "    ~~ cteN_pattern_ac: filtre pos: #%d %s\n" nbocc (topo#prpos (topo#cpos)); flush stdout); 
      true 
    end;
    match topo#findN nbocc cte with
      | None -> raise NoMoreFound
      | Some (cpos_path, last_pos) ->
	  begin
	    assert begin if !trace_opt then 
	      (printf "    ~~ cteN_pattern_ac: use OK\n"; flush stdout); 
	      true
	    end;
	    topo#move last_pos;
	    List.iter topo#use cpos_path;
	    self#update_elem_variables topo cpos_path;
	    if new_cpos then
	    (
	      (** et je propose une pos. pour la continuation **)
	      let (nb, reste) = topo#neighbors topo#cpos
	      and found_path = ref [] in
	        try
	 	  for i = 0 to nb-1 do
		    topo#move reste.(i);
		    match next#filtre topo with
		      | None      -> () (* on passe au suivant possible *)
		      | Some path -> found_path := path; raise FoundPath (* on a trouv *)
		  done;
		  List.iter topo#unuse cpos_path;
		  None
		with
		  | FoundPath -> Some (cpos_path @ !found_path)
		  | NoMoreFound -> List.iter topo#unuse cpos_path; raise NoMoreFound
	    )
	    else
	    (
	      try
		(
	    	  match next#filtre topo with
		    | None      -> List.iter topo#unuse cpos_path; None
		    | Some path -> Some (cpos_path @ path)
		)
	      with NoMoreFound -> List.iter topo#unuse cpos_path; raise NoMoreFound
	    )
	  end

  initializer
    assert(nbocc <> 0)

end;;

(* ------------------------------------------------------- *)

class ['position] record_pattern_ac patvar recspec (varindex : (string*int) list)  next new_cpos =
object(self)
  inherit ['position] record_pattern patvar recspec varindex next as super

  method to_full_string =
    (to_string (prspat (RecordPat recspec)))^(self#string_var)^" "^
    (if new_cpos then "COMMA " else "")^(next#to_full_string)

  method filtre topo =
     assert begin if !trace_opt then 
	(printf "    ~~ record_pattern_ac: filtre pos: %s\n" (topo#prpos (topo#cpos)); flush stdout);
	true
      end;
      let cpos = topo#cpos in
	if topo#used(cpos) then None (* le filtre prcdent a pas su me proposer un element libre *)
	else begin
	  assert begin if !trace_opt then
	    (printf "    ~~ record_pattern_ac: use OK\n"; flush stdout); 
	    true
	  end;

	  if self#check_and_bind_record topo cpos then
	  (
	  (** je dis que j'utilise la pos propose **)
	     topo#use cpos;
	     self#update_elem_variables topo [cpos];

		  if new_cpos then
		  (
		    (** et je propose une pos. pour la continuation **)
		    let (nb, reste) = topo#neighbors topo#cpos in
		      (*if nb = 0 then 
		        (match next#filtre topo with
			| None -> topo#unuse cpos; None
			| Some path -> Some (cpos::path))
			else*)
		    let found_path = ref [] in
		      try
			for i = 0 to nb-1 do
			  topo#move reste.(i);
			  match next#filtre topo with
			    | None      -> () (* on passe au suivant possible *)
			    | Some path -> found_path := path; raise FoundPath (* on a trouv *)
			done;
			topo#unuse cpos;
			None
		      with
			| FoundPath -> Some (cpos::!found_path)
			| NoMoreFound -> topo#unuse cpos; raise NoMoreFound
		  )
		  else
		  (
		    try (
		      match next#filtre topo with
			| None -> topo#unuse cpos; None
			| Some path -> Some (cpos::path)
		    )
		    with NoMoreFound -> topo#unuse cpos; raise NoMoreFound
		  )
	  )
	  else
	  	None
	end

(*
  method filtre topo =
     assert begin if !trace_opt then 
	(printf "    ~~ record_pattern_ac: filtre pos: %s\n" (topo#prpos (topo#cpos)); flush stdout); 
	true 
      end;
     let found_path = ref [] in
     try
       let (nb,usable) = topo#allUsable in
       for i = 0 to (nb-1) do
         if self#check_and_bind_record topo (usable.(i)) then
	   begin
             topo#use usable.(i);
	     match next#filtre topo with
	       | None      -> topo#unuse usable.(i) (* on passe au suivant possible *)
	       | Some path -> found_path := (usable.(i))::path; raise FoundPath (* on a trouv *)
	   end
       done;
       None
     with FoundPath -> Some (!found_path)
*)

end;;

(* ------------------------------------------------------- *)

(** Garde: 
 ** pour simplifier la linarisation d'un filtre, une garde ne consomme rien
 ** en terme d'lments filtrs, mais la valeur retourn peut stopper le
 ** filtrage.
 **)
class ['position] guarded_pattern_ac
  (patvar   : int list)    (* la liste de ses noms : mais une guarde n'a jamais de nom *)
  (depend   : int list)    (* la liste des index de variables de patterns utilises dans la garde *)
  (exp      : term)        (* la garde sous forme de terme *)
  (guard    : mgs_value)   (* la garde sous forme de fonction mgs attendant la fct environnement *)
  (next     : 'position pattern)
  (new_cpos : bool)
  =
object(self)
  inherit ['position] guarded_pattern patvar depend exp guard next as super

  method to_full_string =
    "//" ^  (to_string(Printer_term.prsterm exp)) ^ "// " ^ (if new_cpos then "COMMA " else "")^ next#to_full_string

  method filtre topo =
    begin
      assert begin if !trace_opt then
	(printf "    ~~ guarded_pattern_ac: motif %s filtre\n" 
	   self#to_full_string;
	 flush stdout);
	true
      end;
      let vguard = (pf1 guard) topo#get_val_of_rule_variable in
	assert begin if !trace_opt then 
	  (printf "    ~~ guarded_pattern_ac: guard = %a\n" Printer.prfvalue2 vguard);
	  true
	end;
	if Auxfct.bool_from_val vguard then
	(
	  if (new_cpos && (topo#used topo#cpos)) then
          ( (** et je propose une pos. pour la continuation **)
	    let (nb, reste) = topo#neighbors topo#cpos
	    and found_path = ref [] in
	    try
	      for i = 0 to nb-1 do
	        topo#move reste.(i);
		match next#filtre topo with
		  | None      -> () (* on passe au suivant possible *)
		  | Some path -> found_path := path; raise FoundPath (* on a trouv *)
	      done;
	      None
	    with FoundPath -> Some (!found_path)
          )
          else
            next#filtre topo
	)
	else
	  None
    end
end;;

(* ------------------------------------------------------- *)

(** Update:
 ** pattern "void" insrer  la bonne place pour mettre  jour les
 ** variables qui ne correspondnet pas  un pattern lementaire ou bien
 ** un iterpat. Par exemple, X dans (u, v) as X 
 **)
class ['position] update_pattern_ac
  (patvar   : int list)  
  (v1       : int)
  (v2       : int)
  (next     : 'position pattern)
  (new_cpos : bool)
  =
object(self)
  inherit ['position] update_pattern patvar v1 v2 next as super

  method to_full_string = 
    "//" ^  self#string_var 
    ^ " := "^ (string_of_int v1) 
    ^", " ^(string_of_int v2) 
    ^ "// " ^(if new_cpos then "COMMA " else "")^ next#to_full_string

  method filtre topo =
    begin
      assert begin if !trace_opt then 
	(printf "    ~~ update_pattern_ac: motif %s filtre\n" 
	   self#to_full_string;
	 flush stdout);
	true
      end;
      (* update de la variable *)
      let saved_env = self#save_env topo in
      let (p_v1,p_v2) =
	(
	  match topo#get_env v1 with
	    | ImmediateVal _ ->
		raise (Erreur "Error: internal: immediate value in update_pattern_ac (a)")
	    | EltCollection p1 ->
	     (
	       match  topo#get_env v2 with
		 | ImmediateVal _ -> raise (Erreur "Error: internal: immediate value in update_pattern_ac (b)")
		 | EltCollection p2 -> self#update_path_variables topo [p1; p2]; ([p1],[p2])
		 | PathCollection p2l -> self#update_path_variables topo (p1::p2l) ; ([p1],p2l)
	     )
	 | PathCollection p1l -> 
	     (
	       match  topo#get_env v2 with
		 | ImmediateVal _ -> raise (Erreur "Error: internal: immediate value in update_pattern_ac (c)")
		 | EltCollection p2 -> self#update_path_variables topo (p1l@[p2]) ; (p1l,[p2])
		 | PathCollection p2l -> self#update_path_variables topo (p1l@p2l) ; (p1l,p2l)
	     )
	)
      in
	(*assert begin if !trace_opt then 
	  (printf "    ~~ update_pattern_ac:\n\t- v1 : %s\n\t- v2 : %s\n"
	     (Topology.string_of_list topo#prpos p_v1)
	     (Topology.string_of_list topo#prpos p_v2);
	   flush stdout);
	  true
	end;*)
      (* on passe  la suite du pattern *)
	if (new_cpos && (topo#used topo#cpos)) then
	( (** et je propose une pos. pour la continuation **)
	  let (nb, reste) = topo#neighbors topo#cpos
	  and found_path = ref [] in
	    try
	      for i = 0 to nb-1 do
		topo#move reste.(i);
		match next#filtre topo with
		  | None      -> () (* on passe au suivant possible *)
		  | Some path -> found_path := path; raise FoundPath (* on a trouv *)
	      done;
	      self#load_env topo saved_env ;
	      None
	with FoundPath -> Some (!found_path)
      )
      else
	(
          match next#filtre topo with
	    | None -> self#load_env topo saved_env ; None
	    | path -> path
	)

    end

  method private save_env topo =
    begin
      let rec save = function
	| [] -> []
	| idx::t -> (idx, topo#get_env idx)::(save t)
      in
	save patvar
    end

  method private load_env topo saved_env =
    begin
      let rec load = function
	| [] -> ()
	| (idx,el)::q ->
	    (
	      (match el with
		| EltCollection v -> topo#set_elt idx v
		| PathCollection v -> topo#set_path idx v
		| ImmediateVal v -> topo#set_immediate idx v) ;
	      load q
	    )
      in
	load saved_env
    end    
end;;


class ['position] iter_pattern_ac
  (patvar : int list)
  kiter
  (iterpat : 'position pattern)
  (next : 'position pattern)
  new_cpos_cont
  new_cpos_iter =
object(self)
  inherit ['position] iter_pattern patvar kiter iterpat next as super

  method to_full_string =
    super#general_to_string
      (function () ->
	 (if new_cpos_iter then "COMMA " else "") ^ iterpat#to_full_string
      )
      (function () -> super#string_var ^ " " ^ (if
            new_cpos_cont then "COMMA " else "")^ next#to_full_string
      )

  method filtre topo =
    assert begin if !trace_opt then 
      (printf "    ~~ iter_pattern_ac\n\t- iterpat : %s\n\t- next : %s\n\t- pos : %s\n"
	 iterpat#to_full_string
	 next#to_full_string
	 (topo#prpos (topo#cpos));
       flush stdout);
      true 
    end;
    let ans =
      if super#is_plus_iteration then
	(
	  assert begin if !trace_opt then 
	    (printf "\t- kind_it : plus(+)\n";
	     flush stdout);
	    true 
	  end;
	  self#filtre_iterplus [] topo
	)
      else
	(
	  assert begin if !trace_opt then 
	    (printf "\t- kind_it : star(*)\n";
	     flush stdout);
	    true 
	  end;
	  self#filtre_iterstar [] topo
	)
    in
      match ans with
	| (None, None)      -> None
	| (None, Some p')   ->
	    (
	      (*assert begin if true (*!trace_opt*) then 
		(printf "    ~~ iter_pattern_ac : der maj des variables : %s\n"
		   (Topology.string_of_list topo#prpos []) ; 
		 flush stdout);
		true 
	      end;*)
	      self#update_vars [] topo ;
	      Some p'
	    )
	| (Some p, Some p') ->
	    (
	      (*assert begin if true (*!trace_opt*) then 
		(printf "    ~~ iter_pattern_ac : der maj des variables : %s\n"
		   (Topology.string_of_list topo#prpos p) ; 
		 flush stdout);
		true 
	      end;*)
	      self#update_vars p topo ;
	      Some (p@p')
	    )
	| (Some _, None) -> raise (Erreur "iternal error: iter_pattern_ac: impossible match")

  method private filtre_iterstar acc topo =
    (* acc correspond au chemin dj filtr.                                              *)
    (* renvoie  partir de la position courrante un couple de chemins                     *)
    (* (Some p, Some p') => on peut iterer iterpat sur p puis enchainer avec p' pour next *)
    (* (None, Some p')   => on enchaine directement depuis cpos avec p' pour next         *)
    (* (None, None)      => on ne peut rien faire avancer                                 *)
    (* (Some p, None)    => logiquement impossible                                        *)
    begin
      let cpos = topo#cpos in
	if true then
	  (
	    (* longer path first *)
	    assert begin if !trace_opt then
	      (printf "    ~~ iter_pattern_ac: longer first => (iterpat+ | epsilon)\n";
	       flush stdout);
	      true
	    end;
	    assert begin if !trace_opt then
	      (printf "    ~~ iter_pattern_ac: try first iterpat+\n";
	       flush stdout);
	      true
	    end;
	    match (self#filtre_iterplus acc topo) with
	      | (None,None) ->
		  (
		    assert begin if !trace_opt then
		      (printf "    ~~ iter_pattern_ac: iterpat+ failed ; try epsilon\n";
		       flush stdout);
		      true
		    end;
		    self#update_vars acc topo;
		    topo#goto cpos;
		    let ans = (None, (self#filtre_epsilon topo)) in
		      assert begin if !trace_opt then
			(printf "    ~~ iter_pattern_ac: epsilon ended\n";
			 flush stdout);
			true
		      end;
		      ans
		  )
	      | (None, Some p)     -> raise (Erreur "iternal error: iter_pattern_ac: filtre_iterstar -> iterplus = (None, Some p)")
	      | (Some p1, Some p2) ->
		  (
		    assert begin if !trace_opt then
		      (printf "    ~~ iter_pattern_ac: iterpat+ OK\n";
		       flush stdout);
		      true
		    end;
		    (Some p1, Some p2)
		  )
	      | (Some _, None) -> raise (Erreur "iternal error: iter_pattern_ac: impossible match !!!")
	  )
	else
	  (
	    (* shorter path first *)
	    assert begin if !trace_opt then
	      (printf "    ~~ iter_pattern_ac: shorter first => (epsilon | iterpat+)\n";
	       flush stdout);
	      true
	    end;
	    self#update_vars acc topo;
	    assert begin if !trace_opt then
	      (printf "    ~~ iter_pattern_ac: try first epsilon\n";
	       flush stdout);
	      true
	    end;
	    match (self#filtre_epsilon topo) with
	      | None   ->
		  (
		    assert begin if !trace_opt then
		      (printf "    ~~ iter_pattern_ac: epsilon failed ; try iterpat+\n";
		       flush stdout);
		      true
		    end;
		    topo#goto cpos;
		    let ans = self#filtre_iterplus acc topo in
		      assert begin if !trace_opt then
			(printf "    ~~ iter_pattern_ac: iterpat+ ended\n";
			 flush stdout);
			true
		      end;
		      ans
		  )
	      | Some p ->
		  (
		    assert begin if !trace_opt then
		      (printf "    ~~ iter_pattern_ac: epsilon OK\n";
		       flush stdout);
		      true
		    end;
		    (None, Some p)
		  )
	  )
    end

  method private filtre_epsilon topo =
    begin
      let cpos = topo#cpos in
      let used = topo#used cpos in
	if (used && new_cpos_cont) then
	  (
	    let (nb, reste) = topo#neighbors cpos in
	    let found_path = ref [] in
	      try
		(
		  for i = 0 to nb-1 do
		    topo#move reste.(i);
		    try
		      (
			match next#filtre topo  with
			  | None    -> ()
			  | Some p' -> found_path := p' ; raise FoundPath
		      )
		    with
		      | NoMoreFound -> ()
		  done ;
		  None
		)
	      with FoundPath -> Some (!found_path)
	  )
	else
	  try (next#filtre topo)
	  with NoMoreFound -> None
    end

  method private filtre_iterplus acc topo =
    begin
      let cpos = topo#cpos in
      let used = topo#used cpos in
      let iterplus_f () =
	try
	  (
	    match iterpat#filtre topo with
	      | None   -> (None, None)
	      | Some p ->
		  (
		    match self#filtre_iterstar (acc@p) topo with
		      | (Some p1, Some p1') -> (Some (p@p1), Some p1')
		      | (None, Some p1')    -> (Some p, Some p1')
		      | (None, None)        -> topo#unuse_all  p;(None, None)
		      | (Some _, None)      -> raise (Erreur "iternal error: iter_pattern_ac: impossible match")
		  )
	  )
	with
	  | NoMoreFound -> (None, None)
      in
	assert begin if !trace_opt then
	  (printf "    ~~ iter_pattern_ac: used=%s and new_cpos_iter=%s\n"
	     (string_of_bool used)
	     (string_of_bool new_cpos_iter);
	   flush stdout);
	  true
	end;
	if (used && new_cpos_iter) then
	  (
	    let (nb, reste) = topo#neighbors cpos in
	    let found = ref (None,None) in
	      try
		(
		  for i = 0 to (nb-1) do
		    topo#move reste.(i);
		    match (iterplus_f ()) with
		      | (None, None) -> ()
		      | p            -> found := p ; raise FoundPath
		  done ;
		  (None, None)
		)
	      with
		| FoundPath -> !found
	  )
	else
	  iterplus_f ()
    end

end ;;
