(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: topo_ac.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2006-04-16 16:04:03 $
 * Version: $Revision: 1.4 $
 * -------------------------------------------------------------------
 *
 * Status: voir le commentaire general dans topology.ml
 *          
 * Ici on developpe les instance de generic_topo qui correspondent 
 * aux collections Associatives/Commutative (et parfois idempotente)
 *
 *=========================================================================
 *)

open Global;;
open Valeur;;
open Topology;;
open Terme;;
open Printer;;

__register_version "$RCSfile: topo_ac.ml,v $" "$Revision: 1.4 $" "$Date: 2006-04-16 16:04:03 $";;

exception FoundIntPos of int;;

let in_place_shuffle_array len a = 
  let nb  = ref (if (len <= 1)
		 then 0
		 else 
		   if (len = 2) 
		   then 1 
		   else len) (* Pour eviter que sur un tableau de longueur 2 on fasse 2 permutations inverses. *)
  in                         (* Le risque est moins grand sur un tableau de taille superieure.                 *)
    while (!nb > 0)
    do
      let src = Random.int len
      and dst = Random.int len
      in
	if (src = dst)
	then ()
	else (let tmp = a.(src)
	      in
		a.(src) <- a.(dst);
		a.(dst) <- tmp;
		decr nb)
    done;
    a
;;


class virtual ['collection] topo_ac ((c : 'collection), couleur, (m : mgs_value)) card  =
object(self)
  inherit [int] generic_topo 
    string_of_int                     (** afficheur de position    **)
    (c, couleur)                      (** collection a transformer **)
    m                                 (** l'argument complet       **)
    (Global.crandom card)             (** position de depart       **)

  method virtual iterateur   : (mgs_value -> 'b) -> 'collection -> unit
  method virtual empty       : 'collection
  method virtual add         : mgs_value -> 'collection -> 'collection
  method virtual union       : 'collection -> 'collection -> 'collection
  method virtual from_coll   : mgs_value  -> 'collection
  method virtual to_coll     : ('collection colors) -> mgs_value

  method ajoute_tous ac couleur kind = function
      | (VAL_seq (seq, col)) as valeur -> (if (kind.a_flat = FlatPath) 
					   then Array.fold_right (fun elem suite -> self#add elem suite) seq ac
					   else self#add valeur ac)

      | valeur  when ((is_collection valeur) &
		      (is_same_coll_kind valeur m))  -> (if ((kind.a_flat = NoFlatPath) or (not (Type.is_subtype (collection_type valeur) couleur)))
							   then (self#add valeur ac)
							   else (self#union (self#from_coll(valeur)) ac))
      | x                              -> self#add x ac

  val _used    = Array.make card PosFree
  val _valeurs = Array.make card (VAL_undef "")
  val _voisins = Array.make (max 0 (card - 1)) 0

  method mgs_value_from_pos pos = (VAL_int pos)

  val mutable _batch = ([] : (rule_property * 'position path * mgs_value) list)

  method iter_batch f = List.iter f _batch

  method state p          = _used.(p)
  method update_state f p = _used.(p) <- f(_used.(p))

  method unvisit_all = for i = 0 to (card-1) do if self#visited(i) then self#unvisit(i) done

  method next_usable p =
    try
      for i = 0 to (card-1)
      do
	let position = (p+i) mod card
	in
	  if (_used.(position) = PosFree) then raise (FoundIntPos position)
      done;
      raise NoMoreUsable
    with FoundIntPos p -> (assert begin if !trace_opt then
			     (Printf.printf "---- topoAC: next_usable: found %d\n" p); 
			     true
			   end;
			   p)

  method add_match pt p v = _batch <- (pt, p, v)::_batch
  method get_value_from_pos p = _valeurs.(p)
  method some_neighbors p = function
    | AllDirections    -> (let vois = ref 0 
			   in
			     Array.iteri (fun idx etat -> if ((idx <> p) & (pos_usable etat)) then (_voisins.(!vois) <- idx; incr vois)) _used;
			     !vois, if (!match_random > 0) then
			              in_place_shuffle_array (!vois) (Array.copy _voisins)
			            else
			              (Array.copy _voisins)
			  )

    | SomeDirections d -> (let err = "Error: cannot use direction in set/bag: "
				     ^(Array.fold_left (fun s x -> s^"\t"^(to_string (prsvalue x))^"\n") "" d)
			   in 
			     prerr_endline err;
			     raise (Erreur err))

  method direction_from_to p1 p2 = VAL_undef "direction"
  method build_result =
    let nouveau   = List.fold_right (fun (kind, chemin, rhs) nouveau -> self#ajoute_tous nouveau couleur kind rhs) _batch self#empty
    and untouched = (let ret = ref self#empty
		     in
		       Array.iteri (fun idx etat -> if (pos_usable etat) then ret := self#add _valeurs.(idx) !ret) _used;
		       !ret)
    in
      self#to_coll ((self#union nouveau untouched), couleur)

  method find v =
    try
      for i = 0 to (card-1)
      do
	let index = (self#cpos + i) mod card in
	let w = _valeurs.(index)
	in
          if (self#usable(index) && (Dispatch_sig.equal_mgs_value v w)) then
		raise (FoundIntPos index)
	  done ;
	  None
    with FoundIntPos index -> Some index
      
  method findN n v =
    match (self#find v) with
      | None       -> None
      | Some place -> (
	  if self#dispose_get_NvalueS_from_pos
	  then self#get_NposS_from_pos place n v
	  else raise (Erreur "Topo_ac: topo_ac: findN: NYI: get_NvalueS_from_pos not available")
	)
      
  initializer
  let idx = ref 0
  in
    self#iterateur (function x -> _valeurs.(!idx) <- x; incr idx) c
end
;;
