(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: compile.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2004-09-24 00:17:03 $
 * Version: $Revision: 1.1 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *=========================================================================
 *)

open Unix;;
open Global;;
open Valeur;;
open Terme;;
open Byte_code;;
open Environnement;;
open Printer;;
open Printer_term;;
open Printer_code;;
open Kombinator;;
open Combinatorise;;
open Optimise_combinateur;;
open Dispatch;;

__register_version "$RCSfile: compile.ml,v $" "$Revision: 1.1 $" "$Date: 2004-09-24 00:17:03 $";;



(** ------------------------------------------------------- **)



      

let pstr s = print_string s

and ptrace s t = 
  ((if !trace_opt then Printf.printf "%s %a\n" s (fun out x -> prcode x;()) t); true)
;;


let rec tracegcode x =
  pstr "gcode: "; prcode x; pstr "\n"; 
  gcode x


and gcode = function 
  | C_I  -> mgs_combI
  | C_K  -> mgs_combK
  | C_B  -> mgs_combB
  | C_C  -> mgs_combC
  | C_S  -> mgs_combS
  | C_S4 -> mgs_combS4

  | Cv v -> v

  | Cd name -> (!var_env).(index_cte(name))

  (** freeze des variables et des if **)
  | Cupl(Ofct(index), level, []) as t
    -> (assert (ptrace "++ gcode: ref detecte: " t);
	gcode_var index level)

  | Cupl(Ofct(index), level, l) as t
    -> (assert (ptrace "++ gcode: apply n-aire detecte: " t);
	let larg =  (List.map gcode l)
	in gcode_apply index larg level)

  | Cupl(Oset(index), level, [a]) as t
    -> (assert (ptrace "++ gcode: affectation detecte: " t);
	gcode_set (gcode a) index level)

  | Cupl(Oif, level, [p; q; r]) as t
    -> (assert (ptrace "gcode: if: " t);
	gcode_if p q r level)

  | Cupl(Ofby, level, l) as t
    -> (assert (ptrace "gcode: fby: " t);
	gcode_fby l level)
      
      
  | Capp(p, q) as t
    -> (assert (ptrace "++ gcode: apply: " t);
	let f = (gcode p)
	and arg = (gcode q)
	in (pf "gcode:App" f)(arg))


  | x -> 
      (pstr "Erreur in Gcode: byte-code incorrect non traduit\n";
       error_c x; 
       raise (Erreur "Gcode: internal error: byte-code non traduit"))
      

and gcode_if p q r = function
  | 0 -> 
      begin
	match __val2bool(gcode p) with 
	  | VAL_bool true -> (gcode q) 
	  | _             -> (gcode r)
      end

  | n -> 
      begin
	let p' = gcode p
	and q' = gcode q
	and r' = gcode r in
	let pp = pf "gcode_if_aux:a" p'
	and qq = pf "gcode_if_aux:b" q'
	and rr = pf "gcode_if_aux:c" r'
	in match n with (** les 3 premiers cas pour optimiser un brin **)
	  | 1 -> (VAL_funct(function x -> 
			      match __val2bool(pp x)  with
				| VAL_bool true -> qq x 
				| _             -> rr x))
	  | 2 -> VAL_funct(function x -> 
			     VAL_funct(function y -> 
					 let ppp = pf "gcode_if_aux:aaa" (pp x) in
					   match __val2bool(ppp y) with 
					     | VAL_bool true -> (pf "gcode_if_aux:bbb" (qq x)) y
					     | _             -> (pf "gcode_if_aux:ccc" (rr x)) y)) 
	  | 3 -> VAL_funct(function x -> 
			     VAL_funct(function y -> 
					 VAL_funct(function z -> 
						     match __val2bool((to_funct((to_funct (pp x)) y))z) with 
						       | VAL_bool true -> (to_funct((to_funct (qq x)) y))z
						       | _             -> (to_funct((to_funct (rr x)) y))z)))
	  | n -> VAL_funct(function x -> gcode_if_generique p' q' r' [x] (n-1))
      end


and gcode_if_generique p q r l = function
  | 0 -> do_if p q r l
  | n -> VAL_funct(function x -> gcode_if_generique p q r (x::l) (n-1))

and do_if p q r l =
  let apply_naire fct = List.fold_right (fun a f -> (to_funct f)a) l fct
  in (match __val2bool(apply_naire p) with
	| VAL_bool true -> apply_naire q
	| _             -> apply_naire r)


and gcode_var index = function
  | 0     -> (!var_env).(index)
  | 1     -> VAL_funct(function _ -> (!var_env).(index))
  | level -> (let body = gcode_var index (level - 1)
	      in 
		VAL_funct(function _ -> body))


and gcode_set a index = function
  | 0 -> ((!var_env).(index) <- a;  a)

  | 1 -> (let aa = pf "gcode:set: " a in
	    VAL_funct(function x ->  let rhs = aa x in (!var_env).(index) <- rhs; rhs))

  | n -> (let aa = pf "gcode:set: " a in 
	    VAL_funct(function x -> gcode_set (aa x) index (n-1)))

and gcode_fby l = function
  | 0 -> miter gcode l
  | n -> gcode_fby_aux (List.map gcode l) n

and gcode_fby_aux l = function
  | 1 ->  VAL_funct(function x -> riter x l)

  | n -> (let ll = List.map (pf "gcode_fby:a") l in
            VAL_funct(function x -> gcode_fby_aux (List.map (function f -> (to_funct f) x) l) (n-1)))

and miter f = function
  | []    -> failwith "internal error: miter: liste vide"
  | a::[] -> f a
  | a::l  -> (f a); miter f l

and riter x = function
  | []    -> failwith "internal error: riter: liste vide"
  | f::[] -> (pf "gcode:riter:1" f) x
  | f::l  -> ((pf "gcode:riter:2" f) x); riter x l


(** Attention. 
 ** A cause des references imperatives, on ne peut pas 
 ** appliquer "pf" avant l'application finale 
 **)
and gcode_apply index l level = 
  match (level, l) with
    | 0, []    -> VAL_funct(pf "gcode:apply:1" (!var_env).(index))
    | 0, [a]   -> (pf "gcode:apply:2" (!var_env).(index))(a)
    | 0, [a;b] -> (pf "gcode:apply:3" ((pf "gcode:apply:1" (!var_env).(index)) a)) b
    | 0, l     -> iterapp (pf "gcode:apply:4" (!var_env).(index)) l

    | 1, []    -> VAL_funct(function _ -> (!var_env).(index))
    | 1, [a]   -> VAL_funct(function x -> let f = pf "gcode:apply:5" (!var_env).(index) 
					  and a1 = pf "gcode:apply:6" a 
			                  in f (a1 x))
    | 1, [a;b] -> VAL_funct(function x ->  let f = pf "gcode:apply:7" (!var_env).(index) 
					   and a1 = pf "gcode:apply:8" a 
					   and b1 = pf "gcode:apply:9" b 
			                   in (pf "gcode:apply:10" (f (a1 x)))(b1 x))
    | 1, l     -> VAL_funct(function x -> gcode_apply index (riter2 x l) 0)


    | n, []    -> let nn = n-1 in VAL_funct(function x -> gcode_apply index [] nn)
    | n, [a]   -> let nn = n-1 in VAL_funct(function x -> gcode_apply index [(pf "gcode:apply:11" a) x] nn)
    | n, [a;b] -> let nn = n-1 in VAL_funct(function x -> gcode_apply index [(pf "gcode:apply:12" a) x; (pf "gcode:apply:13" b) x] nn)
    | n, l     -> let nn = n-1 in VAL_funct(function x -> gcode_apply index (riter2 x l) nn)




(** normalement:
 **     riter2 x l = List.map (function a -> (pf1 a)(x)) l 
 ** mais en depliant pour les listes a 0, 1 et 2 arguments, on gagne 10%
 ** si on deplie a la longueur 3, on perd.
 **)

and riter2 x = function
  | [] -> []

  | [a] -> [(pf "gcode:riter2:d" a) x]
  | [a;b] -> [(pf "gcode:riter2:e" a) x; (pf "gcode:riter2:f" b) x]

  | l -> List.map (riter2_aux x) l

and riter2_aux x = function a -> (pf1 a)(x)

and iterapp f = function
  | []   -> VAL_funct(f)
  | [a]  -> f a 
  | a::l -> iterapp (pf "iterapp" (f a)) l
;;



(** ----------- traduction/evaluation ----------------------------- **)




let rec traduit e =
  let res = term2bytecode e
  in 
    if check_byte_code(res)
    then optimise(res)
    else (error_c res; raise (Erreur "internal error: bad byte-code"))


and eval e = gcode(traduit e)

(* traduit_evalue: idem que eval, mais ca fait de l'affichae 
 * intermediaire en plus et ca compte le temps 
 *)
and traduit_evalue e = 
  (if !trace_input then
     begin
       pstr "\ninput> ";
       prterm e;   
       print_newline()
     end);
  let t = traduit e in 
    (if !trace_input then
       begin
	 pstr "\ncombinator form> ";
	 prcode t;
	 print_newline();
	 flush Pervasives.stdout;
       end);
    let debut = (Unix.times()).tms_utime in
    let res = (gcode t) in 
    let duree = (Unix.times()).tms_utime -. debut in
      (if !trace_input then pstr "\nreturn> " else print_string "\n");
      prvalue res;
      print_newline();
      (if duree > 0.5 then Printf.printf "\n\nTime used : %f seconds" duree);
      flush Pervasives.stdout;
      res
;;


(** ----------- ajustement GC ----------------------------- **)

(** file:/usr/share/doc/ocaml/docs/ocaml.html/libref/Gc.html **)
(**
let control = Gc.get()
in Gc.set {
   control with 
	   Gc.verbose = 0x004;
	   Gc.minor_heap_size = 10*control.Gc.minor_heap_size;
	   Gc.major_heap_increment = 2*control.Gc.major_heap_increment;
	   Gc.space_overhead = (6*control.Gc.space_overhead);
	   Gc.max_overhead = (3*control.Gc.max_overhead);
}
;;

**)


(** comparaison ATTENTION a la machine
   test10 (ml):          4.6 pour fib(35)  et  6.9 pour ack 3 8 
   mgs jlg:             27.5 pour fib(35)  et  6.7 pour ack 3 8 
   combinateur initial:  9.3 pour fib(35)  et  4.0 pour ack 3 8
   combinateur unaire:   9.5 pour fib(35)  et 60.0 pour ack 3 8
   combinateur + Sop:    5.1 pour fib(35)  et 48.5 pour ack 3 8
   combinateur+Sop*+*4:  5.0 pour fib(35)  et 37.0 pour ack 3 8

   base :                7.1  pour fib(35)  et 15.7 pour ack 3 8
   Base3:                6.65 pour fib(35)  et 14.7 pour ack 3 8
   recodageBase3:        6.6  pour fib(35)  et 14.7 pour ack 3 8
 **)


