(*=======================================================-*-tuareg-*-======
 *
 * MGS : a general model of systems
 *
 *    Olivier Michel      : michel@lami.univ-evry.fr
 *    Jean-Louis Giavitto : giavitto@lami.univ-evry.fr
 *    Julien Cohen        : jcohen@lami.univ-evry.fr
 *    Mailing-list        : mgs@lami.univ-evry.fr
 *    WWW                 : http://mgs.lami.univ-evry.fr
 *
 * LaMI umr 8042 du CNRS, Genopole (C), Universite d'Evry Val d'Essonne
 *
 *
 * -------------------------------------------------------------------
 * File: $RCSfile: optimise_combinateur.ml,v $
 * Symbolic name : $Name:  $
 * Author: OM, JLG
 * Modified: $Date: 2005-04-28 21:23:18 $
 * Version: $Revision: 1.2 $
 * -------------------------------------------------------------------
 *
 * Status: 
 *=========================================================================
 *)

open Global;;
open Byte_code;;
open Dispatch_sig;;
open Environnement;;


__register_version "$RCSfile: optimise_combinateur.ml,v $" "$Revision: 1.2 $" "$Date: 2005-04-28 21:23:18 $";;




let rec is_tup_in_list level = function
  | [] -> false
  | Cupl(Ofby, n, l)::_ when n = level -> true
  | Cupl(Ofby, n, l)::_ when n <> level -> 
      failwith "Internal error: is_tup: Cupl syntax"
  | _::l -> is_tup_in_list level l

and flatify level = function
  | [] -> []
  | Cupl(Ofby, n, l)::l' when n = level -> l@(flatify level l')
  | Cupl(Ofby, n, l)::l' when n <> level ->
      failwith "internal error: flatify: Cupl syntax"
  | a::l' -> a::(flatify level l')
;;


(** ----------- Optimisation de la Combinatorisation ---------------------- **)

(*** Verifiez qu'il y a moins d'application en partie 
 *** droite d'une rgle qu'en partie gauche.
 ***)

let rec xxoptimise x = x

and yyoptimise x = 
  print_string "start opti\n";
  let res = yyoptimise x
  in (print_string "fin opti\n"; res)

and optimise t = 
  let topt = opt' t
  in if (Pervasives.compare t topt = 0) then t else optimise topt

and opt' = function

        (** optimisation: S (K p) (K q)  ==  K (p q)  **)
  | Capp(Capp(C_S, Capp(C_K, p)), Capp(C_K, q)) ->
      if !trace_opt then print_string "--- opt1: S (K p) (K q)  =  K (p q)\n";
      optimise(Capp(C_K, Capp(p, q)))

	(** optimisation: S (B p q) r == S4 p q r   **)
  | Capp(Capp(C_S, Capp(Capp(C_B, f), a)), b) as t ->
      if !trace_opt then print_string "--- opt8: S (B p q) r == S4 p q r\n";
      optimise(Capp(Capp(Capp(C_S4, f), a), b))

	(** optimisation: S (K p) q == B p q   **)
  | Capp(Capp(C_S, Capp(C_K, p)), q) ->
      if !trace_opt then print_string "--- opt2: S (K p) q == B p q\n";
      optimise(Capp(Capp(C_B, p), q))

	(** optimisation: S p (K q) == C p q   **)
  | Capp(Capp(C_S, p), Capp(C_K, q)) ->
      if !trace_opt then print_string "--- opt3: S p (K q) == C p q\n";
      optimise(Capp(Capp(C_C, p), q))

        (** optimisation: B p (K q)  ==  K (p q) **)
  | Capp(Capp(C_B, p), Capp(C_K, q)) ->  
      if !trace_opt then print_string "--- opt4:  B p (K q)  ==  K (p q)\n";
      optimise(Capp(C_K, Capp(p, q)))

        (** optimisation: C (K p) q  ==  K (p q) **)
  | Capp(Capp(C_C, Capp(C_K, p)), q) ->  
      if !trace_opt then print_string "--- opt5:  C (K p) q  ==  K (p q)\n";
      optimise(Capp(C_K, Capp(p, q)))

        (** optimisation: B p I  ==  p  **)
  | Capp(Capp(C_B, p), C_I) ->  
     if !trace_opt then print_string "--- opt6:  B p I  ==  p\n";
      optimise p


  (** -- opti vers une fonction predefinie... -- **)

  | Capp(Capp(Capp(C_S4, C_S), a), b) -> Capp(Capp(Cd "S4_S", a), b)

  | Capp(C_B, C_K) ->
      Cd("B_K")

(****
  | Capp(Capp(Capp(C_S4, f), a), Capp(C_K, b)) ->
      Capp(Capp(Cd "S4B4", a), b)

  | Capp(Capp(Capp(C_S4, f), Capp(C_K, a)), b) ->
      Capp(Capp(Cd "S4C4", a), b)
***)



  (** -- opti semantique: exemples... -- **)

        (** optimisation: si op <commutatif>: C op a  ==  op a   **)
  | Capp(Capp(C_C, Cd op), q)
    when op = "plus" or op = "equal" ->
      (if !trace_opt then 
        print_string("--- opt7:  commutativite "^op^": C op a  ==  op p\n"));
      Capp(Cd op, q)

(***
        (** optimisation: + 1 = succ   **)
  | Capp(Cd "plus", Cv(Num 1)) ->
      (if !trace_opt then print_string "--- opt8:  + 1 = succ\n");
      Cd("succ")
***)

	(** associativite de Fby **)
  | Cupl(Ofby, level, l) as t
    when is_tup_in_list level l ->
      (if !trace_opt then print_string "--- opt9:  flatify fby\n");
      Cupl(Ofby, level, flatify level l)

	(** extension de l'application **)
(** Cette regle est FAUSSE. considerer par exemple: "f := beep; (\x.f)(1);;"
  | Capp(Cupl(Ofct s, level, l), b) as t ->
      (if !trace_opt then print_string "--- opt10:  extension Cup\n");
      Cupl(Ofct s, level, l@[b])
**)


  (** -- Propagation en profondeur des optimisations et Point-fixe. -- **)

  | Capp(p, q) as t -> 
      Capp(optimise p, optimise q)

  | Cupl(op, level, l) as t ->
      Cupl(op, level, List.map optimise l)

  | x -> x
;;



