/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
#ifndef G_MAP_GENERIC_HH
#define G_MAP_GENERIC_HH
//******************************************************************************
#include "g-map.hh"
#include "math-extension.hh"
#include "transformation-matrix.hh"
//******************************************************************************

namespace GMap3d
{
  /**
   * Dfinit la classe CGMapGeneric qui reprsente une 3-G-carte.
   * Les oprations implmentes dans cette classe sont toutes topologiques.
   * Celles qui modifient la carte ne mettent pas  jour les
   * plongements (exemple: triangulation, qui ne duplique pas
   * les plongements 'face' lors de la triangulation d'une face).
   * La mise  jour des plongements doit donc tre faite dans les
   * mthodes des classes qui drivent de CGMapGeneric.
   *
   * @author MOKA TEAM
   */
  class CGMapGeneric: public CGMap
  {
    // @name Constructeur et destructeur
    // @{
#include "gmg-instance.hh"
    // @}

    // @name Accesseurs inlins
    // @{
#include "gmg-inline.hh"
    // @}

    // @name Mthodes de marquage
    // @{
#include "gmg-selection.hh"
#include "gmg-exploration.hh"
    // @}

    /**
     * @name Mthodes permettant d'allouer et de librer de la mmoire aux
     * champs directInfo des brins, ainsi que d'effectuer la sauvegarde de
     * coutures.
     */
    // @{
#include "gmg-direct-info.hh"
    // @}

    // @name Primitives
    // @{
#include "gmg-primitive.hh"
#include "gmg-primitive-mesh.hh"
    // @}

    // @name Manipulation de polylignes
    // @{
#include "gmg-polyline.hh"
    // @}

    // @name Informations topologiques
    // @{
#include "gmg-topology.hh"
#include "gmg-degree.hh"
    // @}

    // @name Coutures / dcousures
    // @{
#include "gmg-sew.hh"
#include "gmg-unsew.hh"
    // @}

    // @name Duplication et suppression de brins
    // @{
#include "gmg-duplication.hh"
#include "gmg-deletion.hh"
    // @}

    // @name Mthode de manipulation des bords
    // @{
#include "gmg-border.hh"
#include "gmg-stop-up.hh"
    // @}

    // @name Insertions (splits)
    // @{
#include "gmg-insertion.hh"
    // @}

    // @name Fusions (merges) / contractions / suppression de cellules "plates"
    // @{
#include "gmg-merge.hh"
#include "gmg-contraction.hh"
#include "gmg-simplification.hh"
    // @}

    // @name Triangulations et quadrangulations barycentriques
    // @{
#include "gmg-triangulation.hh"
#include "gmg-quadrangulation.hh"
    // @}

    // @name Extrusions et paississement
    // @{
#include "gmg-extrusion.hh"
#include "gmg-thickening.hh"
    // @}

    // @name Mthodes de traitement des artes fictives.
    // @{
#include "gmg-fictive-edge.hh"
    // @}

  };
  
} // namespace GMap3d
//******************************************************************************
#include INCLUDE_INLINE("gmg-inline.icc")
//******************************************************************************
#endif // G_MAP_GENERIC_HH
//******************************************************************************
