/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-generic.hh".
//******************************************************************************

public:

/**
 * Calcule la dimension du bord incident au brin ADart.
 *
 * @param ADart Un brin quelconque
 * @return La dimension du bord de plus petite dimension incident au brin
 *
 * @postcondition La valeur retourne vaut 0, 1, 2, 3 ou 4.
 */
int getBorderDimension(CDart* ADart);

/**
 * Marque les bords de dimension ADim incidents aux brins slectionns.
 *
 * @param AMarkNumber Un numro de marque
 * @param ADim Une dimension (0, 1, 2 ou 3)
 * @return Un boolen indiquant si des bords ont t slectionns
 */
bool markBorders(int AMarkNumber, int ADim);

/**
 * Marque les bords incidents aux brins slectionns.
 * La dimension des bord est automatiquement dtermine
 * avec la mthode 'getBorderDimension'.
 *
 * @param AMarkNumber Un numro de marque
 * @return Un boolen indiquant si des bords ont t slectionns
 */
bool markBorders(int AMarkNumber);

/**
 * Teste si les deux bords de dimension ADim dsigns par les brins
 * ADart1 et ADart2 peuvent tre cousus.
 * Si An est strictement positif, on teste si la couture est possible sur
 * les An premiers brins parcourus sur chaque bord seulement.
 *
 * @param ADart1 Un pointeur sur un premier brin
 * @param ADart2 Un pointeur sur un deuxime brin
 * @param ADim Une dimension
 * @param An Le nombre d'itrations  effectuer
 * @return Un boolen indiquant si les deux bords peuvent tre cousus.
 */
bool canSewBorders(CDart* ADart1, CDart* ADart2, int ADim, int An = 0);

/**
 * Coud les deux bords de dimension ADim dsigns par les brins
 * ADart1 et ADart2.
 * Si An est strictement positif, sa valeur indique le nombre de
 * coutures  effectuer (nombre d'itrations  effectuer sur
 * l'itrateur de bord).
 *
 * @param ADart1 Un pointeur sur un premier brin
 * @param ADart2 Un pointeur sur un deuxime brin
 * @param ADim Une dimension
 * @param An Le nombre d'itrations  effectuer
 *
 * @precondition canSewBorders(ADart1, ADart2, ADim, An)
 */
void borderTopoSew(CDart* ADart1, CDart* ADart2, int ADim, int An = 0);

/**
 * Coud les deux bords de dimension ADim dsigns par les brins
 * ADart1 et ADart2.
 * Si An est strictement positif, sa valeur indique le nombre de
 * coutures  effectuer (nombre d'itrations  effectuer sur
 * l'itrateur de bord).
 *
 * @param ADart1 Un pointeur sur un premier brin
 * @param ADart2 Un pointeur sur un deuxime brin
 * @param ADim Une dimension
 * @param An Le nombre d'itrations  effectuer
 *
 * @precondition canSewBorders(ADart1, ADart2, ADim, An)
 */
void borderSew(CDart* ADart1, CDart* ADart2, int ADim, int An = 0);

/**
 * Coud les deux bords dsigns par les brins ADart1 et ADart2.
 * La dimension des bords est dtermine par la mthode
 * 'getBorderDimension' applique sur ADart1.
 *
 * Si An est strictement positif, sa valeur indique le nombre de
 * coutures  effectuer (nombre d'itrations  effectuer sur
 * l'itrateur de bord).
 *
 * @param ADart1 Un pointeur sur un premier brin
 * @param ADart2 Un pointeur sur un deuxime brin
 * @param An Le nombre d'itrations  effectuer
 * @return Un boolen indiquant si la couture alpha t possible
 */
bool intuitiveBorderSew(CDart* ADart1, CDart* ADart2, int An = 0);

/**
 * Coud les deux 2-bords dsigns par les brins ADart1 et ADart2.
 * Si An est strictement positif, sa valeur indique le nombre d'ARTES  coudre.
 *
 * @param ADart1 Un pointeur sur un premier brin
 * @param ADart2 Un pointeur sur un deuxime brin
 * @param An Le nombre d'artes  coudre
 *
 * @precondition canSewBorders(ADart1, ADart2, 2, 2*An)
 */
void border2TopoSew(CDart* ADart1, CDart* ADart2, int An = 0);

/**
 * Coud les deux 2-bords dsigns par les brins ADart1 et ADart2.
 * Si An est strictement positif, sa valeur indique le nombre d'ARTES  coudre.
 *
 * @param ADart1 Un pointeur sur un premier brin
 * @param ADart2 Un pointeur sur un deuxime brin
 * @param An Le nombre d'artes  coudre
 *
 * @precondition canSewBorders(ADart1, ADart2, 2, 2*An)
 */
void border2Sew(CDart* ADart1, CDart* ADart2, int An = 0);

//******************************************************************************
