/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-generic.hh".
//******************************************************************************

public:

/**
 * Calcule le degr d'une cellule de dimension ACellDim, c'est--dire le
 * nombre de cellules de dimension ACellDim+1 qui lui sont incidentes.
 * Cette mthode est utile pour tester si une fusion est possible
 * (mthode 'canMerge').
 *
 * La valeur de ABoundingCellDim indique la dimension de la cellule
 * englobante.
 * Par exemple, si ACellDim==0 et ABoundingCellDim==3, la mthode calcule
 * le nombre d'artes incidentes au sommet dsign par ADart dont un brin
 * au moins appartient au volume incident  ADart.
 *
 * @param ADart Un brin de la carte
 * @param ACellDim Une dimension
 * @param ABoundingCellDim La dimension de la cellule englobante
 * @return La dimension de la cellule de dimension ACellDim incidente  ADart.
 *
 * @precondition 0 <= ACellDim < 3
 * @precondition ACellDim < ABoundingCell <= 4
 */
int degree(CDart* ADart, int ACellDim, int ABoundingCellDim = 4);

/**
 * Vrifie si la condition alpha_i+1 o alpha_i+2 est une involution pour
 * chaque brin de la cellule.
 *
 * @param ADart Un brin de la carte
 * @param ACellDim Une dimension
 * @return vrai ssi la cellule de dimension ACellDim incidente 
 *   ADart est de "degr local suprieur" gal  2.
 *
 * @precondition 0 <= ACellDim < 3
 * @precondition ADart!=NULL
 */
bool isLocalDegreeTwoSup(CDart* ADart, int ACellDim);

/**
 * Vrifie si la condition alpha_i-1 o alpha_i-2 est une involution pour
 * chaque brin de la cellule.
 *
 * @param ADart Un brin de la carte
 * @param ACellDim Une dimension
 * @return vrai ssi la cellule de dimension ACellDim incidente 
 *   ADart est de "degr local infrieur" gal  2.
 *
 * @precondition 0 <= ACellDim < 3
 * @precondition ADart!=NULL
 */
bool isLocalDegreeTwoInf(CDart* ADart, int ACellDim);

//******************************************************************************
