/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-generic.hh".
//******************************************************************************

public:

/**
 * Alloue ABytes octets au champ directInfo[ADirectInfoIndex] des brins
 * marqus avec la marque AMarkNumber.
 *
 * Si AOrbitUsed est diffrent de ORBIT_NONE, l'allocation n'est faite que
 * pour le brin porteur du plongement AOrbitUsed de chaque cellule d'orbite
 * AOrbitUsed dont un brin au moins est marqu.
 *
 * @param AMarkNumber Un numro de marque
 * @param ADirectInfoIndex L'indice du champ directInfo  utiliser
 * @param ABytes Le nombre d'octets  allouer
 * @param AOrbitUsed Une orbite quelconque
 */
void allocMemoryToDirectInfo(int AMarkNumber, int ADirectInfoIndex,
			     int ABytes, TOrbit AOrbitUsed = ORBIT_NONE);

/**
 * Alloue ABytes octets au champ directInfo[ADirectInfoIndex] des brins
 * correspondant  l'orbite donne par le couple (ADart, AOrbit).
 *
 * Si AOrbitUsed est diffrent de ORBIT_NONE, l'allocation n'est faite que
 * pour le brin porteur du plongement AOrbitUsed de chaque cellule d'orbite
 * AOrbitUsed dont un brin au moins est sur l'orbite (ADart, AOrbit).
 *
 * @param ADart Un brin quelconque de la carte
 * @param AOrbit Une orbite
 * @param ADirectInfoIndex L'indice du champ directInfo  utiliser
 * @param ABytes Le nombre d'octets  allouer
 * @param AOrbitUsed Une orbite quelconque
 */
void allocMemoryToDirectInfo(CDart* ADart, TOrbit AOrbit,
			     int ADirectInfoIndex,
			     int ABytes, TOrbit AOrbitUsed = ORBIT_NONE);

/**
 * Libre l'espace mmoire affect au champ directInfo[ADirectInfoIndex]
 * des brins marqus avec la marque AMarkNumber.
 *
 * Si AOrbitUsed est diffrent de ORBIT_NONE, l'allocation n'est faite que
 * pour le brin porteur du plongement AOrbitUsed de chaque cellule d'orbite
 * AOrbitUsed dont un brin au moins est marqu.
 *
 * @param AMarkNumber Un numro de marque
 * @param ADirectInfoIndex L'indice du champ directInfo  utiliser
 * @param AOrbitUsed Une orbite quelconque
 */
void freeMemoryFromDirectInfo(int AMarkNumber, int ADirectInfoIndex,
			      TOrbit AOrbitUsed = ORBIT_NONE);

/**
 * Libre l'espace mmoire affect au champ directInfo[ADirectInfoIndex]
 * des brins marqus avec la marque AMarkNumber.
 *
 * Si AOrbitUsed est diffrent de ORBIT_NONE, la libration n'est faite que
 * pour le brin porteur du plongement AOrbitUsed de chaque cellule d'orbite
 * AOrbitUsed dont un brin au moins est marqu.
 *
 * @param ADart Un brin quelconque de la carte
 * @param AOrbit Une orbite
 * @param ADirectInfoIndex L'indice du champ directInfo  utiliser
 * @param AOrbitUsed Une orbite quelconque
 */
void freeMemoryFromDirectInfo(CDart* ADart, TOrbit AOrbit,
			      int ADirectInfoIndex,
			      TOrbit AOrbitUsed = ORBIT_NONE);

/**
 * Pour chaque brin b de la carte, copie la valeur de l'image par alpha
 * AAlphaIndex de b dans son champ direcInfo d'indice ADirectInfoIndex.
 *
 * Remarque: La mthode 'getDirectInfoAsDart' permet de rcuprer aisment
 * les valeurs sauves.
 *
 * @param AAlphaIndex Une dimension (0, 1, 2 ou 3)
 * @param ADirectInfoIndex Un indice de champ directInfo
 */
void saveAlphaInDirectInfo(int AAlphaIndex, int ADirectInfoIndex);

//******************************************************************************
