/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-generic.hh".
//******************************************************************************

public:

/**
 * Teste si le brin ADart appartient  une polyligne isole, c'est--dire
 * si tous les brins de la face incidente  ADart sont 2 et 3-libres.
 *
 * @param ADart Un brin de la carte
 * @return 'vrai' si ADart appartient  une polyligne
 */
bool isIsolatedPolyline(CDart* ADart);

/**
 * Teste si le brin ADart appartient  une polyligne dont tous les brins
 * sont 0 et 1-cousus.
 * Remarque: Certains brins de la polyligne peuvent tre 2 ou 3-cousus.
 *
 * @param ADart Un brin de la carte
 * @return 'vrai' si ADart appartient  une face ferme
 */
bool isClosedPolyline(CDart* ADart);

/**
 * Calcule le nombre de sommets qui composent la face incidente  ADart.
 *
 * @param ADart Un brin de la carte
 * @return Un nombre de sommets
 */
int getNbPolylineVertices(CDart* ADart);

/**
 * Calcule le nombre d'artes qui composent la face incidente  ADart.
 *
 * Si AConsider0FreeDartsAsEdges vaut 'vrai' et si ADart est incident  une
 * face ouverte, les ventuels brins 0-libres situs au bout de cette face
 * sont comptabiliss comme des artes.
 *
 * Si AConsider0FreeDartsAsEdges vaut 'faux', les ventuels brins 0-libres
 * situs au bout d'une face ouverte ne sont pas considrs dans le
 * dcompte du nombre d'artes.
 *
 * @param ADart Un brin de la carte
 * @param AConsider0FreeDartsAsEdges Un boolen indiquant comment
 *   considrer les brins 0-libres
 * @return Un nombre d'artes
 */
int getNbPolylineEdges(CDart* ADart, bool AConsider0FreeDartsAsEdges = true);

/**
 * Prolonge une polyligne (en rajoutant une arte  son bout).
 * Si ADart vaut NULL, cre un premier brin d'une nouvelle polyligne.
 *
 * @param ADart Un brin constituant le bout d'une polyligne existante
 * @return Le nouveau bout de la polyligne
 *
 * @precondition isIsolatedPolyline(CDart)
 */
CDart* prolongPolyline(CDart* ADart);

/**
 * Dtruit la dernire arte (dont le brin 0-libre est donn par ADart)
 * d'une polyligne.
 *
 * @param ADart Le bout d'une polyligne
 * @return Le nouveau bout de la polyligne
 *
 * @precondition isIsolatedPolyline(ADart)
 */
CDart* shortenPolyline(CDart* ADart);

/**
 * Ferme une polyligne pour former une face.
 *
 * @param ADart Un brin quelconque d'une polyligne existante
 * @return Un brin de l'arte cre pour fermer la polyligne
 *
 * @precondition isIsolatedPolyline(CDart)
 */
CDart* closePolyline(CDart* ADart);

//******************************************************************************
