/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-generic.hh".
//******************************************************************************

public:

/**
 * Teste s'il est possible de quadranguler la face incidente  ADart.
 * C'est toujours le cas si AInsertVertices vaut 'vrai'.
 * Si AInsertVertices vaut 'faux', il faut que le nombre d'artes de la
 * face soit pair.
 *
 * @param ADart Un brin de la carte
 * @param AInsertVertices Un boolen indiquant s'il faut insrer des
 *   sommets sur les artes de la face avant d'effectuer la quadrangulation
 * @return Un boolen indiquant s'il est possible de quadranguler la face
 */
bool canQuadrangulateFace(CDart* ADart, bool AInsertVertices = true);

/**
 * Quadrangule la face incidente  ADart.
 *
 * Si AInsertVertices vaut 'vrai', un sommet est insr sur chaque arte de
 * la face avant d'effectuer la quadrangulation.
 *
 * Si AInsertVertices vaut 'faux', la quadrangulation est directement
 * effectue. Dans ce cas, il faut que le nombre d'artes qui composent la
 * face soit pair. Un sommet sur deux de la face est modifi. Celui
 * incident  ADart ne l'est pas. Si la face est ouverte, selon la position
 * de ADart sur celle-ci, les artes qui sont au bout de la polyligne
 * seront ou non sur une face ouverte.
 *
 * @param ADart Un brin de la carte
 * @param AInsertVertices Un boolen indiquant s'il faut insrer des sommets
 *   sur les artes de la face avant d'effectuer la quadrangulation
 * @return Un brin du sommet topologique cr
 */
virtual CDart* quadrangulateFace(CDart* ADart, bool AInsertVertices = true);

/**
 * Quadrangule les faces dont un brin au moins est marqu.
 * Pour plus d'informations sur le paramtre AInsertVertices, se reporter 
 * la mthode 'quadrangulateFace'.
 *
 * @param AMarkNumber Un numro de marque
 * @param AInsertVertices Un boolen indiquant s'il faut insrer des sommets sur
 *   les artes des faces avant d'effectuer la quadrangulation
 * @return Le nombre de quadrangulations effectues
 */
int quadrangulateMarkedFaces(int AMarkNumber, bool AInsertVertices = true);

//******************************************************************************
