/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-generic.hh".
//******************************************************************************

public:

/**
 * Marque tous les brins de l'orbite AOrbit incidente  ADart en utilisant
 * la marque numro AMarkNumber.
 *
 * @param ADart Un brin quelconque
 * @param AOrbit Une orbite
 * @param AMarkNumber Un numro de marque
 */
void markOrbit(CDart* ADart, TOrbit AOrbit, int AMarkNumber);

/**
 * Marque un brin sur deux de l'orbite AOrbit incidente  ADart en
 * utilisant la marque numro AMarkNumber.
 * Attention: Cette mthode ne fonctionne pas si l'orbite est une orbite de
 * bord !
 *
 * @param ADart Un brin quelconque
 * @param AOrbit Une orbite
 * @param AMarkNumber Un numro de marque
 *
 * @precondition AOrbit n'est pas une orbite de bord
 * @precondition isOrientable(ADart, AOrbit)
 * @postcondition isMarked(ADart, AMarkNumber)
 * @postcondition Soient b un brin de l'orbite (ADart,AOrbit) et i une
 *   dimension (0, 1, 2 ou 3) tels que alpha(b,i) soit lui aussi sur l'orbite
 *   (ADart, AOrbit). On a :
 *     isMarked(      b   , AMarkNumber) !=
 *     isMarked(alpha(b,i), AMarkNumber)
 */
void halfMarkOrbit(CDart* ADart, TOrbit AOrbit, int AMarkNumber);

/**
 * Dmarque un brin sur deux de l'orbite AOrbit incidente  ADart en
 * utilisant la marque numro AMarkNumber.
 * Attention: Cette mthode ne fonctionne pas si l'orbite est une orbite de
 * bord !
 *
 * @param ADart Un brin quelconque
 * @param AOrbit Une orbite
 * @param AMarkNumber Un numro de marque
 *
 * @precondition AOrbit n'est pas une orbite de bord
 * @precondition isOrientable(ADart, AOrbit)
 * @postcondition !isMarked(ADart, AMarkNumber)
 * @postcondition Soient b un brin de l'orbite (ADart,AOrbit) et i une
 *   dimension (0, 1, 2 ou 3) tels que alpha(b,i) soit lui aussi sur l'orbite
 *   (ADart, AOrbit). On a :
 *     isMarked(      b   , AMarkNumber) !=
 *     isMarked(alpha(b,i), AMarkNumber)
 */
void halfUnmarkOrbit(CDart* ADart, TOrbit AOrbit, int AMarkNumber);

/**
 * Dmarque tous les brins de l'orbite AOrbit incidente  ADart pour la
 * marque numro AMarkNumber.
 *
 * @param ADart Un brin quelconque
 * @param AOrbit Une orbite
 * @param AMarkNumber Un numro de marque
 */
void unmarkOrbit(CDart* ADart, TOrbit AOrbit, int AMarkNumber);

/**
 * Dmarque tous les brins de la carte pour la marque numro AMarkNumber.
 *
 * @param AMarkNumber Un numro de marque
 */
void unmarkAll(int AMarkNumber);

/**
 * Marque avec la marque AMarkNumberDest toutes les cellules correspondant
 *  l'orbite AOrbit dont un brin au moins est marqu avec la marque
 * AMarkNumberSrce.
 *
 * @param AOrbit Une orbite quelconque
 * @param AMarkNumberSrce Un numro de marque
 * @param AMarkNumberDest Un numro de marque
 * @return Le nombre de cellules marques
 *
 * @precondition AMarkNumberSrce!=AMarkNumberDest
 */
int markIncidentCells(TOrbit AOrbit,
		      int AMarkNumberSrce, int AMarkNumberDest);

/**
 * Marque entirement les cellules correspondant  l'orbite AOrbit dont un
 * brin au moins est marqu.
 *
 * @param AOrbit Une orbite quelconque
 * @param AMarkNumber Un numro de marque
 * @return Le nombre de cellules marques
 */
int markIncidentCells(TOrbit AOrbit, int AMarkNumber);

/**
 * Marque avec la marque AMarkNumberDest un brin sur deux des cellules
 * correspondant  l'orbite AOrbit dont un brin au moins est marqu avec la
 * marque AMarkNumberSrce.
 * Lorsqu'une cellule non orientable est marque, elle est totalement
 * slectionne.
 * Voir la mthode 'halfMarkOrbit' pour plus d'informations.
 *
 * @param AOrbit Une orbite qui n'est pas une orbite de bord
 * @param AMarkNumberSrce Un numro de marque
 * @param AMarkNumberDest Un numro de marque
 *
 * @precondition AOrbit>=ORBIT_SELF && AOrbit<=Orbit_CC
 * @precondition AMarkNumberSrce!=AMarkNumberDest
 */
void halfMarkIncidentCells(TOrbit AOrbit,
			   int AMarkNumberSrce, int AMarkNumberDest);

/**
 * Marque un brin sur deux des cellules correspondant  l'orbite AOrbit
 *  dont un brin au moins est marqu.
 * Lorsqu'une cellule non orientable est marque, elle est totalement
 * slectionne.
 * Voir la mthode 'halfMarkOrbit' pour plus d'informations.
 *
 * @param AOrbit Une orbite qui n'est pas une orbite de bord
 * @param AMarkNumber Un numro de marque
 *
 * @precondition AOrbit>=ORBIT_SELF && AOrbit<=Orbit_CC
 */
void halfMarkIncidentCells(TOrbit AOrbit, int AMarkNumber);

/**
 * Marque tous les brins de la carte qui sont  l'intrieur d'une rgion
 * dont le bord est donn par les brins marqus avec la marque AMarkBorder.
 * ADart est un brin situ  l'intrieur de la rgion.
 * Les brins sont marqus avec la marque AMarkInterior. Les brins du bord ne
 * sont pas marqus.
 *
 * @param ADart Un brin de la carte
 * @param ADimension Une dimension (2 ou 3)
 * @param AMarkBorder Un numro de marque
 * @param AMarkInterior Un numro de marque
 */
void markInterior(CDart* ADart, int ADimension,
		  int AMarkBorder, int AMarkInterior = -1);

/**
 * Cette mthode permet de slectionner une partie du bord d'une cellule en
 * calculant l'intersection entre cette cellule et une autre qui lui est
 * adjacente.
 *
 * @param AMarkNumber1 un numro de marque
 * @param AMarkNumber2 un numro de marque diffrent de AMarkNumber1
 * @param AOrbit1 une orbite quelconque (orbite de slection)
 * @param AOrbit2 une orbite quelconque (orbite du rsultat)
 * @param AMarkResult la marque o est stock le rsultat
 * @param AInitialMarkIncidentCells un boolen indiquant s'il faut
 *   initialiser les marquages
 */
void markIntersection(int AMarkNumber1, int AMarkNumber2,
		      TOrbit AOrbit1, TOrbit AOrbit2,
		      int AMarkResult,
		      bool AInitialMarkIncidentCells = false);

/**
 * Pour chacun des brins de l'orbite dtermine par ADart et AOrbit,
 * positionne la marque AMarkNumberTo sur le mme tat que la marque
 * AMarkNumberFrom.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 * @param ADart Un brin de la carte
 * @param AOrbit Une orbite quelconque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markCopy(int AMarkNumberFrom, int AMarkNumberTo,
	      CDart* ADart, TOrbit AOrbit);

/**
 * Pour chacun des brins de la carte,
 * positionne la marque AMarkNumberTo sur le mme tat que la marque
 * AMarkNumberFrom.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markCopy(int AMarkNumberFrom, int AMarkNumberTo);

/**
 * Pour chacun des brins de la carte,
 * positionne la marque AMarkNumberTo  vrai si la marque AMarkNumberFrom
 * est positionne  vrai.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markAdd(int AMarkNumberFrom, int AMarkNumberTo);

/**
 * Pour chacun des brins de l'orbite dtermine par ADart et AOrbit,
 * positionne la marque AMarkNumberTo  vrai si la marque AMarkNumberFrom
 * est positionne  vrai.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 * @param ADart Un brin de la carte
 * @param AOrbit Une orbite quelconque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markAdd(int AMarkNumberFrom, int AMarkNumberTo,
	     CDart* ADart, TOrbit AOrbit);

/**
 * Pour chacun des brins de la carte,
 * positionne la marque AMarkNumberTo  faux si la marque AMarkNumberFrom est
 * positionne  faux.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markAnd(int AMarkNumberFrom, int AMarkNumberTo);

/**
 * Pour chacun des brins de l'orbite dtermine par ADart et AOrbit,
 * positionne la marque AMarkNumberTo  faux si la marque AMarkNumberFrom
 * est positionne  faux.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 * @param ADart Un brin de la carte
 * @param AOrbit Une orbite quelconque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markAnd(int AMarkNumberFrom, int AMarkNumberTo,
	     CDart* ADart, TOrbit AOrbit);

/**
 * Pour chacun des brins de la carte,
 * positionne la marque AMarkNumberTo  faux si la marque AMarkNumberFrom
 * est positionne  vrai.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markSub(int AMarkNumberFrom, int AMarkNumberTo);

/**
 * Pour chacun des brins de l'orbite dtermine par ADart et AOrbit,
 * positionne la marque AMarkNumberTo  faux si la marque AMarkNumberFrom
 * est positionne  vrai.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 * @param ADart Un brin de la carte
 * @param AOrbit Une orbite quelconque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markSub(int AMarkNumberFrom, int AMarkNumberTo,
	     CDart* ADart, TOrbit AOrbit);

/**
 * Pour chacun des brins de la carte,
 * positionne la marque AMarkNumberTo  vrai si la marque AMarkNumberFrom
 * est positionne  vrai, puis positionne la marque AMarkNumberFrom 
 * faux.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markMove(int AMarkNumberFrom, int AMarkNumberTo);

/**
 * Pour chacun des brins de l'orbite dtermine par ADart et AOrbit,
 * positionne la marque AMarkNumberTo  vrai si la marque AMarkNumberFrom
 * est positionne  vrai, puis positionne la marque AMarkNumberFrom  faux.
 *
 * @param AMarkNumberFrom Un numro de marque
 * @param AMarkNumberTo Un numro de marque
 * @param ADart Un brin de la carte
 * @param AOrbit Une orbite quelconque
 *
 * @precondition AMarkNumberFrom!=AMarkNumberTo
 */
void markMove(int AMarkNumberFrom, int AMarkNumberTo,
	      CDart* ADart, TOrbit AOrbit);

/**
 * Teste si tous les brins de la cellule dtermine par le brin ADart et
 * l'orbite AOrbit sont marqus avec la marque AMarkNumber.
 *
 * @param ADart Un brin quelconque de la carte
 * @param AOrbit Une orbite quelconque
 * @param AMarkNumber Un numro de marque
 * @return Un boolen
 */
bool isWholeCellMarked(CDart* ADart, TOrbit AOrbit, int AMarkNumber);

/**
 * Cherche dans la carte les cellules correspondant  AOrbit dont au moins
 * un brin est marqu pour le numro de marque AMarkNumber donn.
 * Pour chacune d'entre elles, un brin marqu est choisi et l'un des
 * paramtres ADart- est positionn sur ce brin.
 * Si plus de 3 cellules sont marques, seules les trois premires trouves
 * sont mmorises et la valeur retourne est suprieure  trois.
 * Si moins de 3 cellules sont marques, les paramtres non utiliss sont
 * positionns  NULL.
 * Si ALastMarked ne vaut pas NULL, il dsigne un brin prioritaire sur les
 * autres. Si ce brin est marqu, il est plac en premire position (ADart1
 * pointe sur lui).
 *
 * @param AOrbit Une orbite quelconque
 * @param AMarkNumber Un numro de marque
 * @param ALastMarked Un pointeur sur le dernier brin marqu
 * @param ADart1 Une rfrence sur un pointeur sur un brin
 * @param ADart2 Une rfrence sur un pointeur sur un brin
 * @param ADart3 Une rfrence sur un pointeur sur un brin
 * @return Le nombre d'orbites marques
 */
int getMarkedCells(TOrbit AOrbit, int AMarkNumber,
		   CDart* ALastMarked = NULL,
		   CDart* * ADart1 = NULL,
		   CDart* * ADart2 = NULL,
		   CDart* * ADart3 = NULL);

//******************************************************************************
