/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-generic.hh".
//******************************************************************************

public:

/**
 * Triangule l'arte incidente  ADart (la "coupe" en deux).
 *
 * @param ADart Un brin de la carte
 * @return Un brin du sommet topologique cr
 */
virtual CDart* triangulateEdge(CDart* ADart);

/**
 * Triangule la face incidente  ADart.
 *
 * @param ADart Un brin de la carte
 * @return Un brin du sommet topologique cr
 */
virtual CDart* triangulateFace(CDart* ADart);

/**
 * Triangule le volume incident  ADart.
 *
 * @param ADart Un brin de la carte
 * @return Un brin du sommet topologique cr
 */
virtual CDart* triangulateVolume(CDart* ADart);

/**
 * Triangule la cellule de dimension ADim incidente  ADart.
 *
 * @param ADart Un brin de la carte
 * @return Un brin du sommet topologique cr
 *
 * @precondition ADim>1 et ADim<=3
 */
CDart* triangulate(CDart* ADart, int ADim);

/**
 * Triangule les cellules de dimension ADim dont un brin au moins est marqu.
 *
 * @param AMarkNumber Un numro de marque
 * @param ADim Une dimension (1, 2 ou 3)
 * @return Le nombre de triangulations effectues
 */
int triangulateMarkedCells(int AMarkNumber, int ADim);

//******************************************************************************
