/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
#ifndef DART_VERTEX_HH
#define DART_VERTEX_HH
//******************************************************************************
#include "dart.hh"
#include "attribute-vertex.hh"

#include <bitset>
#include <iostream>
//******************************************************************************
namespace GMap3d
{
  class CDartVertex: public CDart
  {
  public:
    // @name Constructeurs / destructeur
    // @{

    /**
     * Constructeur par dfaut
     */
    CDartVertex();

    /**
     * Constructeur par copie
     */
    CDartVertex(const CDartVertex& ADart);

    /**
     * Constructeur avec initialisation des marques
     *
     * @param AMarks Les marques telles qu'elles doivent tre positionnes
     * @return Une instance de la classe
     */
    CDartVertex(const std::bitset<NB_MARKS>& AMarks);

    /**
     * Constructeur avec initialisation des marques et du sommet clat
     *
     * @param AMarks Les marques telles qu'elles doivent tre positionnes
     * @param AVertex Un sommet correspondant au plongement clat
     * @return Une instance de la classe
     */
    CDartVertex(const std::bitset<NB_MARKS>& AMarks, const CVertex& AVertex);

    /**
     * Constructeur avec lecture du brin dans un flot
     *
     * @param AStream Un flot dans lequel lire le contenu du brin
     * @param AFormat Le type de format dans lequel le flot est cod
     * @return Une instance de la classe
     */
    CDartVertex(std::istream& AStream, TSaveFormat AFormat);

    /// Destructeur
    ~CDartVertex();

    // @}
    // @name Accesseur au sommet clat
    // @{

    CVertex & getBurstVertex();

    // @}
    // @name Mthodes de sauvegarde et de chargement
    // @{

    /**
     *  Sauvegarde le contenu du brin dans le flot AStream en fonction du format
     *  demand.
     *  Remarque: Lorsque cette mthode est appele, on suppose que tous les
     *  brins ont t numrots et que ces numros ont t placs dans un champ
     *  'directInfo' dont l'indice est pass en paramtre.
     *
     *  @param AStream Un flot o sauvegarder le brin
     *  @param AFormat Le type de format dans lequel doit se faire la sauvegarde
     *  @param ADirectInfoIndex Un indice indiquant o sont stocks les numros
     *   des brins
     *  @return 'vrai' si tout s'est bien pass
     */
    bool save(std::ostream& AStream,
	      TSaveFormat AFormat, int ADirectInfoIndex) const;

    /**
     * Charge le contenu d'un brin dans le flot AStream en fonction du format
     * demand.
     * Remarque:  l'issue de cette mthode, les champs alphaI du brin ne sont
     * pas valides: ils correspondent  des numros de brins et non pas  des
     * pointeurs sur des brins. Ces champs doivent tre encore modifis par la
     * mthode globale de chargement, situe dans la classe GmapV.
     *
     * @param AStream Un flot dans lequel lire le contenu du brin
     * @param AFormat Le type de format dans lequel le flot est cod
     * @return 'vrai' si tout s'est bien pass
     */
    bool load(std::istream& AStream, TSaveFormat AFormat);

    // @}

  private:
    // @name Attributs privs
    // @{

    /// Plongement clat du brin
    CVertex FBurstVertex;

    // @}
  };

} // namespace GMap3d
//******************************************************************************
#include INCLUDE_INLINE("dart-vertex.icc")
//******************************************************************************
#endif // DART_VERTEX_HH
//******************************************************************************
