/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
#ifndef G_MAP_VERTEX_HH
#define G_MAP_VERTEX_HH
//******************************************************************************
#include <cmath>
#include <cstdio>
#include <fstream>
#include <set> // pour les mthodes auxiliaires de co-raffinement.

#include "g-map-generic.hh"
#include "dart-vertex.hh"

#include "vertex.hh"
#include "vector.hh"
#include "transformation-matrix.hh"
#include "mesh-diver.hh"

#include "alt-stdint.hh"
#include "inline-macro.hh"
//******************************************************************************
namespace GMap3d
{
  /**
   * Dfinit la classe CGMapVertex (carte plonge sommets).
   *
   * @author MOKA TEAM
   */
  class CGMapVertex : public CGMapGeneric
  {
  public:
    // @name Constructeur, destructeur et mthode d'initialisation
    // @{
#include "gmv-instance.hh"
    // @}

    // @name Accesseurs
    // @{
#include "gmv-inline.hh"
    // @}

    // @name Mthodes permettant de crer et plonger des maillages
    // @{
#include "gmv-primitive-mesh.hh"
    // @}

    // @name Mthodes permettant la manipulation de polylignes
    // @{
#include "gmv-polyline.hh"
    // @}

    // @name Mthodes de cration des objets de base
    // @{
#include "gmv-primitive.hh"
    // @}

    // @name Mthodes basiques pour les oprations gomtriques
    // @{
#include "gmv-geometry.hh"
    // @}

    // @name Mthodes de calcul d'une droite ou d'un plan de rgression
    // @{
#include "gmv-regression.hh"
    // @}

    // @name Mthodes de placage
    // @{
#include "gmv-plating.hh"
    // @}

    // @name Coutures avec placages
    // @{
#include "gmv-geo-sew.hh"
    // @}

    // @name Mthode d'insertion
    // @{
#include "gmv-insertion.hh"
    // @}

    // @name Mthode de suppression des artes de longueur nulle
    // @{
#include "gmv-null-length-edge.hh"
    // @}

    // @name Matrialisations
    // @{
#include "gmv-materialization.hh"
    // @}

    // @name Chargement et sauvegarde
    // @{
#include "gmv-save.hh"
    // @}

    // Calcul du dual d'une carte, en dimension 2 ou 3.
    // @{
#include "gmv-dual.hh"
    // @}

    // @name Mthodes de mise  jour des plongements clats
    // @{
#include "gmv-burstview.hh"
    // @}

    // @name Bouchage
    // @{
#include "gmv-stop-up.hh"
    // @}

    // @name Triangulations et quadrangulations barycentriques
    // @{
#include "gmv-triangulation.hh"
#include "gmv-quadrangulation.hh"
    // @}

    // @name Mthodes permettant l'allocation et la libration d'un paramtre de
    //   pondration pour les dformations
    // @{
#include "gmv-direct-info.hh"
    // @}

    // @name Mthodes permettant de dformer les objets
    // @{
#include "gmv-deformation.hh"
    // @}

    // @name paississement
    // @{
#include "gmv-thickening.hh"
    // @}

    // @name Extrusions
    // @{
#include "gmv-extrusion.hh"
    // @}

    // @name Fusion de cellules alignes
    // @{
#include "gmv-merge.hh"
    // @}

    // Diffrents exports dans des formats plus ou moins exotiques
    // @{
#include "gmv-export-pov.hh"
    // @}

  private:
    // @name Champs privs
    // @{

    /// Coefficients d'clatement, pour le calcul des plongements clats
    TCoordinate FBurstCoef[4];

    // @}
  };

} // namespace GMap3d
//******************************************************************************
#include INCLUDE_INLINE("gmv-inline.icc")
//******************************************************************************
#endif // G_MAP_VERTEX_HH
//******************************************************************************
