/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
#include "g-map-vertex.hh"
using namespace GMap3d;
//******************************************************************************
#define DUP(D) (getDirectInfoAsDart(D, directInfoIndex))
//******************************************************************************
int CGMapVertex::dualOfMarkedDarts(int ADim, int AMarkNumber)
{
  assert((2 <= ADim) && (ADim <= 3));

  int directInfoIndex = getNewDirectInfo();
  int treated         = getNewMark();

  // Duplication des brins:
  duplicateMarkedDarts(AMarkNumber, directInfoIndex, false,
		       ADim>=0, ADim>=1, ADim>=2, ADim>=3);

  // Mise  jour des coutures (algo. local):
  int nbDuplicated = 0;
  int middle = (ADim-1)/2;

  CDynamicCoverageAll it(this);

  for (; it.cont(); ++it)
    if (!isMarked(*it, treated))
      {
	setMark(*it, treated);

	if (isMarked(*it, AMarkNumber))
	  {
	    CDart * dup = DUP(*it);

	    ++nbDuplicated;

	    setDirectInfo(dup, directInfoIndex, *it);

	    for (int dim = 0; dim <= middle; ++dim)
	      {
		// Permutation:
		CDart * tmp = alpha(dup, dim);
		dup->setAlpha(alpha(dup, ADim-dim), dim);
		dup->setAlpha(tmp, ADim-dim);
	      }
	  }
      }

  negateMaskMark(treated);

  // Maintenant on affecte le plongement sommet des nouveaux brins:
  for (it.reinit(); it.cont(); ++it)
    if (!isMarked(*it, treated))
      {
	if (isMarked(*it, AMarkNumber))
	  {
	    //  optimiser car le barycentre peut tre calcul plusieurs fois...
	    setVertex(DUP(*it), barycenter(*it, ORBIT_CELL[ADim]));

	    for (CDynamicCoverageVertex vertex(this, DUP(*it));
		 vertex.cont(); ++vertex)
	      setMark(DUP(*vertex), treated);
	  }
	else
	  setMark(*it, treated);
      }

  negateMaskMark(treated);

  // Librations:
  freeDirectInfo(directInfoIndex);
  freeMark       (treated);

  return nbDuplicated;
}
//******************************************************************************
#undef DUP
//******************************************************************************
