/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-vertex.hh".
//******************************************************************************

public:

/**
 * Plaque la cellule de dimension ADim incidente  ADart1 sur la cellule de
 * dimension ADim incidente  ADart2 (si cela est possible) avant de les
 * coudre entre elles.
 *
 * @param ADart1 Un brin de la carte
 * @param ADart2 Un brin de la carte
 * @param ADim La dimension dans laquelle doit se faire la couture
 * @param ARotateCells Un boolen pour modifier ou non l'orientation
 * @param AScaleCells Un boolen pour modifier ou non la taille
 * @param ATranslateCells Un boolen pour modifier ou non la position
 *
 * @precondition 0 <= ADim <= 3
 * @precondition canSew(ADart1, ADart2, ADim)
 */
void geoSew(CDart* ADart1, CDart* ADart2, int ADim,
	    bool ARotateCells = true, bool AScaleCells = true,
	    bool ATranslateCells = true);

/**
 * Appelle la mthode 'geoSew' aprs avoir dtermin la dimension dans
 * laquelle doit se faire la couture  l'aide de la mthode 'getSewDimension'.
 *
 * @param ADart1 Un brin de la carte
 * @param ADart2 Un brin de la carte
 * @param ARotateCells Un boolen pour modifier ou non l'orientation
 * @param AScaleCells Un boolen pour modifier ou non la taille
 * @param ATranslateCells Un boolen pour modifier ou non la position
 * @return Un boolen indiquant si la couture alpha pu se faire
 */
bool intuitiveGeoSew(CDart* ADart1, CDart* ADart2,
		     bool ARotateCells = true, bool AScaleCells = true,
		     bool ATranslateCells = true);

/**
 * Plaque le bord dimension ADim incident  ADart1 sur le bord de dimension
 * ADim incident  ADart2 (si cela est possible) avant de les coudre entre eux.
 *
 * @param ADart1 Un brin de la carte
 * @param ADart2 Un brin de la carte
 * @param ADim La dimension dans laquelle doit se faire la couture
 * @param ARotateCells Un boolen pour modifier ou non l'orientation
 * @param AScaleCells Un boolen pour modifier ou non la taille
 * @param ATranslateCells Un boolen pour modifier ou non la position
 *
 * @precondition 0 <= ADim <= 3
 * @precondition canSew(ADart1, ADart2, ADim)
 */
void geoBorderSew(CDart* ADart1, CDart* ADart2, int ADim,
		  bool ARotateCells = true, bool AScaleCells = true,
		  bool ATranslateCells = true);

/**
 * Appelle la mthode 'geoSewBorders' aprs avoir dtermin la dimension dans
 * laquelle doit se faire la couture  l'aide de la mthode
 * 'getBorderDimension'.
 *
 * @param ADart1 Un brin de la carte
 * @param ADart2 Un brin de la carte
 * @param ARotateCells Un boolen pour modifier ou non l'orientation
 * @param AScaleCells Un boolen pour modifier ou non la taille
 * @param ATranslateCells Un boolen pour modifier ou non la position
 * @return Un boolen indiquant si la couture alpha pu se faire
 */
bool intuitiveGeoBorderSew(CDart* ADart1, CDart* ADart2,
			   bool ARotateCells = true, bool AScaleCells = true,
			   bool ATranslateCells = true);

//******************************************************************************
