/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-vertex.hh".
//******************************************************************************

public:

/**
 * Insre un brin isol dans la carte dont les coordonnes sont donnes par
 * le barycentre des sommets incidents au brins de la carte marqus avec la
 * marque AMarkNumber.
 *
 * @param AMarkNumber
 * @return Le brin ayant pour plongement le barycentre
 */
CDart* materializeBarycenter(int AMarkNumber);

/**
 * Matrialise l'axe dtermin par les deux brins donns (qui ne doivent pas
 * tre sur la mme orbite sommet) en crant une nouvelle arte.
 * L'arte cre est constitue de deux brins 0-cousus.
 *
 * @param ADart1 Un brin de la carte
 * @param ADart2 Un brin de la carte
 * @return Le brin de l'axe cr correspondant  ADart2
 */
CDart* materializeAxe(CDart* ADart1, CDart* ADart2);

/**
 * Matrialise le plan dtermin par les trois sommets incidentes aux brins
 * donns. L'objet cr est un triangle.
 *
 * @param ADart1 Un brin de la carte
 * @param ADart2 Un brin de la carte
 * @param ADart3 Un brin de la carte
 * @return Un brin du plan cr
 */
CDart* materializePlane(CDart* ADart1, CDart* ADart2, CDart* ADart3);

/**
 * Matrialise le vecteur normal de la face incidente au brin donn.
 *
 * @param ADart un brin de la carte
 * @return Le brin correspondant  la pointe du vecteur normal lorsque ce
 * vecteur "est pos" sur le barycentre de la face incidente  ADart.
 */
CDart* materializeNormalVector(CDart* ADart);

/**
 * Matrialise le plan normal du vecteur donn.
 *
 * @param ADart1 un brin de la carte
 * @param ADart2 un brin de la carte
 * @return Un brin appartenant au plan cr (un triangle)
 */
CDart* materializeNormalPlane(CDart* ADart1, CDart* ADart2);

/**
 * Matrialise le rfrentiel (OX,OY,OZ) avec 3 faces perpendiculaires
 * 2-cousues. Chaque face est ouverte et n'est compose que de deux artes.
 *
 * @return Un brin du rfrentiel
 */
CDart* materializeReferential();

//******************************************************************************
