/*
 * lib-gmapkernel : Un noyau de 3-G-cartes et des oprations.
 * Copyright (C) Moka Team, moka@sic.univ-poitiers.fr
 *               http://www.sic.univ-poitiers.fr/moka/
 *
 * This file is part of lib-gmapkernel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//******************************************************************************
// Ce fichier fait partie de "g-map-vertex.hh".
//******************************************************************************

public:

/**
 * Numrote les n brins de la carte de 0  n-1 en utilisant un champ
 * directInfo.
 *
 * @param ADirectInfoIndex Un indice indiquant o placer les numros des brins
 */
void giveNumberToDarts(int ADirectInfoIndex);

/**
 * Dtermine le format du fichier dont le nom est pass en paramtre.
 *
 * @param AFilename Un nom de fichier
 * @return Le format du fichier
 */
static TSaveFormat getFileFormat(const char* AFilename);

/**
 * Retourne la chane de caractre place en dbut d'un fichier de sauvegarde
 * et permettant la reconnaissance du format utilis lors de la sauvegarde.
 *
 * @param AFormat Un type de format
 * @return Une chane de caractres
 */
static const char* getFileHeader(TSaveFormat AFormat);

/**
 * Sauvegarde le contenu de la carte dans un fichier.
 *
 * @param AFilename Un nom de fichier
 * @param AFormat Un type de format de sauvegarde
 * @return 'vrai' si la sauvegarde s'est bien droule
 */
bool save(const char* AFilename, TSaveFormat AFormat);

/**
 * Sauvegarde le contenu de la carte dans un flot.
 *
 * @param AStream Un flot dans lequel crire le contenu de la carte
 * @param AFormat Un type de format de sauvegarde
 * @return 'vrai' si la sauvegarde s'est bien droule
 */
virtual bool save(std::ostream & AStream, TSaveFormat AFormat);

/**
 * Charge la carte  partir d'un fichier.
 * Les brins qui composent la carte avant le chargement ne sont pas dtruits.
 *
 * @param AFilename Un nom de fichier
 * @param AFormat Un type de format de fichier
 * @return 'vrai' si le chargement s'est bien droul
 */
bool load(const char* AFilename, TSaveFormat AFormat);

/**
 * Charge la carte  partir d'un flot.
 * Les brins qui composent la carte avant le chargement ne sont pas dtruits.
 *
 * @param AStream Un flot dans lequel lire le contenu de la carte  charger
 * @param AFormat Un type de format de fichier
 * @return 'vrai' si le chargement s'est bien droul
 */
virtual bool load(std::istream & AStream, TSaveFormat AFormat);

//******************************************************************************
