(** Copying graphs *)
module Copy :
sig
  
  (** Persistent implementation *)
  module P :
    functor (Gf : Sig.G) ->
    functor (Gt : Sig.P) ->
    functor
      (M : sig
	 (** [vertex g' v] returns the modified new graph and its corresponding
	   vertex in this graph.
	 *)
         val vertex : Gt.t -> Gf.V.t -> Gt.V.t * Gt.t

         (** [edge g' v1' v2'] returns the modified new graph given the images of
	   two vertices that had an edge in the original graph *)
         val edge : Gt.t -> Gt.V.t -> Gt.V.t -> Gt.t
       end) ->
  sig
    (** [copy g] Copy the graph [g] using [M.vertex] and [M.edge] *)
    val copy : Gf.t -> Gt.t
  end
  
  (** Imperative implementation *)
  module I :
    functor (Gf : Sig.G) ->
    functor (Gt : Sig.I) ->
    functor
      (M : sig
	 (** [vertex g' v] modifies the new graph and return the vertex 
	   corresponding to [v] in this graph.
	 *)
         val vertex : Gt.t -> Gf.V.t -> Gt.V.t

         (** [edge g' v1' v2'] modifies the new graph given the images of
	   two vertices that had an edge in the original graph *)
         val edge : Gt.t -> Gt.V.t -> Gt.V.t -> unit
       end) ->
  sig
    (** [copy g] Copy the graph [g] using [M.vertex] and [M.edge] *)
    val copy : Gf.t -> Gt.t
  end

end
