  (** Compute the neighborhood of a vertex or some vertices in a non-oriented graph
  *)
module Graph(G : Sig.G) =
struct
  
  module VertexSet = Set.Make(G.V)
    
  let flip f x y = f y x
		     
  let of_list l =
    List.fold_left (flip VertexSet.add) 
      VertexSet.empty l       

  let of_vertex g x =
    G.fold_succ 
      (fun x' s -> 
	 VertexSet.add x' s)
      g x VertexSet.empty
      
  let of_vertices g s =
    let nh =
      VertexSet.fold 
	(fun x xs' ->
	   VertexSet.union xs' (of_vertex g x))
	s VertexSet.empty
    in
      VertexSet.diff nh s
	
  let of_vertices_list g xs =
    of_vertices g (of_list xs)
end
